/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qapplication.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qtextedit.h>
#include <qlayout.h>
#include <qheader.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qptrlist.h>
#include <qstringlist.h>
#include <qlistview.h>
#include <qfile.h>
#include <qtextstream.h>

#include <kglobal.h>
#include <kiconloader.h>
#include <kcombobox.h>
#include <kglobal.h>
#include <kcharsets.h>
#include <kpassdlg.h>
#include <klineedit.h>
#include <klistview.h>
#include <ktabwidget.h>
#include <ktoolbarbutton.h>
#include <kmessagebox.h>
#include <kfiledialog.h>

#include "kuftpbookmarks.h"
#include "../kuftpsettings.h"

extern QPtrList<KuFtpBookmarksItem> pl_bmi;
extern KuFtpSettings kuftpGlobalSettings;

KuFtpBookmarks::KuFtpBookmarks(QWidget *parent):QDialog(parent)
{
	setCaption(i18n("Bookmarks -- KuFTP"));
	setupBookmarksTabWidget();
	bookmarksListView=new KListView(this);
	bookmarksListView->addColumn(i18n("Bookmarks"));
	bookmarksListView->header()->setClickEnabled( false );
	bookmarksListView->setRootIsDecorated( true );
	bookmarksListView->setSelectionMode( QListView::Single );
	connect(bookmarksListView,SIGNAL(selectionChanged(QListViewItem *)),this,
			SLOT(SLOT_selectionItemChanged(QListViewItem *)));
	updateBookmarksListView();

	newGroupButton=new QPushButton(i18n("New Group"),this);
	connect(newGroupButton,SIGNAL(clicked()),this,SLOT(SLOT_newGroup()));
	newSiteButton=new QPushButton(i18n("New Site"),this);
	connect(newSiteButton,SIGNAL(clicked()),this,SLOT(SLOT_newSite()));
	removeButton=new QPushButton(i18n("Remove"),this);
	connect(removeButton,SIGNAL(clicked()),this,SLOT(SLOT_remove()));
	applyButton=new QPushButton(i18n("Apply"),this);
	connect(applyButton,SIGNAL(clicked()),this,SLOT(SLOT_apply()));
	closeButton=new QPushButton(i18n("Close"),this);
	connect(closeButton,SIGNAL(clicked()),this,SLOT(accept()));
	closeButton->setDefault(true);

	topHLayout=new QHBoxLayout();
	topHLayout->addWidget(bookmarksListView);
	topHLayout->addWidget(tabWidget);
	bottomHLayout=new QHBoxLayout();
	bottomHLayout->addWidget(newGroupButton);
	bottomHLayout->addWidget(newSiteButton);
	bottomHLayout->addStretch(1);
	bottomHLayout->addWidget(removeButton);
	bottomHLayout->addStretch(1);
	bottomHLayout->addWidget(applyButton);
	bottomHLayout->addWidget(closeButton);
	bottomHLayout->setSpacing(5);
	vLayout=new QVBoxLayout(this);
	vLayout->addLayout(topHLayout);
	vLayout->addLayout(bottomHLayout);
	vLayout->setMargin(5);
	vLayout->setSpacing(5);
}

KuFtpBookmarks::~KuFtpBookmarks()
{
	writeBookmarks();
}

void KuFtpBookmarks::setupBookmarksTabWidget()
{
	tabWidget=new KTabWidget(this);

	generalTab=new QWidget(tabWidget);
        siteNameLabel=new QLabel(i18n("&Sitename: "),generalTab);
        siteNameLineEdit=new KLineEdit(generalTab);
        siteNameLabel->setBuddy(siteNameLineEdit);
        hostLabel=new QLabel(i18n("&Host/IP: "),generalTab);
        hostLineEdit=new KLineEdit(generalTab);
        hostLabel->setBuddy(hostLineEdit);
        portLabel=new QLabel(i18n("Port: "),generalTab);
        portSpinBox=new QSpinBox(generalTab);
	portSpinBox->setMinValue(1);
	portSpinBox->setMaxValue(65535);
        portSpinBox->setValue(21);
        usernameLabel=new QLabel(i18n("&Username: "),generalTab);
        usernameLineEdit=new KLineEdit(generalTab);
        usernameLabel->setBuddy(usernameLineEdit);
	anonymousLabel=new QLabel(i18n("Anonymous"),generalTab);
        anonymousCheckBox=new QCheckBox(generalTab);
        connect(anonymousCheckBox,SIGNAL(clicked()),this,SLOT(SLOT_anonymousCheckBoxClicked()));
	passwordLabel=new QLabel(i18n("&Password: "),generalTab);
        passwordEdit=new KPasswordEdit(KPasswordEdit::OneStar,generalTab,0);
        passwordLabel->setBuddy(passwordEdit);
	remotePathLabel=new QLabel(i18n("&RemotePath: "),generalTab);
        remotePathLineEdit=new KLineEdit(generalTab);
        remotePathLabel->setBuddy(remotePathLineEdit);
	localPathLabel=new QLabel(i18n("&LocalPath"),generalTab);
	localPathLineEdit=new KLineEdit(generalTab);
        localPathLabel->setBuddy(localPathLineEdit);
	openFileButton=new KToolBarButton("fileopen",0,generalTab);
	connect(openFileButton,SIGNAL(clicked()),this,SLOT(SLOT_localSaveDir()));
        proxyLabel=new QLabel(i18n("Proxy: "),generalTab);
        proxyComboBox=new KComboBox(generalTab);
        proxyComboBox->insertItem(i18n("None"));
        proxyComboBox->insertItem(i18n("Default"));
	for(proxy_t *t=kuftpGlobalSettings.pl_proxy.first();t;t=kuftpGlobalSettings.pl_proxy.next())
        {
                proxyComboBox->insertItem(t->name);
        }

	commentLabel=new QLabel(i18n("Comment: "),generalTab);
	commentEdit=new QTextEdit(generalTab);

        generalLayout=new QGridLayout(generalTab);
        generalLayout->addMultiCellWidget(siteNameLabel,0,0,0,0);
        generalLayout->addMultiCellWidget(siteNameLineEdit,0,0,1,3);
        generalLayout->addWidget(hostLabel,1,0);
        generalLayout->addWidget(hostLineEdit,1,1);
        generalLayout->addWidget(portLabel,1,2);
        generalLayout->addWidget(portSpinBox,1,3);
        generalLayout->addWidget(usernameLabel,2,0);
        generalLayout->addWidget(usernameLineEdit,2,1);
	generalLayout->addWidget(anonymousLabel,2,2);
        generalLayout->addWidget(anonymousCheckBox,2,3);
        generalLayout->addMultiCellWidget(passwordLabel,3,3,0,0);
        generalLayout->addMultiCellWidget(passwordEdit,3,3,1,3);
        generalLayout->addMultiCellWidget(remotePathLabel,4,4,0,0);
        generalLayout->addMultiCellWidget(remotePathLineEdit,4,4,1,3);
        generalLayout->addMultiCellWidget(localPathLabel,5,5,0,0);
        generalLayout->addMultiCellWidget(localPathLineEdit,5,5,1,2);
        generalLayout->addMultiCellWidget(openFileButton,5,5,3,3);
        generalLayout->addMultiCellWidget(proxyLabel,6,6,0,0);
        generalLayout->addMultiCellWidget(proxyComboBox,6,6,1,3);
	generalLayout->addMultiCellWidget(commentLabel,7,7,0,0);
	generalLayout->addMultiCellWidget(commentEdit,7,7,1,3);
	generalLayout->setMargin(5);
	generalLayout->setSpacing(5);

	advancedTab=new QWidget(tabWidget);
	modeCheckBox=new QCheckBox(i18n("Use PASV Mode"),advancedTab);
	correctPasvCheckBox=new QCheckBox(i18n("Correct PASV Replies"),advancedTab);
	encryptButtonGroup=new QButtonGroup(1,Qt::Horizontal,"Encrypt Session With SSL",advancedTab);
	//encryptButtonGroup->setExclusive(true);
	noEncryptRadioButton=new QRadioButton(i18n("Do not Encrypt Session"),encryptButtonGroup);
	encryptControlRadioButton=new QRadioButton(i18n("Encrypt Control Channel only"),encryptButtonGroup);
	encryptControlAndDirRadioButton=new QRadioButton(i18n("Encrypt Control Channel and Directory List"),encryptButtonGroup);
	encryptAllRadioButton=new QRadioButton(i18n("Encrypt Everyting"),encryptButtonGroup);
	noEncryptRadioButton->setChecked(true);
	codecGroupBox=new QGroupBox(1,Qt::Vertical,i18n("Server Encoding"),advancedTab);
	codecGroupBox->layout()->setMargin(5);
	codecGroupBox->layout()->setSpacing(10);
	codecComboBox=new KComboBox(codecGroupBox);
	codecComboBox->insertStringList(KGlobal::charsets()->descriptiveEncodingNames());
	codecComboBox->setCurrentText(kuftpGlobalSettings.getDescriptiveEncoding());
	spacer=new QSpacerItem(20,20,QSizePolicy::Minimum,QSizePolicy::Expanding);
	advancedLayout=new QGridLayout(advancedTab,5,1,5);
	advancedLayout->addWidget(modeCheckBox,0,0);
	advancedLayout->addWidget(correctPasvCheckBox,1,0);
	advancedLayout->addWidget(encryptButtonGroup,2,0);
	advancedLayout->addWidget(codecGroupBox,3,0);
	advancedLayout->addItem(spacer);

        tabWidget->addTab(generalTab,i18n("General"));
        tabWidget->addTab(advancedTab,i18n("Advanced"));
}
void KuFtpBookmarks::updateBookmarksListView()
{
	while(QListViewItem *tmp=bookmarksListView->firstChild()) delete tmp;

	myBookmarksItem=new KListViewItem(bookmarksListView,"My Bookmarks");
	myBookmarksItem->setPixmap(0, KGlobal::iconLoader()->loadIcon("bookmark_folder",KIcon::Small));
	myBookmarksItem->setExpandable(true);
	myBookmarksItem->setOpen(true);
	bookmarksListView->setCurrentItem(myBookmarksItem);
	bookmarksListView->setSelected(myBookmarksItem,true);

        if(pl_bmi.isEmpty())
		return;

	KuFtpBookmarksItem *topItem;
	for(topItem=pl_bmi.first();topItem;topItem=pl_bmi.next())
	{
		if(topItem->getIsGroup())
		{
			KListViewItem *groupItem=new KListViewItem(myBookmarksItem,topItem->getSiteName());
			groupItem->setPixmap(0, KGlobal::iconLoader()->loadIcon("folder",KIcon::Small));
			groupItem->setExpandable(true);
			topItem->p_lvi=groupItem;
			KuFtpBookmarksItem *subItem;
			for(subItem=topItem->pl_child.first();subItem;subItem=topItem->pl_child.next())
			{
				KListViewItem *siteItem=new KListViewItem(groupItem,subItem->getSiteName());
				siteItem->setPixmap(0,KGlobal::iconLoader()->loadIcon("bookmark",KIcon::Small));
				subItem->p_lvi=siteItem;
			}
		}
		else
		{
			KListViewItem *siteItem=new KListViewItem(myBookmarksItem,topItem->getSiteName());
			siteItem->setPixmap(0, KGlobal::iconLoader()->loadIcon("bookmark",KIcon::Small));
			topItem->p_lvi=siteItem;
		}
	}
	bookmarksListView->setCurrentItem(myBookmarksItem);
	bookmarksListView->setSelected(myBookmarksItem,true);
}
int KuFtpBookmarks::getEncryptMethod()
{
	if(noEncryptRadioButton->isChecked())
		return 0;
	else if(encryptControlRadioButton->isChecked())
		return 1;
	else if(encryptControlAndDirRadioButton->isChecked())
		return 2;
	else
		return 3;
}
void KuFtpBookmarks::setEncryptButtonGroupChecked(int encryptMethod)
{
	if(encryptMethod==0)
		noEncryptRadioButton->setChecked(true);
	else if(encryptMethod==1)
		encryptControlRadioButton->setChecked(true);
	else if(encryptMethod==2)
		encryptControlAndDirRadioButton->setChecked(true);
	else
		encryptAllRadioButton->setChecked(true);
}

void KuFtpBookmarks::setBookmarksTabEnabled(bool enable)
{
/*	if(enable)
	{
		sitenameLineEdit->setEnabled(true);
		sitehostipLineEdit->setEnabled(true);
		siteportSpinBox->setEnabled(true);
		usernameLineEdit->setEnabled(true);
		anonymousCheckBox->setEnabled(true);
		passwordEdit->setEnabled(true);
		remotepathLineEdit->setEnabled(true);
		localpathLineEdit->setEnabled(true);
		openfileButton->setEnabled(true);
		proxyComboBox->setEnabled(true);
		commentEdit->setEnabled(true);
		advancedTab->setEnabled(true);
	}
	else
	{
		anonymousCheckBox->setChecked(false);
		remotepathLineEdit->clear();
		localpathLineEdit->clear();
		sitehostipLineEdit->setEnabled(false);
		siteportSpinBox->setEnabled(false);
		usernameLineEdit->setEnabled(false);
		anonymousCheckBox->setEnabled(false);
		passwordEdit->setEnabled(false);
		remotepathLineEdit->setEnabled(false);
		localpathLineEdit->setEnabled(false);
		openfileButton->setEnabled(false);
		proxyComboBox->setEnabled(false);
		advancedTab->setEnabled(false);
	}*/
	if(enable)
	{
		siteNameLineEdit->setEnabled(true);
		commentEdit->setEnabled(true);
	}
	else
	{
		anonymousCheckBox->setChecked(false);
		remotePathLineEdit->clear();
		localPathLineEdit->clear();
	}
		hostLineEdit->setEnabled(enable);
		portSpinBox->setEnabled(enable);
		usernameLineEdit->setEnabled(enable);
		anonymousCheckBox->setEnabled(enable);
		passwordEdit->setEnabled(enable);
		remotePathLineEdit->setEnabled(enable);
		localPathLineEdit->setEnabled(enable);
		openFileButton->setEnabled(enable);
		proxyComboBox->setEnabled(enable);
		advancedTab->setEnabled(enable);
}


