/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KUFTPBOOKMARKS_H
#define KUFTPBOOKMARKS_H

#include <qdialog.h>

#include <klistview.h>
#include <klocale.h>

#include "../kuftpsiteinfo.h"

class QWidget;
class QLabel;
class QSpinBox;
class QCheckBox;
class QGridLayout;
class QTextEdit;
class QPushButton;
class QSpacerItem;
class QHBoxLayout;
class QGroupBox;
class QButtonGroup;
class QRadioButton;
class QListViewItem;
class QVBoxLayout;
template<class type>
class QPtrList;

class KComboBox;
class KPasswordEdit;
class KLineEdit;
class KTabWidget;
class KListView;
class KListViewItem;
class KListView;
class KToolBarButton;

class KuFtpBookmarksItem;

struct bookmarksAct_t
{
	int i;
	KuFtpBookmarksItem *ptr;
};
class KuFtpBookmarksItem
{
	public:
		KuFtpBookmarksItem(QString name,QString comment,QString host=NULL,unsigned int port=21, QString username=NULL,QString password=NULL,QString remotePath=NULL,QString localPath=NULL, int proxyIndex=0,int encryptMethod=0,bool pasv=true,bool correctPasv=false, QString encoding="Unicode ( utf8 )");
		void setBookmarksItem(QString name,QString comment,QString host=NULL,unsigned int port=21, QString username=NULL,QString password=NULL,QString remotePath=NULL,QString localPath=NULL, int proxyIndex=0,int encryptMethod=0,bool pasv=true,bool correctPasv=false, QString encoding="Unicode ( utf8 )");
		QString getSiteName(){return siteInfo->getSiteName();}
		QString getComment(){return siteInfo->getComment();}
		QString getHost(){return siteInfo->getHost();}
		unsigned int getPort(){return siteInfo->getPort();}
		QString getUsername(){return siteInfo->getUsername();}
		QString getPassword(){return siteInfo->getPassword();}
		QString getRemotePath(){return siteInfo->getRemotePath();}
		QString getLocalPath(){return siteInfo->getLocalPath();}
		int	getProxyIndex(){return siteInfo->getProxyIndex();}
		int getEncryptMethod(){return siteInfo->getEncryptMethod();}
		bool getIsPasv(){return siteInfo->getIsPasv();}
		bool getIsCorrectPasv(){return siteInfo->getIsCorrectPasv();}
		QString getEncoding(){return siteInfo->getEncoding();}
		bool getIsGroup(){return group;}
		void setGroup(bool g){group=g;}
		void setEncoding(QString s){siteInfo->setEncoding(s);};
		QPtrList<KuFtpBookmarksItem> pl_child;
		KListViewItem *p_lvi;
	private:
		KuFtpSiteInfo *siteInfo;
		bool group;
};

class KuFtpBookmarks:public QDialog
{
	Q_OBJECT
	public:
		KuFtpBookmarks(QWidget *parent=0);
		~KuFtpBookmarks();
		static void readBookmarks();
		static void writeBookmarks();
		static QPtrList<KuFtpBookmarksItem> pl_bmi;
	private:
		bool newSiteSaved;
		
		KListView *bookmarksListView;
		KListViewItem *myBookmarksItem;

		KTabWidget *tabWidget;
		QHBoxLayout *topHLayout;
		QHBoxLayout *bottomHLayout;
		QVBoxLayout *vLayout;

	        QWidget *generalTab;
                QGridLayout *generalLayout;
                QLabel *siteNameLabel;
                QLabel *hostLabel;
                QLabel *portLabel;
                QLabel *usernameLabel;
                QLabel *anonymousLabel;
                QLabel *passwordLabel;
                QLabel *remotePathLabel;
                QLabel *localPathLabel;
		QLabel *proxyLabel;
                KLineEdit *siteNameLineEdit;
                KLineEdit *hostLineEdit;
                QSpinBox  *portSpinBox;
		KLineEdit *usernameLineEdit;
                QCheckBox *anonymousCheckBox;
                KPasswordEdit *passwordEdit;
                KLineEdit *remotePathLineEdit;
                KLineEdit *localPathLineEdit;
		KToolBarButton *openFileButton;
                KComboBox *proxyComboBox;
		QLabel *commentLabel;
		QTextEdit *commentEdit;

		QWidget *advancedTab;
                QGridLayout *advancedLayout;
                QCheckBox *modeCheckBox;
                QCheckBox *correctPasvCheckBox;
		QButtonGroup *encryptButtonGroup;
		QRadioButton *noEncryptRadioButton;
		QRadioButton *encryptControlRadioButton;
		QRadioButton *encryptControlAndDirRadioButton;
		QRadioButton *encryptAllRadioButton;
		QGroupBox *codecGroupBox;
                KComboBox *codecComboBox;
		QSpacerItem *spacer;

		QPushButton *newSiteButton;
		QPushButton *newGroupButton;
		QPushButton *removeButton;
		QPushButton *applyButton;
		QPushButton *closeButton;
	private:
		void setBookmarksTabEnabled(bool enable);
		int getEncryptMethod();
		void setEncryptButtonGroupChecked(int);
		void updateBookmarksListView();
		void setupBookmarksTabWidget();
        private slots:
                void SLOT_anonymousCheckBoxClicked();
		void SLOT_newSite();
		void SLOT_newGroup();
		void SLOT_localSaveDir();
		void SLOT_remove();
		void SLOT_apply();
		void SLOT_connect(){}
		void SLOT_selectionItemChanged(QListViewItem *item);
};

#endif


