/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>

#include <qdom.h>
#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>

#include "kuftpbookmarks.h"

QPtrList<KuFtpBookmarksItem> KuFtpBookmarks::pl_bmi;
void KuFtpBookmarks::readBookmarks()
{
	QDomDocument doc( "KuFtpBookmarks" );
	QFile file(QDir::homeDirPath().append("/.kuftp/bookmarks.xml"));
	if( !file.open( IO_ReadOnly ) ) return ;
	if( !doc.setContent( &file ) )
	{
		file.close();
		return ;
	}
	file.close();

	pl_bmi.setAutoDelete(true);
	QString siteName;
	QString host;
	unsigned int port=21;
	QString username;
	QString password;
	QString remotePath;
	QString localPath;
	int proxyIndex=0;
	QString comment;
	bool encrypt=false;
	bool pasv=true;
	bool correctPasv=false;
	QString encoding;

	QDomElement root = doc.documentElement();
	if( root.tagName() != "Bookmarks" ) return ;
	QDomNode topNode = root.firstChild();
	while( !topNode.isNull() )
	{
		QDomElement topElement = topNode.toElement();
		if( !topElement.isNull() )
		{
			if( topElement.tagName() == "group" )
			{
				siteName=topElement.attribute("name","");
				comment=topElement.attribute("comment","");
				KuFtpBookmarksItem *groupItem=new KuFtpBookmarksItem(siteName,comment);
				groupItem->setGroup(true);
				pl_bmi.append(groupItem);

				QDomNode groupNode = topElement.firstChild();
				while( !groupNode.isNull() )
				{
					
					QDomElement groupElement = groupNode.toElement();
					if(!groupElement.isNull())
					{
						if(groupElement.tagName()=="site")
						{
							siteName=groupElement.attribute("name","");
						QDomNode siteNode = groupElement.firstChild();
						while( !siteNode.isNull() )
						{
							QDomElement siteInfoElement=siteNode.toElement();
							if(!siteInfoElement.isNull())
							{
								if(siteInfoElement.tagName() == "host") 
									host=siteInfoElement.text();
								if(siteInfoElement.tagName() == "port")
								{
									port=siteInfoElement.text().toUInt();
									if(port<=0||port>65535)
										port=21;
								}
								if(siteInfoElement.tagName() == "username")
									username=siteInfoElement.text();
								if(siteInfoElement.tagName() == "password")
									password=siteInfoElement.text();
								if(siteInfoElement.tagName() == "remotepath")
									remotePath=siteInfoElement.text();
								if(siteInfoElement.tagName() == "localpath")
									localPath=siteInfoElement.text();
								if(siteInfoElement.tagName()=="proxyindex")
									proxyIndex=siteInfoElement.text().toInt();
								if(siteInfoElement.tagName() == "comment")
									comment=siteInfoElement.text();
								if(siteInfoElement.tagName()=="encryption")
	       								encrypt=(siteInfoElement.text()=="true"?1:0);
								if(siteInfoElement.tagName()=="pasv")
									pasv=(siteInfoElement.text()=="true"?1:0);
								if(siteInfoElement.tagName()=="correctpasv")
									correctPasv=(siteInfoElement.text()=="true"?1:0);
								if(siteInfoElement.tagName()=="encoding")
									encoding=siteInfoElement.text();
							}
							siteNode=siteNode.nextSibling();
						}
						KuFtpBookmarksItem *siteItem=new KuFtpBookmarksItem(siteName,comment,host,
								port,username,password,remotePath,localPath,proxyIndex,
								encrypt,pasv,correctPasv,encoding);
						siteItem->setGroup(false);
						groupItem->pl_child.append(siteItem);
						}
					}
					groupNode=groupNode.nextSibling();
				}
			}
			else if(topElement.tagName()=="site")
			{	
				siteName=topElement.attribute("name","");
				QDomNode siteNode = topElement.firstChild();
				while( !siteNode.isNull() )
				{
					QDomElement siteInfoElement=siteNode.toElement();
					if(!siteInfoElement.isNull())
					{
					if(siteInfoElement.tagName() == "host") 
						host=siteInfoElement.text();
					if(siteInfoElement.tagName() == "port")
					{
						port=siteInfoElement.text().toUInt();
						if(port<=0||port>65535)
							port=21;
					}
					if(siteInfoElement.tagName() == "username")
						username=siteInfoElement.text();
					if(siteInfoElement.tagName() == "password")
						password=siteInfoElement.text();
					if(siteInfoElement.tagName() == "remotepath")
						remotePath=siteInfoElement.text();
					if(siteInfoElement.tagName() == "localpath")
						localPath=siteInfoElement.text();
					if(siteInfoElement.tagName()=="proxy index")
						proxyIndex=siteInfoElement.text().toInt();
					if(siteInfoElement.tagName() == "comment")
						comment=siteInfoElement.text();
					if(siteInfoElement.tagName()=="encryption")
						encrypt=(siteInfoElement.text()=="true"?1:0);
					if(siteInfoElement.tagName()=="pasv")
						pasv=(siteInfoElement.text()=="true"?1:0);
					if(siteInfoElement.tagName()=="correctpasv")
						correctPasv=(siteInfoElement.text()=="true"?1:0);
					if(siteInfoElement.tagName()=="encoding")
						encoding=siteInfoElement.text();
					}
					siteNode=siteNode.nextSibling();
				}
				KuFtpBookmarksItem *siteItem=new KuFtpBookmarksItem(siteName,comment,host,port,username,password,remotePath,localPath,proxyIndex,encrypt,pasv,correctPasv,encoding);
				siteItem->setGroup(false);
				pl_bmi.append(siteItem);
			}
		}
		topNode = topNode.nextSibling();
	}
}
void KuFtpBookmarks::writeBookmarks()
{
	QDomDocument doc( "KuFtpBookmarks" );
        QDomElement root = doc.createElement( "Bookmarks" );
        doc.appendChild( root );

	for(KuFtpBookmarksItem *topItem=pl_bmi.first();topItem;topItem=pl_bmi.next())
	{
		if(topItem->getIsGroup())
		{
			QDomElement topElement=doc.createElement("group");
			topElement.setAttribute("name",topItem->getSiteName());
		        topElement.setAttribute("comment",topItem->getComment());
			root.appendChild(topElement);
			for(KuFtpBookmarksItem *subItem=topItem->pl_child.first();subItem;subItem=topItem->pl_child.next())
			{
				QDomElement siteElement=doc.createElement("site");
				siteElement.setAttribute("name",subItem->getSiteName());
				topElement.appendChild(siteElement);
				QDomText siteInfoText;
				QDomElement siteInfoElement;

				siteInfoElement=doc.createElement("host");
				siteElement.appendChild( siteInfoElement );
				siteInfoText = doc.createTextNode(subItem->getHost());
				siteInfoElement.appendChild(siteInfoText);

				siteInfoElement=doc.createElement("port");
				siteElement.appendChild( siteInfoElement);
				char port[10];
				sprintf(port,"%d",subItem->getPort());
				siteInfoText=doc.createTextNode(port);
				siteInfoElement.appendChild(siteInfoText);

				siteInfoElement=doc.createElement("username");
				siteElement.appendChild( siteInfoElement);
				siteInfoText=doc.createTextNode(subItem->getUsername());
				siteInfoElement.appendChild(siteInfoText);

				siteInfoElement=doc.createElement("password");
				siteElement.appendChild( siteInfoElement);
				siteInfoText=doc.createTextNode(subItem->getPassword());
				siteInfoElement.appendChild(siteInfoText);

				siteInfoElement=doc.createElement("remotepath");
				siteElement.appendChild( siteInfoElement);
				siteInfoText=doc.createTextNode(subItem->getRemotePath());
				siteInfoElement.appendChild(siteInfoText);

				siteInfoElement=doc.createElement("localpath");
				siteElement.appendChild( siteInfoElement);
				siteInfoText=doc.createTextNode(subItem->getLocalPath());
				siteInfoElement.appendChild(siteInfoText);

				siteInfoElement=doc.createElement("proxyindex");
				siteElement.appendChild( siteInfoElement);
				char proxyIndex[10];
				sprintf(proxyIndex,"%d",subItem->getProxyIndex());
				siteInfoText=doc.createTextNode(proxyIndex);
				siteInfoElement.appendChild(siteInfoText);

				siteInfoElement=doc.createElement("comment");
				siteElement.appendChild( siteInfoElement);
				siteInfoText=doc.createTextNode(subItem->getComment());
				siteInfoElement.appendChild(siteInfoText);

				siteInfoElement=doc.createElement("encryption");
				siteElement.appendChild( siteInfoElement);
				siteInfoText=doc.createTextNode(QString("%1").arg(subItem->getEncryptMethod()));
				siteInfoElement.appendChild(siteInfoText);
				
				siteInfoElement=doc.createElement("pasv");
				siteElement.appendChild( siteInfoElement);
				if(subItem->getIsPasv())
					siteInfoText=doc.createTextNode("true");
				else
					siteInfoText=doc.createTextNode("false");
				siteInfoElement.appendChild(siteInfoText);
				
				siteInfoElement=doc.createElement("correctpasv");
				siteElement.appendChild( siteInfoElement);
				if(subItem->getIsCorrectPasv())
					siteInfoText=doc.createTextNode("true");
				else
					siteInfoText=doc.createTextNode("false");
				siteInfoElement.appendChild(siteInfoText);
				
				siteInfoElement=doc.createElement("encoding");
				siteElement.appendChild( siteInfoElement);
				siteInfoText=doc.createTextNode(subItem->getEncoding());
				siteInfoElement.appendChild(siteInfoText);
			}
		}
		else
		{

			QDomElement siteElement=doc.createElement("site");
			siteElement.setAttribute("name",topItem->getSiteName());
			root.appendChild(siteElement);
			QDomText siteInfoText;
			QDomElement siteInfoElement;

			siteInfoElement=doc.createElement("host");
			siteElement.appendChild( siteInfoElement );
			siteInfoText = doc.createTextNode(topItem->getHost());
			siteInfoElement.appendChild(siteInfoText);

			siteInfoElement=doc.createElement("port");
			siteElement.appendChild( siteInfoElement);
			char port[10];
			sprintf(port,"%d",topItem->getPort());
			siteInfoText=doc.createTextNode(port);
			siteInfoElement.appendChild(siteInfoText);

			siteInfoElement=doc.createElement("username");
			siteElement.appendChild( siteInfoElement);
			siteInfoText=doc.createTextNode(topItem->getUsername());
			siteInfoElement.appendChild(siteInfoText);

			siteInfoElement=doc.createElement("password");
			siteElement.appendChild( siteInfoElement);
			siteInfoText=doc.createTextNode(topItem->getPassword());
			siteInfoElement.appendChild(siteInfoText);

			siteInfoElement=doc.createElement("remotePath");
			siteElement.appendChild( siteInfoElement);
			siteInfoText=doc.createTextNode(topItem->getRemotePath());
			siteInfoElement.appendChild(siteInfoText);

			siteInfoElement=doc.createElement("localPath");
			siteElement.appendChild( siteInfoElement);
			siteInfoText=doc.createTextNode(topItem->getLocalPath());
			siteInfoElement.appendChild(siteInfoText);

			siteInfoElement=doc.createElement("proxyindex");
			siteElement.appendChild( siteInfoElement);
			char proxyindex[10];
			sprintf(proxyindex,"%d",topItem->getProxyIndex());
			siteInfoText=doc.createTextNode(proxyindex);
			siteInfoElement.appendChild(siteInfoText);

			siteInfoElement=doc.createElement("comment");
			siteElement.appendChild( siteInfoElement);
			siteInfoText=doc.createTextNode(topItem->getComment());
			siteInfoElement.appendChild(siteInfoText);
			
			siteInfoElement=doc.createElement("encryption");
			siteElement.appendChild( siteInfoElement);
			siteInfoText=doc.createTextNode(QString("%1").arg(topItem->getEncryptMethod()));
			siteInfoElement.appendChild(siteInfoText);
				
			siteInfoElement=doc.createElement("pasv");
			siteElement.appendChild( siteInfoElement);
			if(topItem->getIsPasv())
				siteInfoText=doc.createTextNode("true");
			else
				siteInfoText=doc.createTextNode("false");
			siteInfoElement.appendChild(siteInfoText);
				
			siteInfoElement=doc.createElement("correctpasv");
			siteElement.appendChild( siteInfoElement);
			if(topItem->getIsCorrectPasv())
				siteInfoText=doc.createTextNode("true");
			else
				siteInfoText=doc.createTextNode("false");
			siteInfoElement.appendChild(siteInfoText);
				
			siteInfoElement=doc.createElement("encoding");
			siteElement.appendChild( siteInfoElement);
			siteInfoText=doc.createTextNode(topItem->getEncoding());
			siteInfoElement.appendChild(siteInfoText);
		}
	}

	QFile file(QDir::homeDirPath().append("/.kuftp/bookmarks.xml"));
        if( !file.open( IO_WriteOnly ) ) 
		return;
        QTextStream stream( &file );
        stream << doc.toString();
        file.close();
}

