/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qapplication.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qtextedit.h>
#include <qlayout.h>
#include <qheader.h>
#include <qptrlist.h>
#include <qlistview.h>
#include <qfile.h>
#include <qtextstream.h>

#include <kglobal.h>
#include <kiconloader.h>
#include <kcombobox.h>
#include <kpassdlg.h>
#include <klineedit.h>
#include <klistview.h>
#include <ktabwidget.h>
#include <kcharsets.h>
#include <ktoolbarbutton.h>
#include <kmessagebox.h>
#include <kfiledialog.h>

#include "../kuftp.h"
#include "kuftpbookmarks.h"
#include "../kuftpsettings.h"

extern QPtrList<KuFtpBookmarksItem> pl_bmi;
extern KuFtpSettings kuftpGlobalSettings;

void KuFtpBookmarks::SLOT_anonymousCheckBoxClicked()
{
        if(anonymousCheckBox->isChecked())
        {
                usernameLineEdit->setText("anonymous");
                passwordEdit->insert(kuftpGlobalSettings.getEmail());
                usernameLineEdit->setEnabled(false);
                passwordEdit->setEnabled(false);
        }
        else
        {
		usernameLineEdit->clear();
		passwordEdit->erase();
                usernameLineEdit->setEnabled(true);
                passwordEdit->setEnabled(true);
        }
}
void KuFtpBookmarks::SLOT_newGroup()
{
	KListViewItem *newGroup=new KListViewItem(myBookmarksItem,i18n("New Group"));
	newGroup->setPixmap(0,KGlobal::iconLoader()->loadIcon("folder",KIcon::Small));
	KuFtpBookmarksItem *newGroupItem=new KuFtpBookmarksItem(i18n("New Group"),0);
	newGroupItem->p_lvi=newGroup;
	newGroupItem->setGroup(true);
	pl_bmi.append(newGroupItem);
	newGroup->setExpandable(true);
	bookmarksListView->setCurrentItem(newGroup);
	setBookmarksTabEnabled(false);
	KuFtp::bookmarksChanged=true;
}
void KuFtpBookmarks::SLOT_newSite()
{	
/*	if( sitehostipLineEdit->text().stripWhiteSpace().isEmpty() ) 
		return KMessageBox::error(this,i18n("You must specify hostname or ip\nin the Host/IP lineedit!",
				"Empty Hostname/IP Error -- KuFtp"));*/
	QListViewItem *curr=bookmarksListView->currentItem();
	KListViewItem *newSite=NULL;
	QListViewItem *parentItem=NULL;
	if(!curr)
		parentItem=myBookmarksItem;
	else if( curr->isExpandable() && curr!=myBookmarksItem )	
		parentItem=curr;
	else if(  curr==myBookmarksItem || curr->parent()==myBookmarksItem )
		parentItem=myBookmarksItem;
	else if(curr->parent()->isExpandable()||curr->parent()!=myBookmarksItem)
		parentItem=curr->parent();

	newSite=new KListViewItem(parentItem,i18n("New Site"));
	newSite->setPixmap(0,KGlobal::iconLoader()->loadIcon("bookmark",KIcon::Small));
	KuFtpBookmarksItem *newSiteItem=new KuFtpBookmarksItem(i18n("New Site"),0);
	newSiteItem->setGroup(false);
	newSiteItem->p_lvi=newSite;
	if(parentItem==myBookmarksItem)
		pl_bmi.append(newSiteItem);
	else
	{
		KuFtpBookmarksItem *topItem=NULL;
		for(topItem=pl_bmi.first();topItem;topItem=pl_bmi.next())
		{
			if(topItem->p_lvi==parentItem)
			{
				topItem->pl_child.append(newSiteItem);
				break;
			}
		}
	}
	setBookmarksTabEnabled(true);
	bookmarksListView->setCurrentItem(newSite);
	bookmarksListView->setSelected(newSite,true);
	hostLineEdit->setText("newsite");
	hostLineEdit->setFocus();
	hostLineEdit->selectAll();
	portSpinBox->setValue(21);
	proxyComboBox->setCurrentItem(0);
	modeCheckBox->setChecked(true);
	correctPasvCheckBox->setChecked(false);
	noEncryptRadioButton->setChecked(true);
	codecComboBox->clear();
	codecComboBox->insertStringList(KGlobal::charsets()->descriptiveEncodingNames());
	codecComboBox->setCurrentText(kuftpGlobalSettings.getDescriptiveEncoding());
	parentItem->setOpen(true);
	KuFtp::bookmarksChanged=true;
}
void KuFtpBookmarks::SLOT_remove()
{
	QListViewItem *currItem=bookmarksListView->currentItem();
	if(!currItem || currItem==myBookmarksItem )	
		return;
/*	else if( currItem->isExpandable() )
		if( !KMessageBox::questionYesNoCancel(this,i18n("You are going to delete a group\nAll items\
				under this group will be deleted!\nDo you really want to delete?",
				"Warning -- KuFtp")))
			return;
			*/
	KuFtpBookmarksItem *topItem=NULL;
	for(topItem=pl_bmi.first();topItem;topItem=pl_bmi.next())
	{
		if(topItem->p_lvi==currItem)
		{
			pl_bmi.remove(topItem);
			if(topItem->getIsGroup())
				topItem->pl_child.clear();
			break;
		}
		if(topItem->getIsGroup())
		{
			for(KuFtpBookmarksItem *subItem=topItem->pl_child.first();subItem;subItem=topItem->pl_child.next())
				if(subItem->p_lvi==currItem)
				{
					topItem->pl_child.remove(subItem);
					break;
				}
		}
	}
	updateBookmarksListView();
	KuFtp::bookmarksChanged=true;
}


void KuFtpBookmarks::SLOT_apply()
{
	QListViewItem *currentItem=bookmarksListView->currentItem();
	if(!currentItem)
		return;
	if(currentItem==myBookmarksItem )
		return;
	KuFtpBookmarksItem *currItem=NULL;
	for(KuFtpBookmarksItem *topItem=pl_bmi.first();topItem;topItem=pl_bmi.next())
	{
		if(topItem->p_lvi==currentItem)
		{
			currItem=topItem;
			break;
		}
		if(topItem->getIsGroup())
			for(KuFtpBookmarksItem *subItem=topItem->pl_child.first();subItem;subItem=topItem->pl_child.next())
			{
				if(subItem->p_lvi==currentItem)
				{
					currItem=subItem;
					break;
				}
			}
	}

	QString siteName=siteNameLineEdit->text();
	QString host=hostLineEdit->text().stripWhiteSpace();
	unsigned int port=portSpinBox->value();
	QString username=usernameLineEdit->text();
	QString password=QString(passwordEdit->password());
	QString remotePath=remotePathLineEdit->text();
	QString localPath=localPathLineEdit->text();
	int proxyIndex=proxyComboBox->currentItem();
	QString comment=commentEdit->text();
	int encryptMethod=getEncryptMethod();
	bool pasv=modeCheckBox->isChecked();
	bool correctPasv=correctPasvCheckBox->isChecked();
	QString encoding=codecComboBox->currentText();
	
/*	if( hostip.isEmpty()&& !currentItem->isExpandable() ) 
		return KMessageBox::error(this,i18n("You must specify hostname or ip\nin the Host/IP lineedit!",
				"Error -- KuFtp"));*/
	if( siteName.stripWhiteSpace().isEmpty() ){
		siteName=host.stripWhiteSpace();
		siteNameLineEdit->setText(siteName);
	}
	currItem->setBookmarksItem(siteName,comment,host,port,username,password,remotePath,localPath,proxyIndex,encryptMethod,pasv,correctPasv,encoding);
	currentItem->setText(0,siteName);
	KuFtp::bookmarksChanged=true;
}


void KuFtpBookmarks::SLOT_localSaveDir()
{
	localPathLineEdit->setText(KFileDialog::getExistingDirectory());
}
void KuFtpBookmarks::SLOT_selectionItemChanged(QListViewItem *currentItem)
{
	if(!currentItem)
		return;
	if(currentItem==myBookmarksItem )
	{
		setBookmarksTabEnabled(false);
		siteNameLineEdit->setText("KuFTP");
		hostLineEdit->setText("KuFTP");
		usernameLineEdit->clear();
		passwordEdit->clear();
		commentEdit->clear();
		siteNameLineEdit->setEnabled(false);
		commentEdit->setEnabled(false);
		return;
	}
	/*if(sitehostipLineEdit->text().stripWhiteSpace().isEmpty())
	{
		bookmarksListView->setSelected(currentItem,true);
		return KMessageBox::error(this,"You must specify hostname or ip\nin the Host/IP lineedit!",
				"Empty Hostname/IP Error -- KuFtp");
	}
	*/
	KuFtpBookmarksItem *currItem=NULL;
	for(KuFtpBookmarksItem *topItem=pl_bmi.first();topItem;topItem=pl_bmi.next())
	{
		if(topItem->p_lvi==currentItem)
		{
			currItem=topItem;
			break;
		}
		if(topItem->getIsGroup())
		for(KuFtpBookmarksItem *subItem=topItem->pl_child.first();subItem;subItem=topItem->pl_child.next())
		{
			if(subItem->p_lvi==currentItem)
			{
				currItem=subItem;
				break;
			}
		}
	}
	siteNameLineEdit->setText(currItem->getSiteName());
	siteNameLineEdit->setFocus();
	hostLineEdit->setText(currItem->getHost());
	portSpinBox->setValue(currItem->getPort());
	usernameLineEdit->setText(currItem->getUsername());
	passwordEdit->erase();
	passwordEdit->insert(currItem->getPassword());
	remotePathLineEdit->setText(currItem->getRemotePath());
	localPathLineEdit->setText(currItem->getLocalPath());
	proxyComboBox->setCurrentItem(currItem->getProxyIndex());
	commentEdit->setText(currItem->getComment());
	modeCheckBox->setChecked(currItem->getIsPasv());
	correctPasvCheckBox->setChecked(currItem->getIsCorrectPasv());
	setEncryptButtonGroupChecked(currItem->getEncryptMethod());
	codecComboBox->clear();
	codecComboBox->insertStringList(KGlobal::charsets()->descriptiveEncodingNames());
	codecComboBox->setCurrentText(currItem->getEncoding());
	if(currentItem->isExpandable())
	{
		setBookmarksTabEnabled(false);
		siteNameLineEdit->setEnabled(true);
		commentEdit->setEnabled(true);
	}
	else
		setBookmarksTabEnabled(true);	
}


