/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qwidget.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qdatetime.h>
#include <qspinbox.h>
#include <qaction.h>
#include <qvaluelist.h>
#include <qptrlist.h>
#include <qstringlist.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qcheckbox.h>
#include <qtoolbutton.h>
#include <qevent.h>
#include <qsignalmapper.h>

#include <ktabwidget.h>
#include <klineedit.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kcombobox.h>
#include <ktextedit.h>
#include <kapplication.h>
#include <kstatusbar.h>
#include <kpopupmenu.h>
#include <ksystemtray.h>
#include <kpassdlg.h>
#include <kmenubar.h>

#include "kuftp.h"
#include "kuftpsettings.h"
#include "kuftpqueue.h"
#include "kuftpconnectdialog.h"
#include "kuftpconfigdialog.h"
#include "kuftpballoon.h"
#include "kuftpstatusinfo.h"

extern void readBookmarks();

bool KuFtp::bookmarksChanged=false;
KuFtpSettings kuftpGlobalSettings;
KSystemTray *KuFtp::systemTray=NULL;

KuFtp::KuFtp():KMainWindow(0,"KuFTP")
{
	kuftpGlobalSettings.readConfig();
	systemTray=new KSystemTray(this);
	systemTray->setPixmap(KSystemTray::loadIcon("kuftp"));
	connect(systemTray,SIGNAL(quitSelected()),this,SLOT(SLOT_quit()));
	statusInfo=NULL;
	if(kuftpGlobalSettings.getIsShowSystray())
	{
		systemTray->show();
		statusInfo=new KuFtpStatusInfo(systemTray,this);
	}
	setKuFtpCaption();
	createAction();
	createMenuBar();
	resize(QApplication::desktop()->size());
	createMain();

//	setMouseTracking(true);
	QDir d(QDir::homeDirPath().append("/.kuftp"));
	if(!d.exists())
	{
		d.mkdir(QDir::homeDirPath().append("/.kuftp"));
	}
	d.setPath(QDir::homeDirPath().append("/.kuftp/saved_queues"));
	if(!d.exists())
	{
		d.mkdir(QDir::homeDirPath().append("/.kuftp/saved_queues"));
	}
}

void KuFtp::setKuFtpCaption()
{
	QDate date=QDate::currentDate();
	QString s;
	if(date.month()==1&&date.day()==1)			//New Year
	{
		s=QString(i18n(" [ Happy New Year ] "));
	}
//	else if(date.month()==2&&date.day()==14)		//Valentine's Day
//	{
//		s=QString(i18n(" [ Happy Valentine's Day ] "));
//	}
//	else if(date.month()==10&&date.day()==1)		//National Day of China
//	{
//		s=QString(i18n(" [ Happy Birthday To China ] "));
//	}
//	else if(date.month()==12&&date.day()==7)		// 
//	{
//		s=QString(i18n(" [ Happy Birthday To JiangJingJing ^_^ ] "));
//	}
	else if(date.month()==12&&date.day()==25)
	{
		s=QString(i18n(" [ Merry Christmas ] "));
	}
	setCaption(QString(i18n("K FTP For You")).append(s));
}

void KuFtp::createAction()
{
	connectAct=new QAction(i18n("&Conncet"),Key_F2,this);
	connectAct->setIconSet(KGlobal::iconLoader()->loadIconSet("connect_no",KIcon::Small));
	connect(connectAct,SIGNAL(activated()),this,SLOT(SLOT_connectDialog()));
	newSessionAct=new QAction(i18n("New &Session"),CTRL+Key_S,this);
	newSessionAct->setIconSet(KGlobal::iconLoader()->loadIconSet("tab_new",KIcon::Small));
	connect(newSessionAct,SIGNAL(activated()),this,SLOT(SLOT_newSession()));
	closeSessionAct=new QAction(i18n("Close Session"),0,this);
	closeSessionAct->setIconSet(KGlobal::iconLoader()->loadIconSet("tab_remove",KIcon::Small));
	connect(closeSessionAct,SIGNAL(activated()),this,SLOT(SLOT_closeSession()));
	minimizedToSystemTrayAct=new QAction(i18n("Minimized To SystemTray"),Key_F9,this);
	connect(minimizedToSystemTrayAct,SIGNAL(activated()),this,SLOT(SLOT_minimizedToSystemTray()));
	exitAct=new QAction(i18n("Quit"),CTRL+Key_Q,this);
	exitAct->setIconSet(KGlobal::iconLoader()->loadIconSet("exit",KIcon::Small));
	connect(exitAct,SIGNAL(activated()),this,SLOT(SLOT_quit()));

	addToBookmarksAct=new QAction(i18n("&Add To Bookmarks"),CTRL+Key_D,this);
	addToBookmarksAct->setIconSet(KGlobal::iconLoader()->loadIconSet("bookmark_add",KIcon::Small));
	connect(addToBookmarksAct,SIGNAL(activated()),this,SLOT(SLOT_addToBookmarks()));
	editBookmarksAct=new QAction(i18n("&Edit Bookmarks"),CTRL+Key_E,this);
	editBookmarksAct->setIconSet(KGlobal::iconLoader()->loadIconSet("bookmark_add",KIcon::Small));
	connect(editBookmarksAct,SIGNAL(activated()),this,SLOT(SLOT_bookmarksDialog()));

	clearLogAct=new QAction(i18n("Clear Log"),0,this);
	clearLogAct->setIconSet(KGlobal::iconLoader()->loadIconSet("eraser",KIcon::Small));
	connect(clearLogAct,SIGNAL(activated()),this,SLOT(SLOT_clearLog()));
	//clearDirCacheAct=new QAction(i18n("Clear Dir Cache"),0,this);
	//clearDirCacheAct->setIconSet(KGlobal::iconLoader()->loadIconSet("editdelete",KIcon::Small));
	//connect(clearDirCacheAct,SIGNAL(activated()),this,SLOT(SLOT_clearDirCache()));
	clearQueueAct=new QAction(i18n("Delete Saved Queues"),0,this);
	clearQueueAct->setIconSet(KGlobal::iconLoader()->loadIconSet("editdelete",KIcon::Small));
	connect(clearQueueAct,SIGNAL(activated()),this,SLOT(SLOT_clearQueue()));

	configKuFtpAct=new QAction(i18n("Configure KuFTP"),0,this);
	configKuFtpAct->setIconSet(KGlobal::iconLoader()->loadIconSet("configure",KIcon::Small));
	connect(configKuFtpAct,SIGNAL(activated()),this,SLOT(SLOT_configKuFtpDialog()));
}

void KuFtp::createMenuBar()
{
	KuFtpBookmarks::readBookmarks();
	sessionMenu =new KPopupMenu(this);
	connectAct->addTo(sessionMenu);
	sessionMenu->insertSeparator();
	newSessionAct->addTo(sessionMenu);
	closeSessionAct->addTo(sessionMenu);
	sessionMenu->insertSeparator();
	minimizedToSystemTrayAct->addTo(sessionMenu);
	sessionMenu->insertSeparator();
	exitAct->addTo(sessionMenu);

	bookmarksMenu=new KPopupMenu(this);
	updateBookmarksSubPopupMenu();

	toolsMenu=new KPopupMenu(this);
	clearLogAct->addTo(toolsMenu);
//	clearDirCacheAct->addTo(toolsMenu);
	clearQueueAct->addTo(toolsMenu);

	settingsMenu=new KPopupMenu(this);
	configKuFtpAct->addTo(settingsMenu);

	helpmenu=new KPopupMenu(this);
	helpmenu=helpMenu();
	menubar=new KMenuBar(this);
	menubar->insertItem(i18n("&Session"),sessionMenu);
	menubar->insertItem(i18n("&Bookmarks"),bookmarksMenu);
	menubar->insertItem(i18n("&Tools"),toolsMenu);
	menubar->insertItem(i18n("S&ettings"),settingsMenu);
	menubar->insertItem(i18n("&Help"),helpmenu);
}

void KuFtp::createMain()
{
	tabwidget_exists=false;
	QVBox *vbox=new QVBox(this);
	setCentralWidget(vbox);
	KuFtpMain *main=new KuFtpMain(vbox);
	connect(main,SIGNAL(bookmarksChanged()),this,SLOT(updateBookmarksSubPopupMenu()));
	connect(main,SIGNAL(mainStatusChanged()),this,SLOT(SLOT_updateTabWidgetTabLabel()));
	current=new page_t;
	current->p_main=main;
	current->p_vbox=vbox;
	pl_page.append(current);
	pl_vbox.append(vbox);
	pl_main.append(main);
	vbox->setMargin(5);
}

void KuFtp::SLOT_addToBookmarks()
{
	current->p_main->SLOT_addToBookmarks();
}

void KuFtp::updateBookmarksSubPopupMenu()
{
	bookmarksMenu->clear();
	addToBookmarksAct->addTo(bookmarksMenu);
	editBookmarksAct->addTo(bookmarksMenu);
	pl_bmi=KuFtpBookmarks::pl_bmi;
	if(pl_bmi.isEmpty())
	{
		return;
	}
	
	bookmarkActMapper=new QSignalMapper(this);
	connect(bookmarkActMapper,SIGNAL(mapped(int)),this,SLOT(SLOT_fromBookmarkAct(int)));
	
	bookmarksMenu->insertSeparator();
	int i=0;
	pl_bookmarksAct.setAutoDelete(true);
	KuFtpBookmarksItem *topItem;
        for(topItem=pl_bmi.first();topItem;topItem=pl_bmi.next())
	{
		if(topItem->getIsGroup())
		{
			KPopupMenu *topGroupPopupMenu=new KPopupMenu(this);
			KuFtpBookmarksItem *subItem;
			for(subItem=topItem->pl_child.first();subItem;subItem=topItem->pl_child.next())
			{
				QAction *subItemAct=new	QAction(subItem->getSiteName(),0,this);
				subItemAct->setIconSet(KGlobal::iconLoader()->loadIconSet("bookmark",KIcon::Small));
				subItemAct->addTo(topGroupPopupMenu);
				i++;
				connect( subItemAct, SIGNAL( activated() ),bookmarkActMapper,SLOT(map()));
				bookmarkActMapper->setMapping(subItemAct,i);
				bookmarksAct_t *tmp=new bookmarksAct_t;
				tmp->i=i;
				tmp->ptr=subItem;
				pl_bookmarksAct.append(tmp);
			}
			bookmarksMenu->insertItem(KGlobal::iconLoader()->loadIconSet("bookmark_folder",KIcon::Small),topItem->getSiteName(),topGroupPopupMenu);
		}
		else
		{
			QAction *topItemAct=new QAction(topItem->getSiteName(),0,this);
			topItemAct->setIconSet(KGlobal::iconLoader()->loadIconSet("bookmark",KIcon::Small));
			topItemAct->addTo(bookmarksMenu);
			i++;
			connect( topItemAct, SIGNAL( activated() ),bookmarkActMapper,SLOT(map()));
			bookmarkActMapper->setMapping(topItemAct,i);
			bookmarksAct_t *tmp=new bookmarksAct_t;
			tmp->i=i;
			tmp->ptr=topItem;
			pl_bookmarksAct.append(tmp);
		}
	}
	for(KuFtpMain *tmp=pl_main.first();tmp;tmp=pl_main.next())
	{
		tmp->SLOT_updateBookmarksSubPopupMenu();
	}
}

void KuFtp::SLOT_fromBookmarkAct(int i)
{
	KuFtpBookmarksItem *t=NULL;
	for(bookmarksAct_t *tmp=pl_bookmarksAct.first();tmp;tmp=pl_bookmarksAct.next())
	{
		if(tmp->i==i)
		{
			t=tmp->ptr;
			break;
		}
	}
	current->p_main->stopRetry();
	current->p_main->maybeConnectTo(new KuFtpSiteInfo(NULL,t->getHost(),t->getPort(),t->getUsername(),t->getPassword(),
				t->getRemotePath(),t->getLocalPath(),t->getProxyIndex(),t->getEncryptMethod(),t->getIsPasv(),
				t->getIsCorrectPasv(),t->getEncoding()));
}

void KuFtp::SLOT_updateTabWidgetTabLabel()
{
	if(!tabwidget_exists||tabwidget->count()<=1)
	{
		return;
	}
	QString label;
	for(page_t *tmp=pl_page.first();tmp;tmp=pl_page.next())
	{
		if(tmp->p_main->current.established)
		{
			label=tmp->p_main->current.host;
		}
		else
		{
			label=i18n("Session");
		}
		tabwidget->setTabLabel(tmp->p_vbox,label);
	}
}

void KuFtp::SLOT_newSession()
{
	if(!tabwidget_exists )
	{
		tabwidget=new KTabWidget(this);
		connect(tabwidget,SIGNAL(currentChanged(QWidget *)),this,SLOT(SLOT_tabChanged(QWidget *)));
		tabwidget_exists=true;
		tabwidget->setTabPosition(QTabWidget::Top);
		newSessionButton = new QToolButton( tabwidget );
		connect(newSessionButton,SIGNAL(clicked()),this,SLOT(SLOT_newSession()));
		newSessionButton->setIconSet( SmallIcon( "tab_new" ) );
		newSessionButton->adjustSize();
	        tabwidget->setCornerWidget( newSessionButton, BottomLeft );
		closeSessionButton = new QToolButton( tabwidget );
		connect(closeSessionButton,SIGNAL(clicked()),this,SLOT(SLOT_closeSession()));
		closeSessionButton->setIconSet( SmallIconSet( "tab_remove" ) );
		closeSessionButton->adjustSize();
	        tabwidget->setCornerWidget( closeSessionButton, BottomRight );

	        setCentralWidget(tabwidget);
	        tabwidget->insertTab(pl_vbox.current(),i18n("Session"));
		QVBox *vbox=new QVBox(tabwidget);
		vbox->setMargin(5);
		KuFtpMain *main=new KuFtpMain(vbox);
		connect(main,SIGNAL(bookmarksChanged()),this,SLOT(updateBookmarksSubPopupMenu()));
		connect(main,SIGNAL(mainStatusChanged()),this,SLOT(SLOT_updateTabWidgetTabLabel()));
		current=new page_t;
		current->p_main=main;
		current->p_vbox=vbox;
		pl_page.append(current);
		pl_vbox.append(vbox);
		pl_main.append(main);
	        tabwidget->insertTab(vbox,i18n("Session"));
	        //tabwidget->showPage(vbox);
		tabwidget->show();
		SLOT_updateTabWidgetTabLabel();
        }
        else
	{
		if(tabwidget->count()==0)
		{
                	tabwidget->insertTab(pl_vbox.current(),i18n("Session"));
	                tabwidget->show();
			SLOT_updateTabWidgetTabLabel();
		}
		QVBox *vbox=new QVBox(tabwidget);
		vbox->setMargin(5);
		KuFtpMain *main=new KuFtpMain(vbox);
		connect(main,SIGNAL(bookmarksChanged()),this,SLOT(updateBookmarksSubPopupMenu()));
		connect(main,SIGNAL(mainStatusChanged()),this,SLOT(SLOT_updateTabWidgetTabLabel()));
		current=new page_t;
		current->p_main=main;
		current->p_vbox=vbox;
		pl_page.append(current);
		pl_vbox.append(vbox);
		pl_main.append(main);
	        tabwidget->insertTab(vbox,i18n("Session"));
	        tabwidget->showPage(vbox);
	}
}

void KuFtp::SLOT_closeSession()
{
        if( !tabwidget_exists|| (tabwidget->count()==1))
	{
		return;
	}
        else if(tabwidget->count()==2)
	{
		if(current->p_main->current.established||current->p_main->current.transfering)
		{
			if(kuftpGlobalSettings.getIsConfirmExit())
			{
				if(!current->p_main->askToConfirm(this,i18n("Transfering or Connection Established!\nDisconnect and Close this Tab?"),i18n("Confirm Close -- KuFTP")))
				{
					return;
				}
			}
		}
		current->p_main->current.transfering=false;
		current->p_main->SLOT_disconnect();

		page_t *t=current;
	       	pl_vbox.remove(t->p_vbox);
		tabwidget->removePage(t->p_vbox);
		pl_main.remove(t->p_main);
		pl_page.remove(t);
		delete t->p_main;
		delete t->p_vbox;
		delete t;

	       	pl_vbox.current()->reparent(this,0,QPoint(0,0),TRUE);
	     	setCentralWidget(pl_vbox.current());
	   	pl_vbox.current()->raise();
	 	pl_vbox.current()->show();
       		return;
      	}
	else
	{
		if(current->p_main->current.established||current->p_main->current.transfering)
		{
			if(kuftpGlobalSettings.getIsConfirmExit())
			{
				if(!current->p_main->askToConfirm(this,i18n("Transfering or Connection Established!\nDisconnect and Close this Tab?"),i18n("Confirm Close -- KuFTP")))
				{
					return;
				}
			}
		}
		current->p_main->current.transfering=false;
		current->p_main->SLOT_disconnect();

		page_t *t=current;
	       	pl_vbox.remove(t->p_vbox);
		tabwidget->removePage(t->p_vbox);
		pl_main.remove(t->p_main);
		pl_page.remove(t);
		delete t->p_main;
		delete t->p_vbox;
		delete t;
	}
}

void KuFtp::SLOT_minimizedToSystemTray()
{
	if(!kuftpGlobalSettings.getIsShowSystray())
	{
		systemTray->show();
		if(statusInfo==NULL)
			statusInfo=new KuFtpStatusInfo(systemTray,this);
	}
	hide();
}

void KuFtp::SLOT_tabChanged(QWidget *newpage)
{
	for(page_t *tmp=pl_page.first();tmp;tmp=pl_page.next())
	{
		if(tmp->p_vbox==newpage)
		{
			current=tmp;
			return;
		}
	}
}

QString KuFtp::getStatusMsg()
{
	QString msg(i18n("    KuFtp Status\n"));

	QStringList siteList;
	QStringList itemList;
	QStringList percentageList;
	QStringList remainTimeList;
	QStringList speedList;

	unsigned int maxSiteLength=0;
	unsigned int maxItemLength=0;

	for(KuFtpMain *tmp=pl_main.first();tmp;tmp=pl_main.next())
	{
		QString site;
		QString item;
		QString percentage;
		QString remainTime;
		QString speed;
		if(!tmp->current.established)
		{
			if(!tmp->current.retrying)
			{
				site=i18n("Not Connected");
				item="";
				percentage="";
				remainTime="";
				speed="";
			}
			else
			{
				site=tmp->current.host;
				item=i18n("ReConnecting");
				percentage="";
				remainTime="";
				speed="";
			}
		}
		else
		{
			site=tmp->current.host;
			if(!tmp->current.transfering)
			{
				item=i18n("Idle");
				percentage="";
				remainTime="";
				speed="";
			}
			else
			{
				QString s=tmp->current.queueItem->getRemotePath();
				item=s.section('/',s.contains('/'),s.contains('/'));
				if(item.length()>30)
				{
					item=item.left(27).append("...");
				}
				off64_t xfered=tmp->current.queueItem->getXfered()+tmp->current.existXfered;
				int i=xfered*100/(tmp->current.queueItem->getSize());
				percentage=QString::number(i)+"%";
				remainTime=tmp->current.remainTimeString;
				speed=tmp->current.speedString;
			}
		}
		if(maxSiteLength<site.length())
		{
			maxSiteLength=site.length();
		}
		if(maxItemLength<item.length())
		{
			maxItemLength=item.length();
		}
		siteList.push_back(site);
		itemList.push_back(item);
		percentageList.push_back(percentage);
		remainTimeList.push_back(remainTime);
		speedList.push_back(speed);
	}
	unsigned int i=0;
	unsigned int n=siteList.count();
	while(i++<n)
	{
		QString site=siteList.front();
		siteList.pop_front();
		QString item=itemList.front();
		itemList.pop_front();
		QString percentage=percentageList.front();
		percentageList.pop_front();
		QString remainTime=remainTimeList.front();
		remainTimeList.pop_front();
		QString speed=speedList.front();
		speedList.pop_front();
		QString msgItem=QString("%1 %2 %3 %4 %5  \n").arg(site,-maxSiteLength).arg(item,-maxItemLength).arg(percentage).arg(remainTime).arg(speed);
		msg.append(QString("  %1. %2").arg(i).arg(msgItem));
	}
	return msg;
}

/*void KuFtp::SLOT_clearDirCache()
{
	for(KuFtpMain *tmp=pl_main.first();tmp;tmp=pl_main.next())
	        tmp->clearDirCache();
}*/

void KuFtp::SLOT_clearQueue()
{
	QDir dir;
	dir.setPath(QDir::homeDirPath().append("/.kuftp/saved_queues"));
	dir.setFilter(QDir::Files);
	const QFileInfoList *filelist = dir.entryInfoList();
	QFileInfoListIterator fit(*filelist);
	fit.atFirst();
	QFileInfo *fi;
	while ( (fi=fit.current())!=0)
	{
		if(fi->isFile())
		{
			dir.remove(fi->fileName());
		}
		++fit;
	}
}

void KuFtp::SLOT_clearLog()
{
	current->p_main->logWindow->clear();
}

void KuFtp::SLOT_quit()
{
	if(kuftpGlobalSettings.getIsConfirmExit())
	{
		for(KuFtpMain *tmp=pl_main.first();tmp;tmp=pl_main.next())
		{
			if(tmp->current.transfering||tmp->current.established)
			{
				if(!tmp->askToConfirm(this,i18n("Transfering Or Connections Established\nAre you sure to quit?"),i18n("Confirm Exit -- KuFTP")))
				{
					return;
				}
				break;
			}
		}
	}
	kuftpGlobalSettings.setConfirmExit(false);
	for(KuFtpMain *tmp=pl_main.first();tmp;tmp=pl_main.next())
	{
		tmp->SLOT_disconnect();
	}
	kapp->quit();
}

void KuFtp::closeEvent(QCloseEvent *e)
{
	if(kuftpGlobalSettings.getIsMinimizedToSystray())
	{
		if(!kuftpGlobalSettings.getIsShowSystray())
		{
			systemTray->show();
		}
		hide();
		return;
	}
	if(kuftpGlobalSettings.getIsConfirmExit())
	{
		for(KuFtpMain *tmp=pl_main.first();tmp;tmp=pl_main.next())
		{
			if(tmp->current.transfering||tmp->current.established)
			{
				if(!tmp->askToConfirm(this,i18n("Transfering Or Connections Established\nAre you sure to quit?"),i18n("Confirm Exit -- KuFTP")))
				{
					return;
				}
				break;
			}
		}
	}
	kuftpGlobalSettings.setConfirmExit(false);
	for(KuFtpMain *tmp=pl_main.first();tmp;tmp=pl_main.next())
	{
		tmp->SLOT_disconnect();
	}
	e->accept();
}

void KuFtp::SLOT_connectDialog()
{
	KuFtpConnectDialog kcd;
	if(kcd.exec())
	{
		current->p_main->stopRetry();
		if(current->p_main->current.established)
		{
			current->p_main->disconnect();
		}
		QString host=kcd.hostLineEdit->text();
		int port=kcd.portSpinBox->value();
		QString username=kcd.usernameLineEdit->text();
		QString password=kcd.passwordEdit->password();
		QString remotePath=kcd.remotePathLineEdit->text();
		int proxyIndex=kcd.proxyComboBox->currentItem();
		int encryptMethod=kcd.getEncryptMethod();
		bool pasv=kcd.modeCheckBox->isChecked();
		bool correctPasv=kcd.correctPasvCheckBox->isChecked();
		QString encoding=kcd.codecComboBox->currentText();
		KuFtpSiteInfo *site=new KuFtpSiteInfo(NULL,host,port,username,password,remotePath,NULL,proxyIndex,encryptMethod,pasv,correctPasv,encoding);
                current->p_main->connectTo(site);
	}
}

void KuFtp::SLOT_bookmarksDialog()
{
	KuFtpBookmarks bookmarks;
	bookmarks.exec();
	if(bookmarksChanged)
	{
		updateBookmarksSubPopupMenu();
	}
	bookmarksChanged=false;
}

void KuFtp::SLOT_configKuFtpDialog()
{
	KuFtpConfigDialog configDialog;
	configDialog.exec();
	if(kuftpGlobalSettings.getIsShowSystray())
	{
		systemTray->show();
		if(statusInfo==NULL)
			statusInfo=new KuFtpStatusInfo(systemTray,this);
	}
	else
	{
		systemTray->hide();
	}
	for(KuFtpMain *tmp=pl_main.first();tmp;tmp=pl_main.next())
	{
		tmp->current.retryInterval=kuftpGlobalSettings.getRetryInterval();
		tmp->current.retryTimes=kuftpGlobalSettings.getRetryTimes();
		tmp->current.noop=kuftpGlobalSettings.getIsEnableNoop();
		tmp->current.noopInterval=kuftpGlobalSettings.getNoopInterval();
		if(kuftpGlobalSettings.getIsEnableDownloadLimit())
		{
			tmp->setMaxDownloadSpeed(kuftpGlobalSettings.getMaxDownloadSpeed());
		}
		else
		{
			tmp->setMaxDownloadSpeed(0);
		}
		if(kuftpGlobalSettings.getIsEnableUploadLimit())
		{
			tmp->setMaxUploadSpeed(kuftpGlobalSettings.getMaxUploadSpeed());
		}
		else
		{
			tmp->setMaxUploadSpeed(0);
		}
	}
}


