/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KUFTP_H
#define KUFTP_H

#include <kmainwindow.h>
#include <qvbox.h>

#include "bookmarks/kuftpbookmarks.h"
#include "kuftpmain.h"

class QMenuBar;
class QToolBar;
class QActionGroup;
class QAction;
class QVBox;
class QListView;
class QLineEdit;
class QToolButton;
class QCloseEvent;
class QSignalMapper;
template<class type>
class QPtrList;
template<class type>
class QValueList;

class KPopupMenu;
class KTabWidget;
class KSystemTray;
class KStatusBar;

class KuFtpSiteInfo;
class KuFtpStatusInfo;

class KuFtp:public KMainWindow
{
	Q_OBJECT

	public:
		KuFtp();
		static bool bookmarksChanged;
		static KSystemTray *systemTray;
		KuFtpStatusInfo *statusInfo;
		QString getStatusMsg();

	private:
		QPtrList<bookmarksAct_t> pl_bookmarksAct;
		KTabWidget *tabwidget;
		QPtrList<KuFtpMain> pl_main;
		QPtrList<QVBox> pl_vbox;
		struct page_t
		{
			KuFtpMain *p_main;
			QVBox *p_vbox;
		} *current;
		QPtrList<page_t> pl_page;
		QPtrList<KuFtpBookmarksItem> pl_bmi;
		QSignalMapper *bookmarkActMapper;
		bool tabwidget_exists;
		QToolButton *newSessionButton;
		QToolButton *closeSessionButton;

		KMenuBar *menubar;
		KPopupMenu *sessionMenu;
		QAction *connectAct;
		QAction *newSessionAct;
		QAction *closeSessionAct;
		QAction *minimizedToSystemTrayAct;
		QAction *exitAct;

		KPopupMenu *bookmarksMenu;
		QAction *addToBookmarksAct;
		QAction *editBookmarksAct;

		KPopupMenu *toolsMenu;
		QAction *clearLogAct;
//		QAction *clearDirCacheAct;
		QAction *clearQueueAct;

		KPopupMenu *settingsMenu;
		QAction *configKuFtpAct;

		KPopupMenu *helpmenu;

	private:
		void setKuFtpCaption();
		void createAction();
		void createMenuBar();
		void createMain();

	private slots:
		void SLOT_newSession();
		void SLOT_closeSession();
		void SLOT_minimizedToSystemTray();
		void SLOT_fromBookmarkAct(int);
		void SLOT_tabChanged(QWidget *);
		void SLOT_addToBookmarks();
		void updateBookmarksSubPopupMenu();
		void SLOT_updateTabWidgetTabLabel();

	public slots:
		void SLOT_connectDialog();
		//void SLOT_clearDirCache();
		void SLOT_clearLog();
		void SLOT_quit();
		void SLOT_clearQueue();
		void SLOT_configKuFtpDialog();
		void closeEvent(QCloseEvent *event);
		void SLOT_bookmarksDialog();

};

#endif

