/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qlabel.h>
#include <qlayout.h>
#include <qptrlist.h>
#include <qpushbutton.h>

#include <klineedit.h>
#include <kcombobox.h>
#include <klocale.h>

#include "kuftpaddbookmarkdialog.h"
#include "bookmarks/kuftpbookmarks.h"

KuFtpAddBookmarkDialog::KuFtpAddBookmarkDialog(QWidget *parent):QDialog(parent)
{
	setCaption(i18n("Add Bookmark - KuFTP"));
	nameLabel=new QLabel(i18n("&Name: "),this);
	nameLineEdit=new KLineEdit(this);
	nameLabel->setBuddy(nameLineEdit);
	groupLabel=new QLabel(i18n("Create in: "),this);
	groupComboBox=new KComboBox(this);
	connect(groupComboBox,SIGNAL(activated(int)),this,SLOT(SLOT_groupComboBoxChanged(int)));
	groupComboBox->setMaxCount(50);
	groupComboBox->setSizeLimit(18);
	groupComboBox->insertItem(i18n("My Bookmarks"));
	groupComboBoxText="My Bookmarks";

	//newGroupButton = new QPushButton( i18n("New Group"), this);
	okButton = new QPushButton( i18n("OK"), this);
	okButton->setEnabled( false );
        okButton->setDefault( TRUE );
        cancelButton = new QPushButton(i18n("Cancel") ,this );
        //connect( newGroupButton, SIGNAL( clicked() ), this, SLOT( SLOT_newGroup() ) );
        connect( okButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
        connect( cancelButton, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( nameLineEdit, SIGNAL( textChanged(const QString&) ), this, SLOT( SLOT_enableOkButton() ) );

	layout=new QGridLayout(this,3,4,5);
	layout->addMultiCellWidget(nameLabel,0,0,0,0);
	layout->addMultiCellWidget(nameLineEdit,0,0,1,3);
	layout->addMultiCellWidget(groupLabel,1,1,0,0);
	layout->addMultiCellWidget(groupComboBox,1,1,1,3);
	//layout->addMultiCellWidget(newGroupButton,2,2,0,0);
	layout->addMultiCellWidget(okButton,2,2,2,2);
	layout->addMultiCellWidget(cancelButton,2,2,3,3);
	layout->setMargin(10);
	layout->setSpacing(5);

	updateGroupComboBox();
	resize(280,120);
}

void KuFtpAddBookmarkDialog::updateGroupComboBox()
{
	QPtrList<KuFtpBookmarksItem> pl_bmi;
	pl_bmi=KuFtpBookmarks::pl_bmi;
	if(pl_bmi.isEmpty())
	{
	        return;
	}
	KuFtpBookmarksItem *item;
	for(item=pl_bmi.first();item;item=pl_bmi.next())
	{
		if(item->getIsGroup())
		{
			groupComboBox->insertItem(item->getSiteName());
		}
	}
}

void KuFtpAddBookmarkDialog::SLOT_groupComboBoxChanged(int )
{
	groupComboBoxText=groupComboBox->currentText();
}

void KuFtpAddBookmarkDialog::SLOT_enableOkButton()
{
	okButton->setEnabled(true);
}

void KuFtpAddBookmarkDialog::SLOT_newGroup()
{
}


