/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*   original copyright notes               */
/********************************************
 *   Jernej Kos <kostko@jweb-network.net>
 *   Markus Brueffer <markus@brueffer.de>
 *  
 *   Malte Starostik <malte@kde.org>
 *   Duncan Mac-Vicar Prett <duncan@kde.org>
 ********************************************/



#include <qpointarray.h>
#include <qpushbutton.h>
#include <qpoint.h>
#include <qevent.h>
#include <qtooltip.h>
#include <qlayout.h>
#include <qlabel.h>

#include <kapplication.h>
#include <kdialog.h>
#include <klocale.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kstandarddirs.h>
#include <kactivelabel.h>
#include <kglobalsettings.h>

#include "kuftpballoon.h"

KuFtpBalloon::KuFtpBalloon(const QString &text, const QString &pix)
	:QWidget(0,NULL,WStyle_StaysOnTop | WStyle_Customize | WStyle_NoBorder | WStyle_Tool | WX11BypassWM)
{
      	setCaption("");
      	QVBoxLayout *vLayout=new QVBoxLayout(this,22,KDialog::spacingHint());
      	QHBoxLayout *hLayout=new QHBoxLayout(vLayout,KDialog::spacingHint());
      	KActiveLabel *caption = new KActiveLabel(text,this);
	connect(caption,SIGNAL(clicked(int,int)),this,SIGNAL(signalBalloonClicked()));
      	caption->setPalette(QToolTip::palette());
      	caption->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Minimum);
      	if(!pix.isEmpty())
	{
	    	QLabel *image=new QLabel(this,"mImage");
	    	image->setScaledContents(FALSE);
	    	image->setPixmap(KGlobal::iconLoader()->loadIcon(pix,KIcon::Small,0,true));
	    	hLayout->addWidget(image);
      	}
      	hLayout->addWidget(caption);
      	setPalette(QToolTip::palette());
      	setAutoMask(true);
}

void KuFtpBalloon::mousePressEvent(QMouseEvent *e)
{
      	emit signalBalloonClicked();
      	e->accept();
}

void KuFtpBalloon::setAnchor(const QPoint &Anchor)
{
      	anchor = Anchor;
      	updateMask();
}

void KuFtpBalloon::updateMask()
{
      	QRegion mask(10, 10, width() - 20, height() - 20);
      	QPoint corners[8] = 
	{
	    	QPoint(width() - 50, 10),
	    	QPoint(10, 10),
	    	QPoint(10, height() - 50),
	    	QPoint(width() - 50, height() - 50),
	    	QPoint(width() - 10, 10),
	    	QPoint(10, 10),
	    	QPoint(10, height() - 10),
	    	QPoint(width() - 10, height() - 10)
      	};
      	for (int i = 0; i < 4; ++i)
       	{
	    	QPointArray corner;
	    	corner.makeArc(corners[i].x(), corners[i].y(), 40, 40, i * 16 * 90, 16 * 90);
	    	corner.resize(corner.size() + 1);
	    	corner.setPoint(corner.size() - 1, corners[i + 4]);
	    	mask -= corner;
      	}
      	QRect deskRect = KGlobalSettings::desktopGeometry(anchor);
      	bool bottom = (anchor.y() + height()) > (deskRect.height() - 48);
      	bool right = (anchor.x() + width()) > (deskRect.width() - 48);
      	QPointArray arrow(4);
      	arrow.setPoint(0, QPoint(right ? width() : 0, bottom ? height() : 0));
      	arrow.setPoint(1, QPoint(right ? width() - 10 : 10, bottom ? height() - 30 : 30));
      	arrow.setPoint(2, QPoint(right ? width() - 30 : 30, bottom ? height() - 10 : 10));
      	arrow.setPoint(3, arrow[0]);
      	mask += arrow;
      	setMask(mask);
      	move(right ? anchor.x() - width() : anchor.x(), bottom ? anchor.y() - height() : anchor.y());
}



