/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qwidget.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qstringlist.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qlistview.h>

#include <kglobal.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kcharsets.h>
#include <kcombobox.h>
#include <klineedit.h>
#include <klistview.h>
#include <kfiledialog.h>
#include <kcolorbutton.h>
#include <kpassdlg.h>
#include <ktoolbarbutton.h>

#include "kuftpconfigdialog.h"
#include "kuftpsettings.h"

extern KuFtpSettings kuftpGlobalSettings;

KuFtpConfigDialog::KuFtpConfigDialog(QWidget *parent):QTabDialog(parent)
{
	setCaption(i18n("Configure -- KuFTP"));

	generalTab=new QWidget(this);
	setupGeneralTab();
	addTab(generalTab,i18n("General"));
	optionsTab=new QWidget(this);
	setupOptionsTab();
	addTab(optionsTab,i18n("Options"));
	proxyTab=new QWidget(this);
	setupProxyTab();
	addTab(proxyTab,i18n("Proxy"));
	displayTab=new QWidget(this);
	setupDisplayTab();
	addTab(displayTab,i18n("Display"));

	setOkButton();
	setCancelButton();
	connect(this,SIGNAL(applyButtonPressed()),this,SLOT(SLOT_okButtonClicked()));
}

KuFtpConfigDialog::~KuFtpConfigDialog(){}

void KuFtpConfigDialog::setupGeneralTab()
{
	generalGroupBox=new QGroupBox(0,Qt::Vertical,i18n("General"),generalTab);
	generalGroupBox->layout()->setMargin(8);
	generalGroupBox->layout()->setSpacing(10);
	generalGroupBoxLayout=new QGridLayout(generalGroupBox->layout());
	generalGroupBoxLayout->setAlignment(Qt::AlignTop);
	QLabel *emailLabel=new QLabel(i18n("Email Address: "),generalGroupBox);
	emailLineEdit=new KLineEdit(generalGroupBox);
	emailLineEdit->setText(kuftpGlobalSettings.getEmail());
	QLabel *defaultDownloadDirLabel=new QLabel(i18n("Default Folder: "),generalGroupBox);
	defaultDownloadDirLineEdit=new KLineEdit(generalGroupBox);
	defaultDownloadDirLineEdit->setText(kuftpGlobalSettings.getDefaultDownloadDir());
	KToolBarButton *openfileButton=new KToolBarButton("fileopen",0,generalGroupBox);
	connect(openfileButton,SIGNAL(clicked()),this,SLOT(SLOT_setDefaultDownloadDir()));
	enableLogFileCheckBox=new QCheckBox(i18n("Save Logs To File"),generalGroupBox);
	enableLogFileCheckBox->setChecked(kuftpGlobalSettings.getIsEnableLogFile());
	enableLogFileCheckBox->setEnabled(false);
	miniToSystrayByClickXCheckBox=new QCheckBox(i18n("Minimized To Systray By Clicking X Button"),generalGroupBox);
	miniToSystrayByClickXCheckBox->setChecked(kuftpGlobalSettings.getIsMinimizedToSystray());

	generalGroupBoxLayout->addWidget(emailLabel,0,0);
	generalGroupBoxLayout->addMultiCellWidget(emailLineEdit,0,0,1,2);
	generalGroupBoxLayout->addWidget(defaultDownloadDirLabel,1,0);
	generalGroupBoxLayout->addWidget(defaultDownloadDirLineEdit,1,1);
	generalGroupBoxLayout->addWidget(openfileButton,1,2);
	generalGroupBoxLayout->addWidget(enableLogFileCheckBox,2,0);
	generalGroupBoxLayout->addWidget(miniToSystrayByClickXCheckBox,3,0);

	retryGroupBox=new QGroupBox(0,Qt::Vertical,i18n("Reconnect If Failed"),generalTab);
	retryGroupBox->layout()->setMargin(5);
	retryGroupBox->layout()->setSpacing(10);
	retryGroupBoxLayout=new QGridLayout(retryGroupBox->layout());
	QLabel *retryLabel1=new QLabel(i18n("Retry Every "),retryGroupBox);
	retryIntervalSpinBox=new QSpinBox(retryGroupBox);
	retryIntervalSpinBox->setMinValue(1);
	retryIntervalSpinBox->setMaxValue(100);
	retryIntervalSpinBox->setValue(kuftpGlobalSettings.getRetryInterval());
	QLabel *retryLabel2=new QLabel(i18n("Seconds "),retryGroupBox);
	QLabel *retryLabel3=new QLabel(i18n("Retry Times: "),retryGroupBox);
	retryTimesSpinBox=new QSpinBox(retryGroupBox);
	retryTimesSpinBox->setMinValue(0);
	retryTimesSpinBox->setMaxValue(999);
	retryTimesSpinBox->setValue(kuftpGlobalSettings.getRetryTimes());
	retryGroupBoxLayout->addWidget(retryLabel1,0,0);
	retryGroupBoxLayout->addWidget(retryIntervalSpinBox,0,1);
	retryGroupBoxLayout->addWidget(retryLabel2,0,2);
	retryGroupBoxLayout->addWidget(retryLabel3,1,0);
	retryGroupBoxLayout->addWidget(retryTimesSpinBox,1,1);

	noopGroupBox=new QGroupBox(0,Qt::Vertical,i18n("Keep Alive"),generalTab);
	noopGroupBox->layout()->setMargin(5);
	noopGroupBoxLayout=new QGridLayout(noopGroupBox->layout());
	enableNoopCheckBox=new QCheckBox(i18n("NOOP Every "),noopGroupBox);
	enableNoopCheckBox->setChecked(kuftpGlobalSettings.getIsEnableNoop());
	connect(enableNoopCheckBox,SIGNAL(clicked()),this,SLOT(SLOT_enableNoopCheckBoxClicked()));
	noopIntervalSpinBox=new QSpinBox(noopGroupBox);
	noopIntervalSpinBox->setValue(kuftpGlobalSettings.getNoopInterval());
	if(!kuftpGlobalSettings.getIsEnableNoop())
		noopIntervalSpinBox->setEnabled(false);
	QLabel *noopLabel=new QLabel(i18n("Seconds "),noopGroupBox);
	noopGroupBoxLayout->addWidget(enableNoopCheckBox,0,0);
	noopGroupBoxLayout->addWidget(noopIntervalSpinBox,0,1);
	noopGroupBoxLayout->addWidget(noopLabel,0,2);

	speedLimitGroupBox=new QGroupBox(0,Qt::Vertical,i18n("Speed Limits"),generalTab);
	speedLimitGroupBox->layout()->setMargin(5);
	speedLimitGroupBox->layout()->setSpacing(10);
	speedLimitGroupBoxLayout=new QGridLayout(speedLimitGroupBox->layout());
	enableDownloadLimitCheckBox=new QCheckBox(i18n("Maximum Download Speed"),speedLimitGroupBox);
	connect(enableDownloadLimitCheckBox,SIGNAL(clicked()),this,SLOT(SLOT_enableDownloadLimitCheckBoxClicked()));
	enableDownloadLimitCheckBox->setChecked(kuftpGlobalSettings.getIsEnableDownloadLimit());
	downloadLimitSpinBox=new QSpinBox(10,100000,1,speedLimitGroupBox);
	downloadLimitSpinBox->setValue(kuftpGlobalSettings.getMaxDownloadSpeed());
	if(!kuftpGlobalSettings.getIsEnableDownloadLimit())
	{
		downloadLimitSpinBox->setEnabled(false);
	}
	QLabel *downloadLimitLabel=new QLabel(i18n("KB/S"),speedLimitGroupBox);
	enableUploadLimitCheckBox=new QCheckBox(i18n("Maximum Upload Speed"),speedLimitGroupBox);
	connect(enableUploadLimitCheckBox,SIGNAL(clicked()),this,SLOT(SLOT_enableUploadLimitCheckBoxClicked()));
	enableUploadLimitCheckBox->setChecked(kuftpGlobalSettings.getIsEnableUploadLimit());
	uploadLimitSpinBox=new QSpinBox(10,100000,1,speedLimitGroupBox);
	uploadLimitSpinBox->setValue(kuftpGlobalSettings.getMaxUploadSpeed());
	if(!kuftpGlobalSettings.getIsEnableUploadLimit())
	{
		uploadLimitSpinBox->setEnabled(false);
	}
	QLabel *uploadLimitLabel=new QLabel(i18n("KB/S"),speedLimitGroupBox);
	speedLimitGroupBoxLayout->addWidget(enableDownloadLimitCheckBox,0,0);
	speedLimitGroupBoxLayout->addWidget(downloadLimitSpinBox,0,1);
	speedLimitGroupBoxLayout->addWidget(downloadLimitLabel,0,2);
	speedLimitGroupBoxLayout->addWidget(enableUploadLimitCheckBox,1,0);
	speedLimitGroupBoxLayout->addWidget(uploadLimitSpinBox,1,1);
	speedLimitGroupBoxLayout->addWidget(uploadLimitLabel,1,2);

	networkTimeoutGroupBox=new QGroupBox(0,Qt::Vertical,i18n("Network Timeout"),generalTab);
	networkTimeoutGroupBox->layout()->setMargin(5);
	networkTimeoutGroupBoxLayout=new QGridLayout(networkTimeoutGroupBox->layout());
	QLabel *networkTimeoutLabel=new QLabel(i18n("Network Timeout ( Seconds 1 to 600 ) "),networkTimeoutGroupBox);
	networkTimeoutSpinBox=new QSpinBox(networkTimeoutGroupBox);
	networkTimeoutSpinBox->setMinValue(1);
	networkTimeoutSpinBox->setMaxValue(600);
	networkTimeoutSpinBox->setValue(kuftpGlobalSettings.getNetworkTimeout());
	networkTimeoutGroupBoxLayout->addMultiCellWidget(networkTimeoutLabel,0,0,0,2);
	networkTimeoutGroupBoxLayout->addMultiCellWidget(networkTimeoutSpinBox,0,0,3,3);

	generalLayout=new QVBoxLayout(generalTab,5);
	generalLayout->addWidget(generalGroupBox);
	generalLayout->addWidget(networkTimeoutGroupBox);
	generalLayout->addWidget(retryGroupBox);
	generalLayout->addWidget(noopGroupBox);
	generalLayout->addWidget(speedLimitGroupBox);

	//setTabOrder()
}

void KuFtpConfigDialog::setupOptionsTab()
{
	confirmGroupBox=new QGroupBox(0,Qt::Vertical,i18n("Confirm"),optionsTab);
	confirmGroupBox->layout()->setMargin(5);
	confirmGroupBox->layout()->setSpacing(10);
	confirmGroupBoxLayout=new QVBoxLayout(confirmGroupBox->layout());
	confirmExitCheckBox=new QCheckBox(i18n("Confirm Exit"),confirmGroupBox);
	confirmExitCheckBox->setChecked(kuftpGlobalSettings.getIsConfirmExit());
	confirmDeleteFileCheckBox=new QCheckBox(i18n("Confirm Delete File/Dirs"),confirmGroupBox);
	confirmDeleteFileCheckBox->setChecked(kuftpGlobalSettings.getIsConfirmDeleteFile());
	confirmClearQueueCheckBox=new QCheckBox(i18n("Confirm Clear Queue"),confirmGroupBox);
	confirmClearQueueCheckBox->setChecked(kuftpGlobalSettings.getIsConfirmClearQueue());
	confirmDeleteQueueItemCheckBox=new QCheckBox(i18n("Confirm Delete Queue Items"),confirmGroupBox);
	confirmDeleteQueueItemCheckBox->setChecked(kuftpGlobalSettings.getIsConfirmDeleteQueueItem());
	resumeExistFileCheckBox=new QCheckBox(i18n("Resume Exists File Rather Than Overwrite"),confirmGroupBox);
	resumeExistFileCheckBox->setChecked(kuftpGlobalSettings.getIsResumeExistFile());
	confirmGroupBoxLayout->addWidget(confirmExitCheckBox);
	confirmGroupBoxLayout->addWidget(confirmDeleteFileCheckBox);
	confirmGroupBoxLayout->addWidget(confirmClearQueueCheckBox);
	confirmGroupBoxLayout->addWidget(confirmDeleteQueueItemCheckBox);
	confirmGroupBoxLayout->addWidget(resumeExistFileCheckBox);

	codecGroupBox=new QGroupBox(0,Qt::Vertical,i18n("Server Encoding"),optionsTab);
	codecGroupBox->layout()->setMargin(5);
	codecGroupBox->layout()->setSpacing(10);
	codecGroupBoxLayout=new QVBoxLayout(codecGroupBox->layout());
	codecComboBox=new KComboBox(codecGroupBox);
	codecComboBox->insertStringList(KGlobal::charsets()->descriptiveEncodingNames());
	codecComboBox->setCurrentText(kuftpGlobalSettings.getDescriptiveEncoding());
	codecGroupBoxLayout->addWidget(new QLabel(i18n("Set Most Server Encoding Here"),codecGroupBox));
	codecGroupBoxLayout->addWidget(codecComboBox);
	
	optionsSpacer=new QSpacerItem(20,20,QSizePolicy::Minimum,QSizePolicy::Expanding);

	optionsLayout=new QVBoxLayout(optionsTab,5,10);
	optionsLayout->addWidget(confirmGroupBox);
	optionsLayout->addWidget(codecGroupBox);
	optionsLayout->addItem(optionsSpacer);
}

void KuFtpConfigDialog::setupProxyTab()
{
	proxyListView=new KListView(proxyTab);
	proxyListView->addColumn(i18n("Name"));
	proxyListView->addColumn(i18n("Host"));
	proxyListView->addColumn(i18n("Port"));
	proxyListView->addColumn(i18n("Type"));
	proxyListView->addColumn(i18n("Default"));
	pl_proxy.clear();
	for(proxy_t *t=kuftpGlobalSettings.pl_proxy.first();t;t=kuftpGlobalSettings.pl_proxy.next())
	{
		pl_proxy.append(t);
	}
	updateProxyListView();	
	connect(proxyListView,SIGNAL(clicked(QListViewItem *)),this,SLOT(SLOT_proxyListViewClicked(QListViewItem *)));
	proxyGroupBox=new QGroupBox(0,Qt::Vertical,i18n("Proxy Setup"),proxyTab);
	proxyGroupBox->layout()->setMargin(5);
	proxyGroupBox->layout()->setSpacing(10);
	proxyGroupBoxLayout=new QGridLayout(proxyGroupBox->layout());
	proxyNameLineEdit=new KLineEdit(proxyGroupBox);
	proxyTypeButtonGroup = new QButtonGroup(1,Qt::Horizontal,"Proxy Type",proxyGroupBox);
	//proxyTypeButtonGroup->setExclusive(true);
	httpProxyRadioButton=new QRadioButton("Http",proxyTypeButtonGroup);
	socks4ProxyRadioButton=new QRadioButton("Socks4",proxyTypeButtonGroup);
	socks5ProxyRadioButton=new QRadioButton("Socks5",proxyTypeButtonGroup);
	//use buttongroup's clicked() instead
	connect(httpProxyRadioButton,SIGNAL(clicked()),this,SLOT(SLOT_httpRadioButtonClicked()));
	connect(socks4ProxyRadioButton,SIGNAL(clicked()),this,SLOT(SLOT_socksRadioButtonClicked()));
	connect(socks5ProxyRadioButton,SIGNAL(clicked()),this,SLOT(SLOT_socksRadioButtonClicked()));
	httpProxyRadioButton->setChecked(true);
	proxyHostLineEdit=new KLineEdit(proxyGroupBox);
	proxyPortSpinBox=new QSpinBox(proxyGroupBox);
	proxyPortSpinBox->setMinValue(1);
	proxyPortSpinBox->setMaxValue(65535);
	proxyPortSpinBox->setValue(8080);
	proxyUserLineEdit=new KLineEdit(proxyGroupBox);
	proxyPasswordEdit=new KPasswordEdit(KPasswordEdit::OneStar,proxyGroupBox,0);
	proxyGroupBoxLayout->addMultiCellWidget(new QLabel(i18n("Proxy Name"),proxyGroupBox),0,0,0,0);
	proxyGroupBoxLayout->addMultiCellWidget(proxyNameLineEdit,0,0,1,3);
	proxyGroupBoxLayout->addMultiCellWidget(proxyTypeButtonGroup,1,4,0,0);
	proxyGroupBoxLayout->addMultiCellWidget(new QLabel(i18n("Host"),proxyGroupBox),1,1,1,1);
	proxyGroupBoxLayout->addMultiCellWidget(proxyHostLineEdit,1,1,2,3);
	proxyGroupBoxLayout->addMultiCellWidget(new QLabel(i18n("Port"),proxyGroupBox),2,2,1,1);
	proxyGroupBoxLayout->addMultiCellWidget(proxyPortSpinBox,2,2,2,2);
	proxyGroupBoxLayout->addMultiCellWidget(new QLabel(i18n("Username"),proxyGroupBox),3,3,1,1);
	proxyGroupBoxLayout->addMultiCellWidget(proxyUserLineEdit,3,3,2,3);
	proxyGroupBoxLayout->addMultiCellWidget(new QLabel(i18n("Password"),proxyGroupBox),4,4,1,1);
	proxyGroupBoxLayout->addMultiCellWidget(proxyPasswordEdit,4,4,2,3);
	newProxyButton=new QPushButton(i18n("New"),proxyTab);
	connect(newProxyButton,SIGNAL(clicked()),this,SLOT(SLOT_newProxyButtonClicked()));
	applyProxyButton=new QPushButton(i18n("Apply"),proxyTab);
	connect(applyProxyButton,SIGNAL(clicked()),this,SLOT(SLOT_applyProxyButtonClicked()));
	setDefaultButton=new QPushButton(i18n("Set Default"),proxyTab);
	connect(setDefaultButton,SIGNAL(clicked()),this,SLOT(SLOT_setDefaultButtonClicked()));
	removeProxyButton=new QPushButton(i18n("Remove"),proxyTab);
	connect(removeProxyButton,SIGNAL(clicked()),this,SLOT(SLOT_removeProxyButtonClicked()));
	applyProxyButton->setEnabled(false);
	setDefaultButton->setEnabled(false);
	removeProxyButton->setEnabled(false);
	proxyLayout=new QGridLayout(proxyTab,4,4,5);
	proxyLayout->addMultiCellWidget(proxyListView,0,0,0,3);
	proxyLayout->addMultiCellWidget(proxyGroupBox,1,1,0,3);
	proxyLayout->addMultiCellWidget(newProxyButton,2,2,0,0);
	proxyLayout->addMultiCellWidget(applyProxyButton,2,2,1,1);
	proxyLayout->addMultiCellWidget(setDefaultButton,2,2,2,2);
	proxyLayout->addMultiCellWidget(removeProxyButton,2,2,3,3);
}

void KuFtpConfigDialog::setupDisplayTab()
{
	colorGroupBox=new QGroupBox(0,Qt::Vertical,i18n("Color Setup"),displayTab);
	colorGroupBox->layout()->setMargin(5);
	colorGroupBox->layout()->setSpacing(10);
	colorGroupBoxLayout=new QGridLayout(colorGroupBox->layout());
	inColorButton=new KColorButton(displayTab);
	inColorButton->setColor(kuftpGlobalSettings.getInColor());
	outColorButton=new KColorButton(displayTab);
	outColorButton->setColor(kuftpGlobalSettings.getOutColor());
	colorGroupBoxLayout->addMultiCellWidget(new QLabel(i18n("Server Response Color"),colorGroupBox),0,0,0,0);
	colorGroupBoxLayout->addMultiCellWidget(inColorButton,0,0,1,1);
	colorGroupBoxLayout->addMultiCellWidget(new QLabel(i18n("Client Command Color"),colorGroupBox),1,1,0,0);
	colorGroupBoxLayout->addMultiCellWidget(outColorButton,1,1,1,1);

	displayGroupBox=new QGroupBox(0,Qt::Vertical,i18n("Display"),displayTab);
	displayGroupBox->layout()->setMargin(5);
	displayGroupBox->layout()->setSpacing(10);
	displayGroupBoxLayout=new QVBoxLayout(displayGroupBox->layout());
	showHiddenCheckBox=new QCheckBox(i18n("Show Hidden Files/Dirs"),displayGroupBox);
	showHiddenCheckBox->setChecked(kuftpGlobalSettings.getIsShowHidden());
	showSystrayCheckBox=new QCheckBox(i18n("Show SystemTray Icon"),displayGroupBox);
	showSystrayCheckBox->setChecked(kuftpGlobalSettings.getIsShowSystray());
	showBalloonWhenConnectionEstablishedCheckBox=new QCheckBox(i18n("Show Balloon When Connection Established"),displayGroupBox);
	showBalloonWhenConnectionEstablishedCheckBox->setChecked(kuftpGlobalSettings.getIsShowBalloonWhenConnectionEstablished());
	showBalloonWhenTransferFinishedCheckBox=new QCheckBox(i18n("Show Balloon When Transfer Finished"),displayGroupBox);
	showBalloonWhenTransferFinishedCheckBox->setChecked(kuftpGlobalSettings.getIsShowBalloonWhenTransferFinished());
	displayGroupBoxLayout->addWidget(showHiddenCheckBox);
	displayGroupBoxLayout->addWidget(showSystrayCheckBox);
	displayGroupBoxLayout->addWidget(showBalloonWhenConnectionEstablishedCheckBox);
	displayGroupBoxLayout->addWidget(showBalloonWhenTransferFinishedCheckBox);

	displaySpacer=new QSpacerItem(20,20,QSizePolicy::Minimum,QSizePolicy::Expanding);
	displayLayout=new QVBoxLayout(displayTab,5,10);
	displayLayout->addWidget(colorGroupBox);
	displayLayout->addWidget(displayGroupBox);
	displayLayout->addItem(displaySpacer);
}

void KuFtpConfigDialog::SLOT_socksRadioButtonClicked()
{
	proxyPortSpinBox->setValue(1080);
}

void KuFtpConfigDialog::SLOT_httpRadioButtonClicked()
{
	proxyPortSpinBox->setValue(8080);
}

void KuFtpConfigDialog::SLOT_okButtonClicked()
{
	kuftpGlobalSettings.setEmail(emailLineEdit->text()==""?kuftpGlobalSettings.getEmail():emailLineEdit->text());
	kuftpGlobalSettings.setDefaultDownloadDir(defaultDownloadDirLineEdit->text()==""?QDir::homeDirPath():defaultDownloadDirLineEdit->text());
	kuftpGlobalSettings.setEnableLogFile(enableLogFileCheckBox->isChecked());
	kuftpGlobalSettings.setMinimizedToSystray(miniToSystrayByClickXCheckBox->isChecked());
	kuftpGlobalSettings.setRetryInterval(retryIntervalSpinBox->value());
	kuftpGlobalSettings.setRetryTimes(retryTimesSpinBox->value());
	kuftpGlobalSettings.setEnableNoop(enableNoopCheckBox->isChecked());
	kuftpGlobalSettings.setNoopInterval(noopIntervalSpinBox->value());
	kuftpGlobalSettings.setEnableDownloadLimit(enableDownloadLimitCheckBox->isChecked());
	kuftpGlobalSettings.setMaxDownloadSpeed(downloadLimitSpinBox->value());
	kuftpGlobalSettings.setEnableUploadLimit(enableUploadLimitCheckBox->isChecked());
	kuftpGlobalSettings.setMaxUploadSpeed(uploadLimitSpinBox->value());
	kuftpGlobalSettings.setNetworkTimeout(networkTimeoutSpinBox->value());
	kuftpGlobalSettings.setConfirmExit(confirmExitCheckBox->isChecked());
	kuftpGlobalSettings.setConfirmDeleteFile(confirmDeleteFileCheckBox->isChecked());
	kuftpGlobalSettings.setConfirmDeleteQueueItem(confirmDeleteQueueItemCheckBox->isChecked());
	kuftpGlobalSettings.setConfirmClearQueue(confirmClearQueueCheckBox->isChecked());
	kuftpGlobalSettings.setResumeExistFile(resumeExistFileCheckBox->isChecked());
	kuftpGlobalSettings.setDescriptiveEncoding(codecComboBox->currentText());
	kuftpGlobalSettings.setInColor(inColorButton->color());
	kuftpGlobalSettings.setOutColor(outColorButton->color());
	kuftpGlobalSettings.setShowSystray(showSystrayCheckBox->isChecked());
	kuftpGlobalSettings.setShowHidden(showHiddenCheckBox->isChecked());
	kuftpGlobalSettings.setShowBalloonWhenConnectionEstablished(showBalloonWhenConnectionEstablishedCheckBox->isChecked());
	kuftpGlobalSettings.setShowBalloonWhenTransferFinished(showBalloonWhenTransferFinishedCheckBox->isChecked());
	kuftpGlobalSettings.pl_proxy.clear();
	for(proxy_t *t=pl_proxy.first();t;t=pl_proxy.next())
	{
		kuftpGlobalSettings.pl_proxy.append(t);
	}
	kuftpGlobalSettings.writeConfig();
}

void KuFtpConfigDialog::SLOT_enableNoopCheckBoxClicked()
{
	if(enableNoopCheckBox->isChecked())
	{
		noopIntervalSpinBox->setEnabled(true);
	}
	else
	{
		noopIntervalSpinBox->setEnabled(false);
	}
}

void KuFtpConfigDialog::SLOT_enableDownloadLimitCheckBoxClicked()
{
	if(enableDownloadLimitCheckBox->isChecked())
	{
		downloadLimitSpinBox->setEnabled(true);
	}
	else
	{
		downloadLimitSpinBox->setEnabled(false);
	}
}

void KuFtpConfigDialog::SLOT_enableUploadLimitCheckBoxClicked()
{
	if(enableUploadLimitCheckBox->isChecked())
	{
		uploadLimitSpinBox->setEnabled(true);
	}
	else
	{
		uploadLimitSpinBox->setEnabled(false);
	}
}

void KuFtpConfigDialog::updateProxyListView()
{
	QListViewItem *tmp;
        while((tmp=proxyListView->firstChild()))
	{
                delete tmp;
	}
	pl_proxyPair.clear();
	for(proxy_t *t=pl_proxy.first();t;t=pl_proxy.next())
	{
		QString type;
		if(t->type==http)
		{
			type="http";
		}
		else if(t->type==socks4)
		{
			type="socks4";
		}
		else
		{
			type="socks5";
		}
		QListViewItem *item=new QListViewItem(proxyListView,t->name,t->host,QString("%1").arg(t->port),type,t->isDefault?"Default":NULL,t->username,t->password);
		proxyPair_t *proxyPair=new proxyPair_t;
		proxyPair->p_item=item;
		proxyPair->p_proxy=t;
		pl_proxyPair.append(proxyPair);
	}
}

void KuFtpConfigDialog::SLOT_newProxyButtonClicked()
{
	if(proxyHostLineEdit->text().stripWhiteSpace()=="")
	{
		return;
	}
	proxy_t *proxy=new proxy_t;
	proxy->host=proxyHostLineEdit->text().stripWhiteSpace();
	QString s=proxyNameLineEdit->text().stripWhiteSpace();
	proxy->name=(s==""?proxy->host:s);
	if(httpProxyRadioButton->isChecked())
	{
		proxy->type=http;
	}
	else if(socks4ProxyRadioButton->isChecked())
	{
		proxy->type=socks4;
	}
	else
	{
		proxy->type=socks5;
	}
	proxy->port=proxyPortSpinBox->value();
	proxy->username=proxyUserLineEdit->text();
	proxy->password=QString(proxyPasswordEdit->password());
	proxy->isDefault=false;
	proxy->index=pl_proxy.count();
	pl_proxy.append(proxy);
	updateProxyListView();
	proxyNameLineEdit->clear();
	proxyHostLineEdit->clear();
	proxyPortSpinBox->setValue(8080);
	httpProxyRadioButton->setChecked(true);
	proxyUserLineEdit->clear();
	proxyPasswordEdit->erase();
	proxyHostLineEdit->setFocus();
	proxyListView->clearSelection();
}

void KuFtpConfigDialog::SLOT_applyProxyButtonClicked()
{
	if(currentProxyPair==NULL)
	{
		return;
	}
	if(proxyHostLineEdit->text().stripWhiteSpace()=="")
	{
		return;
	}
	currentProxyPair->p_proxy->host=proxyHostLineEdit->text().stripWhiteSpace();
	QString s=proxyNameLineEdit->text().stripWhiteSpace();
	currentProxyPair->p_proxy->name=(s==""?currentProxyPair->p_proxy->host:s);
	if(httpProxyRadioButton->isChecked())
	{
		currentProxyPair->p_proxy->type=http;
	}
	else if(socks4ProxyRadioButton->isChecked())
	{
		currentProxyPair->p_proxy->type=socks4;
	}
	else
	{
		currentProxyPair->p_proxy->type=socks5;
	}
	currentProxyPair->p_proxy->port=proxyPortSpinBox->value();
	currentProxyPair->p_proxy->username=proxyUserLineEdit->text();
	currentProxyPair->p_proxy->password=QString(proxyPasswordEdit->password());
	currentProxyPair->p_proxy->isDefault=false;
	updateProxyListView();
	proxyNameLineEdit->clear();
	proxyHostLineEdit->clear();
	proxyPortSpinBox->setValue(8080);
	httpProxyRadioButton->setChecked(true);
	proxyUserLineEdit->clear();
	proxyPasswordEdit->erase();
	proxyListView->clearSelection();
	newProxyButton->setEnabled(true);
	applyProxyButton->setEnabled(false);
	setDefaultButton->setEnabled(false);
	removeProxyButton->setEnabled(false);
}

void KuFtpConfigDialog::SLOT_setDefaultButtonClicked()
{
	if(currentProxyPair==NULL)
	{
		return;
	}
	if(currentProxyPair->p_proxy->isDefault)
	{
		return;
	}
	for(proxy_t *t=pl_proxy.first();t;t=pl_proxy.next())
	{
		t->isDefault=false;
	}
	currentProxyPair->p_proxy->isDefault=true;
	updateProxyListView();
	proxyNameLineEdit->clear();
	proxyHostLineEdit->clear();
	proxyPortSpinBox->setValue(8080);
	httpProxyRadioButton->setChecked(true);
	proxyUserLineEdit->clear();
	proxyPasswordEdit->erase();
	proxyListView->clearSelection();
	newProxyButton->setEnabled(true);
	applyProxyButton->setEnabled(false);
	setDefaultButton->setEnabled(false);
	removeProxyButton->setEnabled(false);
}

void KuFtpConfigDialog::SLOT_removeProxyButtonClicked()
{
	if(currentProxyPair==NULL)
	{
		return;
	}
	pl_proxy.remove(currentProxyPair->p_proxy);
	updateProxyListView();
	proxyNameLineEdit->clear();
	proxyHostLineEdit->clear();
	proxyPortSpinBox->setValue(8080);
	httpProxyRadioButton->setChecked(true);
	proxyUserLineEdit->clear();
	proxyPasswordEdit->erase();
	proxyListView->clearSelection();
	proxyNameLineEdit->clear();
	proxyHostLineEdit->clear();
	proxyPortSpinBox->setValue(8080);
	httpProxyRadioButton->setChecked(true);
	proxyUserLineEdit->clear();
	proxyPasswordEdit->erase();
	proxyListView->clearSelection();
	newProxyButton->setEnabled(true);
	applyProxyButton->setEnabled(false);
	setDefaultButton->setEnabled(false);
	removeProxyButton->setEnabled(false);
}

void KuFtpConfigDialog::SLOT_proxyListViewClicked(QListViewItem *item)
{
	if(item==NULL)
	{
		newProxyButton->setEnabled(true);
		applyProxyButton->setEnabled(false);
		setDefaultButton->setEnabled(false);
		removeProxyButton->setEnabled(false);
		proxyNameLineEdit->clear();
		proxyHostLineEdit->clear();
		proxyPortSpinBox->setValue(8080);
		httpProxyRadioButton->setChecked(true);
		proxyUserLineEdit->clear();
		proxyPasswordEdit->erase();
		return;
	}
	newProxyButton->setEnabled(false);
	applyProxyButton->setEnabled(true);
	setDefaultButton->setEnabled(true);
	removeProxyButton->setEnabled(true);
	for(proxyPair_t *t=pl_proxyPair.first();t;t=pl_proxyPair.next())
	{
		if(t->p_item==item)
		{
			currentProxyPair=t;
			break;
		}
	}
	proxyNameLineEdit->setText(currentProxyPair->p_proxy->name);
	proxyHostLineEdit->setText(currentProxyPair->p_proxy->host);
	proxyPortSpinBox->setValue(currentProxyPair->p_proxy->port);
	proxyUserLineEdit->setText(currentProxyPair->p_proxy->username);
	proxyPasswordEdit->insert(currentProxyPair->p_proxy->password);
	if(currentProxyPair->p_proxy->type==http)
	{
		httpProxyRadioButton->setChecked(true);
	}
	else if(currentProxyPair->p_proxy->type==socks4)
	{
		socks4ProxyRadioButton->setChecked(true);
	}
	else
	{
		socks5ProxyRadioButton->setChecked(true);
	}
}

void KuFtpConfigDialog::SLOT_setDefaultDownloadDir()
{
	defaultDownloadDirLineEdit->setText(KFileDialog::getExistingDirectory());
}


