/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KUFTPCONFIGDIALOG_H
#define KUFTPCONFIGDIALOG_H

#include <qtabdialog.h>

class QWidget;
class QLabel;
class QSpinBox;
class QCheckBox;
class QSpacerItem;
class QGroupBox;
class QButtonGroup;
class QRadioButton;
class QPushButton;
class QListViewItem;
class QVBoxLayout;
class QGridLayout;

class KComboBox;
class KLineEdit;
class KListView;
class KPasswordEdit;
class KColorButton;
class KComboBox;

template<class type>
class QPtrList;

struct proxy_t;

class KuFtpConfigDialog:public QTabDialog
{
	Q_OBJECT
	public:
		KuFtpConfigDialog(QWidget *parent=0);
		~KuFtpConfigDialog();

		QWidget *generalTab;
		QVBoxLayout *generalLayout;
		QGroupBox *generalGroupBox;
		QGridLayout *generalGroupBoxLayout;
		KLineEdit *emailLineEdit;
		KLineEdit *defaultDownloadDirLineEdit;
		QCheckBox *enableLogFileCheckBox;
		QCheckBox *miniToSystrayByClickXCheckBox;
		QGroupBox *retryGroupBox;
		QGridLayout *retryGroupBoxLayout;
		QSpinBox *retryIntervalSpinBox;
		QSpinBox *retryTimesSpinBox;
		QGroupBox *noopGroupBox;
		QGridLayout *noopGroupBoxLayout;
		QCheckBox *enableNoopCheckBox;
		QSpinBox *noopIntervalSpinBox;
		QGroupBox *speedLimitGroupBox;
		QGridLayout *speedLimitGroupBoxLayout;
		QCheckBox *enableDownloadLimitCheckBox;
		QSpinBox *downloadLimitSpinBox;
		QCheckBox *enableUploadLimitCheckBox;
		QSpinBox *uploadLimitSpinBox;
		QGroupBox *networkTimeoutGroupBox;
		QGridLayout *networkTimeoutGroupBoxLayout;
		QSpinBox *networkTimeoutSpinBox;
		//

		QWidget *optionsTab;
		QVBoxLayout *optionsLayout;
		QGroupBox *confirmGroupBox;
		QVBoxLayout *confirmGroupBoxLayout;
		QCheckBox *confirmExitCheckBox;
		QCheckBox *confirmDeleteFileCheckBox;
		QCheckBox *confirmClearQueueCheckBox;
		QCheckBox *confirmDeleteQueueItemCheckBox;
		QCheckBox *resumeExistFileCheckBox;
		QGroupBox *codecGroupBox;
		QVBoxLayout *codecGroupBoxLayout;
		KComboBox *codecComboBox;
		QSpacerItem *optionsSpacer;

		QWidget *proxyTab;
		QGridLayout *proxyLayout;
		KListView *proxyListView;
		QGroupBox *proxyGroupBox;
		QGridLayout *proxyGroupBoxLayout;
		KLineEdit *proxyNameLineEdit;
		QButtonGroup *proxyTypeButtonGroup;
		QRadioButton *httpProxyRadioButton;
		QRadioButton *socks4ProxyRadioButton;
		QRadioButton *socks5ProxyRadioButton;
		KLineEdit *proxyHostLineEdit;
		QSpinBox *proxyPortSpinBox;
		KLineEdit *proxyUserLineEdit;
		KPasswordEdit *proxyPasswordEdit;
		QPushButton *newProxyButton;
		QPushButton *applyProxyButton;
		QPushButton *setDefaultButton;
		QPushButton *removeProxyButton;

		QWidget *displayTab;
		QVBoxLayout *displayLayout;
		QGroupBox *colorGroupBox;
		QGridLayout *colorGroupBoxLayout;
		KColorButton *inColorButton;
		KColorButton *outColorButton;
		QGroupBox *displayGroupBox;
		QVBoxLayout *displayGroupBoxLayout;
		QCheckBox *showHiddenCheckBox;
		QCheckBox *showSystrayCheckBox;
		QCheckBox *showBalloonWhenConnectionEstablishedCheckBox;
		QCheckBox *showBalloonWhenTransferFinishedCheckBox;
		QSpacerItem *displaySpacer;

		QPtrList<proxy_t> pl_proxy;
		struct proxyPair_t
		{
			QListViewItem *p_item;
			proxy_t *p_proxy;
		} *currentProxyPair;
		QPtrList<proxyPair_t> pl_proxyPair;

	private:
		void setupGeneralTab();
		void setupOptionsTab();
		void setupProxyTab();
		void setupDisplayTab();
		void updateProxyListView();

	private slots:
		void SLOT_enableNoopCheckBoxClicked();
		void SLOT_enableDownloadLimitCheckBoxClicked();
		void SLOT_enableUploadLimitCheckBoxClicked();
		void SLOT_setDefaultDownloadDir();
		void SLOT_httpRadioButtonClicked();
		void SLOT_socksRadioButtonClicked();
		void SLOT_okButtonClicked();
		void SLOT_newProxyButtonClicked();
		void SLOT_applyProxyButtonClicked();
		void SLOT_setDefaultButtonClicked();
		void SLOT_removeProxyButtonClicked();
		void SLOT_proxyListViewClicked(QListViewItem *item);
};

#endif

