/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>

#include <qlabel.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qptrlist.h>
#include <qstring.h>
#include <qvbox.h>
#include <qdir.h>
#include <qurl.h>
#include <qdom.h>

#include <klineedit.h>
#include <kcombobox.h>
#include <ktabwidget.h>
#include <klocale.h>
#include <kglobal.h>
#include <kcharsets.h>
#include <ktoolbarbutton.h>
#include <kpassdlg.h>

#include "kuftpconnectdialog.h"
#include "kuftpsiteinfo.h"
#include "kuftpsettings.h"

extern KuFtpSettings kuftpGlobalSettings;

KuFtpConnectDialog::KuFtpConnectDialog(QWidget *parent):QTabDialog(parent)
{
	setCaption(i18n("Quick Connect - KuFTP"));
	generalTab=new QWidget(this);
	historyLabel=new QLabel(i18n("Hi&story: "),generalTab);
	historyComboBox=new KComboBox(generalTab);
	historyLabel->setBuddy(historyComboBox);
	historyComboBox->setMaxCount(50);
	historyComboBox->setSizeLimit(18);
	connect(historyComboBox,SIGNAL(activated(int)),this,SLOT(SLOT_historyItemSelectChanged(int)));
	clearHistoryButton=new KToolBarButton("clear_left",0,generalTab);
	connect(clearHistoryButton,SIGNAL(clicked()),this,SLOT(SLOT_clearHistory()));
	hostLabel=new QLabel(i18n("&Host: "),generalTab);
	hostLineEdit=new KLineEdit(generalTab);
	connect(hostLineEdit,SIGNAL(lostFocus()),this,SLOT(SLOT_parseHostLineEditText()));
	hostLabel->setBuddy(hostLineEdit);
	portLabel=new QLabel(i18n("Por&t: "),generalTab);
	portSpinBox=new QSpinBox(generalTab);
	portLabel->setBuddy(portSpinBox);
	portSpinBox->setMinValue(1);
	portSpinBox->setMaxValue(65535);
	portSpinBox->setValue(21);
	usernameLabel=new QLabel(i18n("&Username: "),generalTab);
	usernameLineEdit=new KLineEdit(generalTab);
	usernameLabel->setBuddy(usernameLineEdit);
	anonymousCheckBox=new QCheckBox(i18n("&Anomymous"),generalTab);
	connect(anonymousCheckBox,SIGNAL(clicked()),this,SLOT(SLOT_anonymousCheckBoxClicked()));
	passwordLabel=new QLabel(i18n("Pass&word: "),generalTab);
	passwordEdit=new KPasswordEdit(KPasswordEdit::OneStar,generalTab,0);
	passwordLabel->setBuddy(passwordEdit);
	remotePathLabel=new QLabel(i18n("&RemotePath: "),generalTab);
	remotePathLineEdit=new KLineEdit(generalTab);
	remotePathLabel->setBuddy(remotePathLineEdit);
	proxyLabel=new QLabel(i18n("Proxy: "),generalTab);
	proxyComboBox=new KComboBox(generalTab);
	proxyComboBox->insertItem(i18n("None"));
	proxyComboBox->insertItem(i18n("Default"));
	for(proxy_t *t=kuftpGlobalSettings.pl_proxy.first();t;t=kuftpGlobalSettings.pl_proxy.next())
	{
		proxyComboBox->insertItem(t->name);
	}
	generalLayout=new QGridLayout(generalTab,2,2,5);
	generalLayout->addMultiCellWidget(historyLabel,0,0,0,0);
	generalLayout->addMultiCellWidget(historyComboBox,0,0,1,2);
	generalLayout->addMultiCellWidget(clearHistoryButton,0,0,3,3);
	generalLayout->addWidget(hostLabel,1,0);
	generalLayout->addWidget(hostLineEdit,1,1);
	generalLayout->addWidget(portLabel,1,2);
	generalLayout->addWidget(portSpinBox,1,3);
	generalLayout->addMultiCellWidget(anonymousCheckBox,2,2,0,0);
	generalLayout->addMultiCellWidget(usernameLabel,3,3,0,0);
	generalLayout->addMultiCellWidget(usernameLineEdit,3,3,1,3);
	generalLayout->addMultiCellWidget(passwordLabel,4,4,0,0);
	generalLayout->addMultiCellWidget(passwordEdit,4,4,1,3);
	generalLayout->addMultiCellWidget(remotePathLabel,5,5,0,0);
	generalLayout->addMultiCellWidget(remotePathLineEdit,5,5,1,3);
	generalLayout->addMultiCellWidget(proxyLabel,6,6,0,0);
	generalLayout->addMultiCellWidget(proxyComboBox,6,6,1,3);

	advancedTab=new QWidget(this);
	modeCheckBox=new QCheckBox(i18n("Use PASV Mode"),advancedTab);
	modeCheckBox->setChecked(true);
	correctPasvCheckBox=new QCheckBox(i18n("Correct PASV Replies"),advancedTab);
	correctPasvCheckBox->setChecked(false);
	encryptButtonGroup = new QButtonGroup(1,Qt::Horizontal,i18n("Encrypt Session With SSL"),advancedTab);
	//encryptButtonGroup->setExclusive(true);
 	noEncryptRadioButton=new QRadioButton(i18n("Do not Encrypt Session"),encryptButtonGroup);
	noEncryptRadioButton->setChecked(true);
 	encryptControlRadioButton=new QRadioButton(i18n("Encrypt Control Channel only"),encryptButtonGroup);
 	encryptControlAndDirRadioButton=new QRadioButton(i18n("Encrypt Control Channel and Directory List"),encryptButtonGroup);
 	encryptAllRadioButton=new QRadioButton(i18n("Encrypt Everyting"),encryptButtonGroup);
	setEncryptButtonGroupChecked(0);
	codecGroupBox=new QGroupBox(1,Qt::Vertical,i18n("Server Encoding"),advancedTab);
	codecGroupBox->layout()->setMargin(5);
	codecGroupBox->layout()->setSpacing(10);
        codecComboBox=new KComboBox(codecGroupBox);
	codecComboBox->insertStringList(KGlobal::charsets()->descriptiveEncodingNames());
	codecComboBox->setCurrentText(kuftpGlobalSettings.getDescriptiveEncoding());
	spacer=new QSpacerItem(20,20,QSizePolicy::Minimum,QSizePolicy::Expanding);
	advancedLayout=new QGridLayout(advancedTab,5,1,5);
	advancedLayout->addWidget(modeCheckBox,0,0);
	advancedLayout->addWidget(correctPasvCheckBox,1,0);
	advancedLayout->addWidget(encryptButtonGroup,2,0);
	advancedLayout->addWidget(codecGroupBox,3,0);
	advancedLayout->addItem(spacer);

	addTab(generalTab,i18n("General"));
	addTab(advancedTab,i18n("Advanced"));

	setOkButton(i18n("Connect"));
	setCancelButton();
	hostLineEdit->setFocus();
	setTabOrder(hostLineEdit,anonymousCheckBox);
	setTabOrder(anonymousCheckBox,usernameLineEdit);
	setTabOrder(usernameLineEdit,passwordEdit);
	
	readHistory();
	int index=0;
	for(historyItem_t *p=pl_historyItem.first();p;p=pl_historyItem.next())
	{
		historyComboBox->insertItem(p->ptr->getSiteName());
		p->index=index;
		index++;
	}
		
}

KuFtpConnectDialog::~KuFtpConnectDialog()
{
	for(historyItem_t *p=pl_historyItem.first();p;p=pl_historyItem.next())
	{
		if(p->ptr->getHost()==hostLineEdit->text() && (int)(p->ptr->getPort())==portSpinBox->value()
				&& p->ptr->getUsername()==usernameLineEdit->text()
				&& p->ptr->getPassword()==passwordEdit->password()
//				&& p->ptr->getProxyIndex()==proxyComboBox->currentItem()
				&& p->ptr->getEncryptMethod()==getEncryptMethod()
				&& p->ptr->getIsPasv()==modeCheckBox->isChecked()
				&& p->ptr->getEncoding()==codecComboBox->currentText())
			return;
	}
	KuFtpSiteInfo *t=new KuFtpSiteInfo(hostLineEdit->text(),hostLineEdit->text(),portSpinBox->value(),
			usernameLineEdit->text(),passwordEdit->password(),remotePathLineEdit->text(),0,
			proxyComboBox->currentItem(),getEncryptMethod(),modeCheckBox->isChecked(),
			correctPasvCheckBox->isChecked(),codecComboBox->currentText());
	if(pl_historyItem.count()==50)
	{
		pl_historyItem.removeLast();
	}
	historyItem_t *ht=new historyItem_t;
	ht->ptr=t;
	ht->index=0;
	pl_historyItem.prepend(ht);
	writeHistory();
}

int KuFtpConnectDialog::getEncryptMethod()
{
	if(noEncryptRadioButton->isChecked())
	{
		return 0;
	}
	else if(encryptControlRadioButton->isChecked())
	{
		return 1;
	}
	else if(encryptControlAndDirRadioButton->isChecked())
	{
		return 2;
	}
	else
	{
		return 3;
	}
}

void KuFtpConnectDialog::setEncryptButtonGroupChecked(int encryptMethod)
{
	if(encryptMethod==0)
	{
		noEncryptRadioButton->setChecked(true);
	}
	else if(encryptMethod==1)
	{
		encryptControlRadioButton->setChecked(true);
	}
	else if(encryptMethod==2)
	{
		encryptControlAndDirRadioButton->setChecked(true);
	}
	else
	{
		encryptAllRadioButton->setChecked(true);
	}
}

void KuFtpConnectDialog::SLOT_anonymousCheckBoxClicked()
{
	if(anonymousCheckBox->isChecked())
	{
		usernameLineEdit->setText("anonymous");
		passwordEdit->erase();
		passwordEdit->insert(kuftpGlobalSettings.getEmail());
		usernameLineEdit->setEnabled(false);
		passwordEdit->setEnabled(false);
	}
	else
	{
		usernameLineEdit->clear();
		passwordEdit->erase();
		usernameLineEdit->setEnabled(true);
		passwordEdit->setEnabled(true);
	}
}

void KuFtpConnectDialog::SLOT_parseHostLineEditText()
{
	QString s=hostLineEdit->text();
	s.stripWhiteSpace();
	if(!s.startsWith("ftp://",false))
	{
		s.prepend("ftp://");
	}
	QUrl url(s);
	hostLineEdit->setText(url.host());
	int port=url.port();
	if(port<=0||port>65535)
	{
		port=21;
	}
	portSpinBox->setValue(port);
	if(url.hasUser())
	{
		usernameLineEdit->setText(url.user());
	}
	if(url.hasPassword())
	{
		passwordEdit->erase();
		passwordEdit->insert(url.password());
	}
	if(url.hasPath())
	{
		remotePathLineEdit->setText(url.path());
	}
}

void KuFtpConnectDialog::SLOT_clearHistory()
{
	historyComboBox->clear();
	pl_historyItem.clear();
	QFile::remove(QDir::homeDirPath().append("/.kuftp/history.xml"));
}

void KuFtpConnectDialog::SLOT_historyItemSelectChanged(int index)
{
	for(historyItem_t *p=pl_historyItem.first();p;p=pl_historyItem.next())
	{
		if(p->index==index)
		{
			hostLineEdit->setText(p->ptr->getHost());
			usernameLineEdit->setText(p->ptr->getUsername());
			portSpinBox->setValue(p->ptr->getPort());
			passwordEdit->erase();
			passwordEdit->insert(p->ptr->getPassword());
			remotePathLineEdit->setText(p->ptr->getRemotePath());
			proxyComboBox->setCurrentItem(p->ptr->getProxyIndex());
			setEncryptButtonGroupChecked(p->ptr->getEncryptMethod());
			modeCheckBox->setChecked(p->ptr->getIsPasv());
			correctPasvCheckBox->setChecked(p->ptr->getIsCorrectPasv());
			codecComboBox->setCurrentText(p->ptr->getEncoding());
			return;
		}
	}
}	
	
void KuFtpConnectDialog::readHistory()
{
	QDomDocument doc( "KuFtpConnectHistory" );
	QFile file(QDir::homeDirPath().append("/.kuftp/history.xml"));
	if( !file.open( IO_ReadOnly ) ) 
	{
		return ;
	}
	if( !doc.setContent( &file ) )
	{
		file.close();
		return ;
	}
	file.close();

	pl_historyItem.setAutoDelete(true);
	pl_historyItem.clear();
	
	QString siteName;
	QString host;
	unsigned int port=21;
	QString username;
	QString password;
	QString remotePath;
	int proxyIndex=0;
	int encryptMethod=0;
	bool pasv=true;
	bool correctPasv=false;
	QString encoding;
	
	
	QDomElement root = doc.documentElement();
	if( root.tagName() != "History" )
	{
		return ;
	}
	QDomNode topNode = root.firstChild();
	int index=0;
	while( !topNode.isNull() )
	{
		QDomElement topElement = topNode.toElement();
		if( !topElement.isNull() )
		{
			if(topElement.tagName()=="site")
			{
				siteName=topElement.attribute("name","");
				QDomNode siteNode = topElement.firstChild();
				while( !siteNode.isNull() )
				{
					QDomElement siteInfoElement=siteNode.toElement();
					if(!siteInfoElement.isNull())
					{
						if(siteInfoElement.tagName() == "host")
							host=siteInfoElement.text();
						if(siteInfoElement.tagName() == "port")
						{
							port=siteInfoElement.text().toUInt();
							if(port<=0||port>65535)
								port=21;
						}
						if(siteInfoElement.tagName() == "username")
							username=siteInfoElement.text();
						if(siteInfoElement.tagName() == "password")
							password=siteInfoElement.text();
						if(siteInfoElement.tagName() == "remotepath")
							remotePath=siteInfoElement.text();
						if(siteInfoElement.tagName()=="proxyindex")
							proxyIndex=siteInfoElement.text().toInt();
						if(siteInfoElement.tagName()=="encryption")
							encryptMethod=siteInfoElement.text().toInt();
						if(siteInfoElement.tagName()=="pasv")
							pasv=(siteInfoElement.text()=="true"?1:0);
						if(siteInfoElement.tagName()=="correctpasv")
							correctPasv=(siteInfoElement.text()=="true"?1:0);
						if(siteInfoElement.tagName()=="encoding")
							encoding=siteInfoElement.text();
					}
					siteNode=siteNode.nextSibling();
				}
				if(host.simplifyWhiteSpace().isEmpty())
				{
					topNode = topNode.nextSibling();
					continue;
				}
				if(siteName.simplifyWhiteSpace().isEmpty())
					siteName=host;
				KuFtpSiteInfo *siteInfo=new KuFtpSiteInfo(siteName,host,port,username,password,remotePath,0,proxyIndex,encryptMethod,pasv,correctPasv,encoding);
				historyItem_t *ht=new historyItem_t;
				ht->ptr=siteInfo;
				ht->index=index;
				pl_historyItem.append(ht);
				index++;
			}
		}
		topNode = topNode.nextSibling();
	}
}
void KuFtpConnectDialog::writeHistory()
{
	QDomDocument doc( "KuFtpConnectHistory" );
	QDomElement root = doc.createElement( "History" );
	doc.appendChild( root );

	for(historyItem_t *topItem=pl_historyItem.first();topItem;topItem=pl_historyItem.next())
	{
		QDomElement siteElement=doc.createElement("site");
		siteElement.setAttribute("name",topItem->ptr->getSiteName());
		root.appendChild(siteElement);
		QDomText siteInfoText;
		QDomElement siteInfoElement;

		siteInfoElement=doc.createElement("host");
		siteElement.appendChild( siteInfoElement );
		siteInfoText = doc.createTextNode(topItem->ptr->getHost());
		siteInfoElement.appendChild(siteInfoText);

		siteInfoElement=doc.createElement("port");
		siteElement.appendChild( siteInfoElement);
		char port[10];
		sprintf(port,"%d",topItem->ptr->getPort());
		siteInfoText=doc.createTextNode(port);
		siteInfoElement.appendChild(siteInfoText);

		siteInfoElement=doc.createElement("username");
		siteElement.appendChild( siteInfoElement);
		siteInfoText=doc.createTextNode(topItem->ptr->getUsername());
		siteInfoElement.appendChild(siteInfoText);

		siteInfoElement=doc.createElement("password");
		siteElement.appendChild( siteInfoElement);
		siteInfoText=doc.createTextNode(topItem->ptr->getPassword());
		siteInfoElement.appendChild(siteInfoText);	

		siteInfoElement=doc.createElement("remotepath");
		siteElement.appendChild( siteInfoElement);
		siteInfoText=doc.createTextNode(topItem->ptr->getRemotePath());
		siteInfoElement.appendChild(siteInfoText);	

		siteInfoElement=doc.createElement("proxyindex");
		siteElement.appendChild( siteInfoElement);
		char proxyIndex[10];
 		sprintf(proxyIndex,"%d",topItem->ptr->getProxyIndex());
		siteInfoText=doc.createTextNode(proxyIndex);
		siteInfoElement.appendChild(siteInfoText);	

		siteInfoElement=doc.createElement("encryption");
		siteElement.appendChild( siteInfoElement);
		char encryptMethod[10];
 		sprintf(encryptMethod,"%d",topItem->ptr->getEncryptMethod());
		siteInfoText=doc.createTextNode(encryptMethod);
		siteInfoElement.appendChild(siteInfoText);	

		siteInfoElement=doc.createElement("pasv");
		siteElement.appendChild( siteInfoElement);
		if(topItem->ptr->getIsPasv())
			siteInfoText=doc.createTextNode("true");
		else
			siteInfoText=doc.createTextNode("false");
		siteInfoElement.appendChild(siteInfoText);	

		siteInfoElement=doc.createElement("correctpasv");
		siteElement.appendChild( siteInfoElement);
		if(topItem->ptr->getIsCorrectPasv())
			siteInfoText=doc.createTextNode("true");
		else
			siteInfoText=doc.createTextNode("false");
		siteInfoElement.appendChild(siteInfoText);	

		siteInfoElement=doc.createElement("encoding");
		siteElement.appendChild( siteInfoElement);
		siteInfoText=doc.createTextNode(topItem->ptr->getEncoding());
		siteInfoElement.appendChild(siteInfoText);	
	}	
	QFile file(QDir::homeDirPath().append("/.kuftp/history.xml"));
	if( !file.open( IO_WriteOnly ) )
		return;
	QTextStream stream( &file );
	stream << doc.toString();
	file.close();
}		


