/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <qstring.h>
#include <qdatetime.h>
#include <qptrlist.h>
#include <qvaluelist.h>
#include <qevent.h>

#include "kuftpcore.h"
#include "kuftpmain.h"
#include "kuftpqueue.h"

KuFtpCore::KuFtpCore():QObject()
{
	thread=new KuFtpThread();
	installEventFilter(this);
	thread->setEventReceiver(this);
}
KuFtpCore::~KuFtpCore()
{
	delete thread;
}
bool KuFtpCore::event(QEvent *event)
{
	type t=(type)event->type();
	if(t==FTP_TIMEOUT)
	{
		emit
			timeout();
		return true;
	}
	else if(t==FTP_XFERED)
	{
		off64_t *i=static_cast<off64_t *>((static_cast<QCustomEvent *>(event))->data());
		emit
			xferedChanged(*i);
		return true;
	}
	else if(t==FTP_LOG)
	{
		log_t *log=static_cast<log_t *>((static_cast<QCustomEvent *>(event))->data());
		emit
			logChanged(*log);
		return true;
	}
	else if(t==FTP_CONNECTIONLOST)
	{
		emit 
			connectionLost();
		return true;
	}
	retData_t *ret=static_cast<retData_t *>((static_cast<QCustomEvent *>(event))->data());
	emit
	       	signalFromCore(*ret,t);
	return true;
}
void KuFtpCore::startCheckTimeout()
{
	thread->startCheckTimeout();
}
void KuFtpCore::stopCheckTimeout()
{
	thread->stopCheckTimeout();
}
void KuFtpCore::setFtpUseProxy(bool i)
{
	thread->SetFtpUseProxy(i);
}
void KuFtpCore::setNetworkTimeout(int i)
{
	thread->SetNetworkTimeout(i);
}
void KuFtpCore::stopTransfer()
{
	thread->StopTransfer();
}
void KuFtpCore::setMaxDownloadSpeed(int i)
{
	thread->SetMaxDownloadSpeed(i);
}
void KuFtpCore::setMaxUploadSpeed(int i)
{
	thread->SetMaxUploadSpeed(i);
}
void KuFtpCore::connect(QString host,unsigned short port)
{
	task_t t;
	t.host=host;
	t.port=port;
	t.tasktype=FTP_CONNECT;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::connect(QString host,unsigned short port,proxy_t *proxy)
{
	task_t t;
	t.host=host;
	t.port=port;
	t.proxy=proxy;
	t.tasktype=FTP_CONNECT_USE_PROXY;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::auth()
{
	task_t t;
	t.tasktype=FTP_AUTH;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::setEncryptMethod(int encryptMethod)
{
	thread->setEncryptMethod(encryptMethod);
}
void KuFtpCore::setConnMode(connmode cmode)
{
	thread->ConnMode(cmode);
}
void KuFtpCore::correctPasv()
{
	thread->CorrectPasv();
}
void KuFtpCore::login(QString user,QString pass)
{
	task_t t;
	t.user=user;
	t.pass=pass;
	t.tasktype=FTP_LOGIN;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::dir()
{
	task_t t;
//	t.remotepath=remotepath;
//	t.localpath=localpath;
	t.tasktype=FTP_DIR;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::mirror(QStringList mirrorDirs)
{
	task_t t;
	t.tasktype=FTP_MIRROR;
	t.mirrorDirs=mirrorDirs;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::systype()
{
	task_t t;
	t.tasktype=FTP_SYSTYPE;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::feat()
{
	task_t t;
	t.tasktype=FTP_FEAT;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::get(QString localpath,QString remotepath,transfermode tmode,off64_t offset)
{
	task_t t;
	t.localpath=localpath;
	t.remotepath=remotepath;
	t.tmode=tmode;
	t.offset=offset;
	t.tasktype=FTP_GET;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::put(QString localpath,QString remotepath,transfermode tmode,off64_t offset)
{
	task_t t;
	t.remotepath=remotepath;
	t.localpath=localpath;
	t.tmode=tmode;
	t.offset=offset;
	t.tasktype=FTP_PUT;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::chdir(QString remotepath)
{
	task_t t;
	t.remotepath=remotepath;
	t.tasktype=FTP_CHDIR;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::cdup()
{
	task_t t;
	t.tasktype=FTP_CDUP;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::pwd()
{
	task_t t;
	t.tasktype=FTP_PWD;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::mkdir(QString remotepath)
{
	task_t t;
	t.remotepath=remotepath;
	t.tasktype=FTP_MKDIR;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::rmdir(QString remotepath)
{
	task_t t;
	t.remotepath=remotepath;
	t.tasktype=FTP_RMDIR;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::noop()
{
	task_t t;
	t.tasktype=FTP_NOOP;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::sendRawCmd(QString cmd)
{
	task_t t;
	t.cmd=cmd;
	t.tasktype=FTP_RAW;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::sendSiteCmd(QString cmd)
{
	task_t t;
	t.cmd=cmd;
	t.tasktype=FTP_SITE;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::rename(QString remotesrc,QString remotedst)
{
	task_t t;
	t.remotepath=remotesrc;
	t.localpath=remotedst;
	t.tasktype=FTP_RENAME;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::del(QString remotepath)
{
	task_t t;
	t.remotepath=remotepath;
	t.tasktype=FTP_DEL;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::quit()
{
	task_t t;
	t.tasktype=FTP_QUIT;
	thread->vl_task.push_back(t);
	if(!thread->running())
		thread->start();
}
void KuFtpCore::abort()
{
	thread->terminate();
	thread->wait(1000);
	thread->vl_task.clear();
}
void KuFtpCore::clearThreadQueue()
{
	thread->vl_task.clear();
}
void KuFtpCore::setFtpEncoding(QString encoding)
{
	thread->setFtpEncoding(encoding);
}
void KuFtpCore::setFtpCurrentQueue(KuFtpQueue *q)
{
	thread->setFtpCurrentQueue(q);
}
void KuFtpCore::setFtpCurrentLocalPath(QString c)
{
	thread->setFtpCurrentLocalPath(c);
}
void KuFtpCore::setFtpCurrentRemotePath(QString c)
{
	thread->setFtpCurrentRemotePath(c);
}
void KuFtpCore::setFtpDirCacheValueList(QValueList<dirCache_t> *vl_d)
{
	thread->setFtpDirCacheValueList(vl_d);
}


