/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KUFTPCORE_H
#define KUFTPCORE_H

#include <qobject.h>
#include "kuftpthread.h"

class QString;
class QStringList;
class QEvent;

class KuFtpQueue;

class KuFtpCore:public QObject
{
	Q_OBJECT
	public:
		KuFtpCore();
		~KuFtpCore();
	public:
		void connect(QString host,unsigned short port);
		void connect(QString host,unsigned short port,proxy_t *proxy);
		void auth();
		void setEncryptMethod(int encryptMethod);
		void setConnMode(connmode cmode);
		void correctPasv();
		void login(QString user,QString pass);
		void dir();
		void mirror(QStringList mirrorDirs);
		void systype();
		void feat();
		void get(QString localpath,QString remotepath,transfermode mode,off64_t offset=0);
		void put(QString localpath,QString remotepath,transfermode mode,off64_t  offset=0);
		void chdir(QString remotepath);
		void cdup();
		void pwd();
		void mkdir(QString remotepath);
		void rmdir(QString remotepath);
		//void size(QString remotepath,transfermode mode);
		void rename(QString remotesrc,QString remotedst);
		void del(QString remotepath);
		void sendRawCmd(QString cmd);
		void sendSiteCmd(QString cmd);
		void quit();
		void abort();
		void noop();
		void clearThreadQueue();
		void startCheckTimeout();
		void stopCheckTimeout();
		void stopTransfer();
		void setFtpUseProxy(bool);
		void setMaxDownloadSpeed(int i);
		void setMaxUploadSpeed(int i);
		void setFtpEncoding(QString encoding);
		void setNetworkTimeout(int seconds);
		void setFtpCurrentQueue(KuFtpQueue *q);
		void setFtpCurrentLocalPath(QString c);
		void setFtpCurrentRemotePath(QString c);
		void setFtpDirCacheValueList(QValueList<dirCache_t> *vl_d);

	private:
		KuFtpThread *thread;
		bool event(QEvent *event);
	signals:
		void xferedChanged(off64_t);
		void logChanged(log_t);
		void signalFromCore(retData_t,type);
		void timeout();
		void connectionLost();
};


#endif

