/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*   original copyright notes */
/***************************************************************************
 *   copyright            : (C) 2003 by mkulke
 *   email                : sikor_sxe@radicalapproach.de
 *
 *   KuFtpLib.c - callable ftp access routines
 *   Copyright (C) 1996, 1997, 1998 Thomas Pfau, pfau@cnj.digex.net
 *   73 Catherine Street, South Bound Brook, NJ, 08880
 ***************************************************************************/


// enable > 2gb support (LFS)

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>

#include "kuftplib.h"

#define net_read read
#define net_write write
#define net_close close

using namespace std;

// socket values 
#define SETSOCKOPT_OPTVAL_TYPE (void *)
#define FTPLIB_BUFSIZ 1024
#define ACCEPT_TIMEOUT 30

//io types 
#define FTPLIB_CONTROL 0
#define FTPLIB_READ 1
#define FTPLIB_WRITE 2

#define HTTP_PROXY 1
#define SOCKS4_PROXY 2
#define SOCKS5_PROXY 3

#define INLOG 0
#define OUTLOG 1
#define COMMONLOG 2
#define EMERLOG 3

KuFtpLib::KuFtpLib()
{
	SSL_library_init();

	mp_ftphandle = static_cast<ftphandle *>(calloc(1,sizeof(ftphandle)));
	if(mp_ftphandle == NULL) 
	{
		//perror("calloc error");
		return;
	}
	mp_ftphandle->buf = static_cast<char *>(malloc(FTPLIB_BUFSIZ));
	if(mp_ftphandle->buf == NULL)
	{
		//perror("calloc error");
		free(mp_ftphandle);
	}
	mp_ftphandle->ctx = SSL_CTX_new(SSLv3_method());
	SSL_CTX_set_verify(mp_ftphandle->ctx, SSL_VERIFY_NONE, NULL);
	mp_ftphandle->ssl = SSL_new(mp_ftphandle->ctx);
	ClearHandle();
}
KuFtpLib::~KuFtpLib()
{
	SSL_free(mp_ftphandle->ssl);
	SSL_CTX_free(mp_ftphandle->ctx);
	free(mp_ftphandle->buf);
	free(mp_ftphandle);
}

/*
 * socket_wait - wait for socket to receive or flush data
 *
 * return 1 if no user callback, otherwise, return value returned by
 * user callback
 */
int KuFtpLib::socket_wait(ftphandle *ctl)
{
	fd_set fd,*rfd = NULL,*wfd = NULL;
	struct timeval tv;
	int rv = 0;

	if(ctl->idlecb == NULL)
	       	return 1;

	/*if ((ctl->dir == FTPLIB_CONTROL)
		|| (ctl->idlecb == NULL)
		|| ((ctl->idletime.tv_sec == 0)
		&& //(ctl->idletime.tv_usec 0))
	return 1;*/

	if(ctl->dir == FTPLIB_WRITE) 
		wfd = &fd;
	else 
		rfd = &fd;

	FD_ZERO(&fd);
//	do
//	{
		FD_SET(ctl->handle,&fd);
		tv = ctl->idletime;
		rv = select(ctl->handle+1, rfd, wfd, NULL, &tv);
		if (rv == -1)
		{
			rv = 0;
			//strncpy(ctl->ctrl->response, strerror(errno), sizeof(ctl->ctrl->response));
			//break;
		}
		else if (rv > 0)
		{
			rv = 1;
			//break;
		}
		else if(rv==0)//timeout
		{
			mp_ftphandle->timeout=true;
			//printf("%s\n",strerror(errno));
			net_close(ctl->handle);
			if(mp_ftphandle!=ctl)
				net_close(mp_ftphandle->handle);
			//ctl->idlecb(ctl->cbarg);
		}
//	} while ((rv = ctl->idlecb(ctl->cbarg)));

	return rv;
}

/*
 * read a line of text
 *
 * return -1 on error or bytecount
 */
int KuFtpLib::readline(char *buf,int max,ftphandle *ctl)
{
	int x,retval = 0;
	char *end,*bp=buf;
	int eof = 0;

	if ((ctl->dir != FTPLIB_CONTROL) && (ctl->dir != FTPLIB_READ))
		return -1;
	if (max == 0)
		return 0;
	do
	{
		if (ctl->cavail > 0)
		{
			x = (max >= ctl->cavail) ? ctl->cavail : max-1;
			end = static_cast<char*>(memccpy(bp,ctl->cget,'\n',x));
			if (end != NULL)
				x = end - bp;
			retval += x;
			bp += x;
			*bp = '\0';
			max -= x;
			ctl->cget += x;
			ctl->cavail -= x;
			if (end != NULL)
			{
				bp -= 2;
				if (strcmp(bp,"\r\n") == 0)
				{
					*bp++ = '\n';
					*bp++ = '\0';
					--retval;
				}
				break;
			}
		}
		if (max == 1)
		{
			*buf = '\0';
			break;
		}
		if (ctl->cput == ctl->cget)
		{
			ctl->cput = ctl->cget = ctl->buf;
			ctl->cavail = 0;
			ctl->cleft = FTPLIB_BUFSIZ;
		}
		if (eof)
		{
			if (retval == 0)
			retval = -1;
			break;
		}

		if (!socket_wait(ctl))
		       	return retval;

		if (ctl->tlsdata) x = SSL_read(ctl->ssl, ctl->cput, ctl->cleft);
		else
		{
			if (ctl->tlsctrl) 
				x = SSL_read(ctl->ssl, ctl->cput, ctl->cleft);
			else 
				x = net_read(ctl->handle,ctl->cput,ctl->cleft);
		}
		if ( x == -1)
		{
			//perror("read error");
			retval = -1;
			break;
		}

		// LOGGING FUNCTIONALITY!!!

		if ((ctl->dir == FTPLIB_CONTROL) && (mp_ftphandle->logcb != NULL))
		{
			*((ctl->cput)+x) = '\0';
			mp_ftphandle->logcb(ctl->cput, mp_ftphandle->cbarg, OUTLOG);
		}

		if (x == 0) eof = 1;
		ctl->cleft -= x;
		ctl->cavail += x;
		ctl->cput += x;
	} while (1);
	return retval;
}

/*
 * write lines of text
 *
 * return -1 on error or bytecount
 */
int KuFtpLib::writeline(char *buf, int len, ftphandle *nData)
{
	int x, nb=0, w;
	char *ubp = buf, *nbp;
	char lc=0;

	if (nData->dir != FTPLIB_WRITE)
		return -1;
	nbp = nData->buf;
	for (x=0; x < len; x++)
	{
		if ((*ubp == '\n') && (lc != '\r'))
		{
			if (nb == FTPLIB_BUFSIZ)
			{
				if (!socket_wait(nData))
				       	return x;
				if (nData->tlsctrl) 
					w = SSL_write(nData->ssl, nbp, FTPLIB_BUFSIZ);
				else 
					w = net_write(nData->handle, nbp, FTPLIB_BUFSIZ);
				if (w != FTPLIB_BUFSIZ)
				{
					//printf("write(1) returned %d, errno = %d\n", w, errno);
					return(-1);
				}
				nb = 0;
			}
			nbp[nb++] = '\r';
		}
		if (nb == FTPLIB_BUFSIZ)
		{
			if (!socket_wait(nData))
				return x;
			if (nData->tlsctrl) 
				w = SSL_write(nData->ssl, nbp, FTPLIB_BUFSIZ);
			else 
				w = net_write(nData->handle, nbp, FTPLIB_BUFSIZ);
			if (w != FTPLIB_BUFSIZ)
			{
				//printf("write(2) returned %d, errno = %d\n", w, errno);
				return(-1);
			}
			nb = 0;
		}
		nbp[nb++] = lc = *ubp++;
	}
	if (nb)
	{
		if (!socket_wait(nData))
		       	return x;
		if (nData->tlsctrl)
		       	w = SSL_write(nData->ssl, nbp, nb);
		else 
			w = net_write(nData->handle, nbp, nb);
		if (w != nb)
		{
			//printf("write(3) returned %d, errno = %d\n", w, errno);
			return(-1);
		}
	}
	return len;
}

/*
 * read a response from the server
 *
 * return 0 if first char doesn't match
 * return 1 if first char matches
 */
int KuFtpLib::readresp(char c, ftphandle *nControl)
{
	char match[5];
	
	if (readline(nControl->response,256,nControl) == -1)
	{
		//perror("Control socket read failed");
		return 0;
	}
	
	if (nControl->response[3] == '-')
	{
		strncpy(match,nControl->response,3);
		match[3] = ' ';
		match[4] = '\0';
		do
		{
			if (readline(nControl->response,256,nControl) == -1)
			{
				//perror("Control socket read failed");
				return 0;
			}
		} while (strncmp(nControl->response,match,4));
	}
	if (nControl->response[0] == c)
	       	return 1;
	return 0;
}
int KuFtpLib::Timeout()
{
        fd_set rset;
        FD_ZERO(&rset);
        FD_SET(mp_ftphandle->handle,&rset);
	struct timeval tv={0,0};
        if(select(mp_ftphandle->handle+1,&rset,NULL,NULL,&tv))
	{
		if(mp_ftphandle->checkTimeout)
		{
			if(readline(mp_ftphandle->response,256,mp_ftphandle)== -1 ) //not timeout
				return 0;
			if(strncmp(mp_ftphandle->response,"421",3)==0)//timeout
				return 1;
		}
	}
	return 0;
}
int KuFtpLib::ConnectionLost()
{
	return mp_ftphandle->timeout;
}
/*
 * FtpLastResponse - return a pointer to the last response received
 */
char* KuFtpLib::LastResponse()
{
	if ((mp_ftphandle) && (mp_ftphandle->dir == FTPLIB_CONTROL)) 
		return mp_ftphandle->response;
	return NULL;
}

/*
 * KuFtpLib::Connect - connect to remote server
 *
 * return 1 if connected, 0 if not
 */
int KuFtpLib::Connect(const char *host,unsigned short port)
{
	int sControl;
	struct sockaddr_in sin;
	struct hostent *phe;
	//struct servent *pse;
	int on=1;
	int ret;
	
	mp_ftphandle->dir = FTPLIB_CONTROL;
	mp_ftphandle->ctrl = NULL;
	mp_ftphandle->xfered = 0;
	mp_ftphandle->xfered1 = 0;
	mp_ftphandle->tlsctrl = 0;
	mp_ftphandle->tlsdata = 0;
	mp_ftphandle->offset = 0;
	mp_ftphandle->handle = 0;
	mp_ftphandle->maxDownloadSpeed=0;
	mp_ftphandle->maxUploadSpeed=0;
	mp_ftphandle->checkTimeout=false;
	mp_ftphandle->timeout=false;
	
	memset(&sin,0,sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_port = htons(port);

/*	lhost = strdup(host);
	pnum = strchr(lhost,':');
	if (pnum == NULL)
	{
		if ((pse = getservbyname("ftp","tcp")) == NULL)
		{
			//perror("getservbyname error");
			return 0;
		}
		sin.sin_port = pse->s_port;
	}
	else
	{
		*pnum++ = '\0';
		if (isdigit(*pnum)) 
			sin.sin_port = htons(atoi(pnum));
		else
		{
			pse = getservbyname(pnum,"tcp");
			sin.sin_port = pse->s_port;
		}
	}*/
	

	ret = inet_aton(host, &sin.sin_addr);
	if (ret == 0)
	{
		if ((phe = gethostbyname(host)) == NULL)
		{
			//perror("gethostbyname error");
			return 0;
		}
		memcpy((char *)&sin.sin_addr, phe->h_addr, phe->h_length);
	}
	
	sControl = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (sControl == -1)
	{
		//perror("socket error");
		return 0;
	}
	
	if (setsockopt(sControl,SOL_SOCKET,SO_REUSEADDR, SETSOCKOPT_OPTVAL_TYPE &on, sizeof(on)) == -1)
	{
		//perror("setsockopt error");
		net_close(sControl);
		return 0;
	}
	if (connect(sControl, (struct sockaddr *)&sin, sizeof(sin)) == -1)
	{
		//perror("connect error");
		net_close(sControl);
		return 0;
	}

	mp_ftphandle->handle = sControl;

	if(mp_ftphandle->useProxy==false)
	{
		if (readresp('2', mp_ftphandle) == 0)
		{
			net_close(sControl);
			mp_ftphandle->handle = 0;
			return 0;
		}
	}

	return 1;
}

int KuFtpLib::Connect(const char *host,unsigned short port,const char *proxyHost,unsigned short proxyPort,const char *proxyUser,const char *proxyPass,int proxyType)
{
      	if(!Connect(proxyHost,proxyPort))
     	{
   		return 0;
  	}
	delete mp_ftphandle->proxyHost;
	delete mp_ftphandle->proxyUser;
	delete mp_ftphandle->proxyPass;
	mp_ftphandle->proxyHost=new char[strlen(proxyHost)+1];
	memcpy(mp_ftphandle->proxyHost,proxyHost,strlen(proxyHost));
	mp_ftphandle->proxyHost[strlen(proxyHost)]=0;
	mp_ftphandle->proxyUser=new char[strlen(proxyUser)+1];
	memcpy(mp_ftphandle->proxyUser,proxyUser,strlen(proxyUser));
	mp_ftphandle->proxyUser[strlen(proxyUser)]=0;
	mp_ftphandle->proxyPass=new char[strlen(proxyPass)+1];
	memcpy(mp_ftphandle->proxyPass,proxyPass,strlen(proxyPass));
	mp_ftphandle->proxyPass[strlen(proxyPass)]=0;
	mp_ftphandle->proxyPort=proxyPort;
	mp_ftphandle->proxyType=proxyType;

	int success=0;
	if(proxyType==HTTP_PROXY)
	{
		success=DoHttpProxy(proxyUser,proxyPass,host,port);
	}
	else if(proxyType==SOCKS4_PROXY)
	{
		success=DoSocks4Proxy(host,port);
	}
	else
	{
		success=DoSocks5Proxy(proxyUser,proxyPass,host,port);
	}
	if(!success)
		return 0;
	char *log="Connected with proxy\r\n";
	mp_ftphandle->logcb(log,mp_ftphandle->cbarg,OUTLOG);
        if (readresp('2', mp_ftphandle) == 0)
	{
		net_close(mp_ftphandle->handle);
		mp_ftphandle->handle = 0;
		return 0;
	}
	return 1;
}
void KuFtpLib::base64Encode (const char *string, char *encoded)
{
        char base64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
        int i, length;
        length = strlen (string);
        for (i = 0; i < length; i += 3)
        {
                *encoded++ = base64[string[0] >> 2];
                *encoded++ = base64[((string[0] & 3) << 4) + (string[1] >> 4)];
                *encoded++ = base64[((string[1] & 0xf) << 2) + (string[2] >> 6)];
                *encoded++ = base64[string[2] & 0x3f];
                string += 3;
        }
        if (i == length + 1)
                *(encoded - 1) = '=';
        else if (i == length + 2)
                *(encoded - 1) = *(encoded - 2) = '=';
        *encoded = '\0';
}
int KuFtpLib::ReadCrLfLine(int fd,void *buf,int size)
{
	int count=0;
	unsigned char cr=0;
	char *buffer=(char *)buf;
/*
	fd_set rset;
	FD_ZERO(&rset);
	struct timeval timeout={5,0};
	FD_SET(sockfd,&rset);
	maxfd=sockfd+1;
	switch(select(maxfd,&rset,NULL,NULL,&timeout)){*/
	while (1)
	{
		read (fd, &buffer[count], 1);
		if ((buffer[count] == '\n') && (cr== '\r'))
			break;
		if (count == size)
		{
			return -1;
		}
		cr = buffer[count];
		count++;
	}
	buffer[count+1]=0;
	return count+1;
}
void KuFtpLib::LogHttpProxyReturn(int fd)
{
	char buffer[1024];
	while(1)
	{
		memset(buffer,0,1024);
		int n=0;
		int count=0;
		unsigned char cr=0;
		for(count=0;count<1024;count++)
		{
			if((n=read(fd,&cr, 1))==1)
			{
				buffer[count]=cr;
				if(cr=='\n')
					break;
			}
		}
		if(n==0)
			break;
		if(count==1024)
		{
			buffer[1022]='\r';
			buffer[1023]='\n';
		}
		else if(buffer[strlen(buffer)-2]!='\r')
		{
			buffer[strlen(buffer)-1]='\r';
			buffer[strlen(buffer)]='\n';
		}
		mp_ftphandle->logcb(buffer,mp_ftphandle->cbarg,OUTLOG);
	}
}
int KuFtpLib::DoHttpProxy(const char *proxyUser,const char *proxyPass,const char *ftpHost,unsigned short ftpPort,int Fd)
{
	char string[256]={0};
	char port[6]={0};
	sprintf(port,"%d",ftpPort);
	strcpy (string, "CONNECT ");
	strncat (string, ftpHost,strlen(ftpHost));
	strcat (string, ":");
	strncat (string, port,strlen(port));
	strcat (string, " HTTP/1.1\r\nHost: ");
	strncat (string, ftpHost,strlen(ftpHost));
	strcat (string, ":");
	strncat (string, port,strlen(port));
	strcat (string, "\r\n");

	if(strlen(proxyUser)!=0||strlen(proxyPass)!=0)
	{
		char auth[257]={0};
		char authBase64[257]={0};
		strcat (auth, proxyUser);
		strcat (auth, ":");
		strcat (auth, proxyPass);
		base64Encode (auth, authBase64);
		strcat (string, "Proxy-authorization: Basic ");
		strcat (string, authBase64);
		strcat (string, "\r\n");
	}
	strcat (string, "\r\n");
	int fd=0;
	if(Fd==0)
	{
		fd=mp_ftphandle->handle;
	}
	else
	{
		fd=Fd;
	}
	write(fd,string,strlen(string));

	int count=0;
	char buffer[1024]={0};
	if((count=ReadCrLfLine(fd,buffer,1024))<0)
	{
		return 0;
	}
	char code[4]={0};
	if (!memcmp(buffer,"HTTP",4))
	{
		for (count = 0; buffer[count] != ' '; count++)
			if (count == 1024)
				break;
		if (count < 1024)
		{
			memcpy (code, &buffer[count + 1], 3);
			code[3] = 0;
		}
	}
	else
	{
		LogHttpProxyReturn(fd);
		return 0;
	}
	if (!strcmp(code, "200")) //proxy established, throw proxy return until <cr><lf><cr><lf>
	{
		while(memcmp(buffer,"\r\n",2))
		{
			if(ReadCrLfLine(fd,buffer,1024) < 0)
		        {
      				return -1;
			}
		}
	}
	else//proxy error, log proxy return
	{
		LogHttpProxyReturn(fd);
		return 0;
	}
	return 1;
}
void KuFtpLib::LogSocks5ProxyError(int error)
{
	char log[256]="SOCKS5 Server Error: ";
	switch(error)
	{
		case 1: 
			strcat(log,"general SOCKS server failure\r\n");
			break;
		case 2:
			strcat(log,"connection not allowed by ruleset\r\n");
			break;
		case 3:
			strcat(log,"Network unreachable\r\n");
			break;
		case 4:
			strcat(log,"Host unreachable\r\n");
			break;
		case 5:
			strcat(log,"Connection refused\r\n");
			break;
		case 6:
			strcat(log,"TTL expired\r\n");
			break;
		case 7:
			strcat(log,"Command not supported\r\n");
			break;
		case 8:
			strcat(log,"Address type not supported\r\n");
			break;
		default:
			strcat(log,"unkown error\r\n");
			break;
	}
	mp_ftphandle->logcb(log,mp_ftphandle->cbarg,EMERLOG);
}

int KuFtpLib::DoSocks4Proxy(const char *ftpHost,unsigned short ftpPort,int Fd)
{
	char socks4Request[9];
        socks4Request[0]=0x04;
        socks4Request[1]=0x01;
        *(unsigned short*)(socks4Request+2)=ntohs(ftpPort);
	struct hostent *phe=NULL;
	if ((phe = gethostbyname(ftpHost)) == NULL)
	{
		//perror("gethostbyname error");
		return 0;
	}
        unsigned long host=((in_addr *)(phe->h_addr))->s_addr;
        *(unsigned long*)(socks4Request+4)=host;
        socks4Request[8]=0;
	int fd=0;
	if(Fd==0)
	{
		fd=mp_ftphandle->handle;
	}
	else
	{
		fd=Fd;
	}
        if(write(fd,socks4Request,9)==-1)
        {
                //printf("write error \n");
                return 0;
        }
        char socks4Recv[8];
        memset(socks4Recv,0,8);
        int n=0;
        if((n=read(fd,socks4Recv,8))!=8)
        {
                //printf("read error\n");
                return 0;
        }
        if(socks4Recv[0]!=0||socks4Recv[1]!=90)
        {
                char log[256]="SOCKS4 Server Error: ";
		if(socks4Recv[1]==91)
		{
			strcat(log,"request rejected or failed\r\n");
		}
		else if(socks4Recv[1]==92)
		{
			strcat(log,"request rejected becasue SOCKS server cannot connect to identd on the client\r\n");
		}
		else if(socks4Recv[1]==93)
		{
			strcat(log,"request rejected because the client program and identd report different user-ids\r\n");
		}
		else
		{
			strcat(log,"unkown error\r\n");
		}
		mp_ftphandle->logcb(log,mp_ftphandle->cbarg,EMERLOG);
                return 0;
        }
	return 1;
}

int KuFtpLib::DoSocks5Proxy(const char *proxyUser,const char *proxyPass,const char *ftpHost,unsigned short ftpPort,int Fd)
{
	char socks5Request1[4];
        socks5Request1[0]=0x05;
        socks5Request1[1]=0x02;
        socks5Request1[2]=0x00;
        socks5Request1[3]=0x02;

	int fd=0;
	if(Fd==0)
	{
		fd=mp_ftphandle->handle;
	}
	else
	{
		fd=Fd;
	}
        if(write(fd,socks5Request1,4)==-1)
        {
       //         printf("write error \n");
                return 0;
        }
        char socks5Recv1[8];
        memset(socks5Recv1,0,8);
        int n=0;
        n=read(fd,socks5Recv1,8);
        if(socks5Recv1[0]!=5||(socks5Recv1[1]!=0&&socks5Recv1[1]!=2))
        {
                char log[256]="SOCKS5 Server Error: Method not support\r\n";//should log proxy logs;
		mp_ftphandle->logcb(log,mp_ftphandle->cbarg,EMERLOG);
                return 0;
        }
	if(socks5Recv1[1]==2)//need user/pass
        {
                const char *user=proxyUser;
                const char *pass=proxyPass;
                char *socks5AuthReq=(char *)malloc(1+1+strlen(user)+1+strlen(pass)+1);
                socks5AuthReq[0]=1;
                socks5AuthReq[1]=(char)strlen(user);
                memcpy(&socks5AuthReq[2],user,strlen(user));
                socks5AuthReq[2+strlen(user)]=(char)strlen(pass);
                memcpy(&socks5AuthReq[3+strlen(user)],pass,strlen(pass));
                socks5AuthReq[3+strlen(user)+strlen(pass)]=0;
                if(write(fd,socks5AuthReq,strlen(socks5AuthReq))==-1)
                {
//                        printf("write error \n");
                        return 0;
                }
                char socks5AuthRecv[8];
                memset(socks5AuthRecv,0,8);
                int n=0;
                n=read(fd,socks5AuthRecv,8);
                if(socks5AuthRecv[0]!=1||socks5AuthRecv[1]!=0)
                {
                	char log[256]="SOCKS5 Server Error: authentication error\r\n";
			mp_ftphandle->logcb(log,mp_ftphandle->cbarg,EMERLOG);
                        return 0;
                }
        }
	char socks5Request2[10];
        socks5Request2[0]=0x05;
        socks5Request2[1]=0x01;
        socks5Request2[2]=0x00;
        socks5Request2[3]=0x01;
	struct hostent *phe;
	if ((phe = gethostbyname(ftpHost)) == NULL)
	{
		//perror("gethostbyname error");
		return 0;
	}
        unsigned long host=((in_addr *)(phe->h_addr))->s_addr;
        *(unsigned long*)(socks5Request2+4)=host;
        *(unsigned short*)(socks5Request2+8)=ntohs(ftpPort);

        if(write(fd,socks5Request2,10)==-1)
        {
//                printf("write error \n");
                return 0;
        }
        char socks5Recv2[10];
        memset(socks5Recv2,0,10);
        n=read(fd,socks5Recv2,10);
        if(socks5Recv2[0]!=5 || socks5Recv2[1] != 0)
        {
		LogSocks5ProxyError(socks5Recv2[1]);
		return 0;
        }
	return 1;
}

/*
 * FtpSendCmd - send a command and wait for expected response
 *
 * return 1 if proper response received, 0 otherwise
 */
int KuFtpLib::FtpSendCmd(const char *cmd, char expresp, ftphandle *nControl)
{
	char buf[256];
	int x;

	if (!nControl->handle)
	       	return 0;
	if (nControl->dir != FTPLIB_CONTROL)
	       	return 0;
	sprintf(buf,"%s\r\n",cmd);

	if (nControl->tlsctrl)
	       	x = SSL_write(nControl->ssl,buf,strlen(buf));
	else 
		x = net_write(nControl->handle,buf,strlen(buf));
	if (x <= 0)
	{
		//perror("FtpSendCmd write error");
		return 0;
	}

	if (mp_ftphandle->logcb != NULL) 
		mp_ftphandle->logcb(buf, mp_ftphandle->cbarg, INLOG);
	
	return readresp(expresp, nControl);
}

/*
 * FtpLogin - log in to remote server
 *
 * return 1 if logged in, 0 otherwise
 */
int KuFtpLib::Login(const char *user, const char *pass)
{
	char tempbuf[64];

	if (((strlen(user) + 7) > sizeof(tempbuf)) || ((strlen(pass) + 7) > sizeof(tempbuf))) 
		return 0;
	sprintf(tempbuf, "USER %s", user);
	if (!FtpSendCmd(tempbuf,'3',mp_ftphandle))
	{
		if (mp_ftphandle->ctrl != NULL) 
			return 1;
		if (*LastResponse() == '2') 
			return 1;
		return 0;
	}
	sprintf(tempbuf,"PASS %s",pass);
	int success=FtpSendCmd(tempbuf,'2',mp_ftphandle);
	if(!success)
	{
		net_close(mp_ftphandle->handle);
	}
	return success;
}

/*
 * FtpAcceptConnection - accept connection from server
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::FtpAcceptConnection(ftphandle *nData, ftphandle *nControl)
{
	int sData;
	struct sockaddr addr;
	socklen_t l;
	int i;
	struct timeval tv;
	fd_set mask;
	int rv = 0;

	FD_ZERO(&mask);
	FD_SET(nControl->handle, &mask);
	FD_SET(nData->handle, &mask);
	tv.tv_usec = 0;
	tv.tv_sec = ACCEPT_TIMEOUT;
	i = nControl->handle;
	if (i < nData->handle) 
		i = nData->handle;
	i = select(i+1, &mask, NULL, NULL, &tv);

	if (i == -1)
	{
		strncpy(nControl->response, strerror(errno), sizeof(nControl->response));
		net_close(nData->handle);
		nData->handle = 0;
		rv = 0;
	}
	else if (i == 0)
	{
		strcpy(nControl->response, "timed out waiting for connection");
		net_close(nData->handle);
		nData->handle = 0;
		rv = 0;
	}
	else
	{
		if (FD_ISSET(nData->handle, &mask))
		{
			l = sizeof(addr);
			sData = accept(nData->handle, &addr, &l);
			i = errno;
			net_close(nData->handle);
			if (sData > 0)
			{
				rv = 1;
				nData->handle = sData;
				nData->ctrl = nControl;
			}
			else
			{
				strncpy(nControl->response, strerror(i), sizeof(nControl->response));
				nData->handle = 0;
				rv = 0;
			}
		}
		else if (FD_ISSET(nControl->handle, &mask))
		{
			net_close(nData->handle);
			nData->handle = 0;
			readresp('2', nControl);
			rv = 0;
		}
	}
	return rv;
}

/*
 * FtpAccess - return a handle for a data stream
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::FtpAccess(const char *path, accesstype type, transfermode mode, ftphandle *nControl, ftphandle **nData)
{
	char buf[256];
	int dir, ret;

	if ((path == NULL) && ((type == KuFtpLib::filewrite)
		|| (type == KuFtpLib::fileread)
		|| (type == KuFtpLib::filereadappend)
		|| (type == KuFtpLib::filewriteappend)))
	{
		sprintf(nControl->response,"Missing path argument for file transfer\n");
		return 0;
	}
	sprintf(buf, "TYPE %c", mode);
	if (!FtpSendCmd(buf, '2', nControl)) return 0;

	switch (type)
	{
	case KuFtpLib::dir:
		strcpy(buf,"NLST");
		dir = FTPLIB_READ;
		break;
	case KuFtpLib::dirverbose:
		strcpy(buf,"LIST -aL");
		dir = FTPLIB_READ;
		break;
	case KuFtpLib::filereadappend:
	case KuFtpLib::fileread:
		strcpy(buf,"RETR");
		dir = FTPLIB_READ;
		break;
	case KuFtpLib::filewriteappend:
	case KuFtpLib::filewrite:
		strcpy(buf,"STOR");
		dir = FTPLIB_WRITE;
		break;
	default:
		sprintf(nControl->response, "Invalid open type %d\n", type);
		return 0;
	}
	if (path != NULL)
	{
		int i = strlen(buf);
		buf[i++] = ' ';
		if ((strlen(path) + i) >= sizeof(buf)) 
			return 0;
		strcpy(&buf[i],path);
	}

	if (nControl->cmode == KuFtpLib::pasv)
	{
		if (FtpOpenPasv(nControl, nData, mode, dir, buf) == -1) 
			return 0;
	}

	if (nControl->cmode == KuFtpLib::port)
	{
		if (FtpOpenPort(nControl, nData, mode, dir, buf) == -1) 
			return 0;
		if (!FtpAcceptConnection(*nData,nControl))
		{
			FtpClose(*nData);
			*nData = NULL;
			return 0;
		}
	}

	if (nControl->tlsdata)
	{
		(*nData)->ssl = SSL_new(nControl->ctx);
		(*nData)->sbio = BIO_new_socket((*nData)->handle, BIO_NOCLOSE);
		SSL_set_bio((*nData)->ssl,(*nData)->sbio,(*nData)->sbio);
		ret = SSL_connect((*nData)->ssl);
		if (ret != 1) 
			return 0;
		(*nData)->tlsdata = 1;
	}
	return 1;
}

/*
 * FtpOpenPort - Establishes a PORT connection for data transfer
 *
 * return 1 if successful, -1 otherwise
 */
int KuFtpLib::FtpOpenPort(ftphandle *nControl, ftphandle **nData, transfermode mode, int dir, char *cmd)
{
	if(mp_ftphandle->useProxy) // BIND method not support by now
	{
		return -1;
	}
	int sData;
	union {
	struct sockaddr sa;
	struct sockaddr_in in;
	} sin;
	struct linger lng = { 0, 0 };
	socklen_t l;
	int on=1;
	ftphandle *ctrl;
	char buf[256];

	if (nControl->dir != FTPLIB_CONTROL) 
		return -1;
	if ((dir != FTPLIB_READ) && (dir != FTPLIB_WRITE))
	{
		sprintf(nControl->response, "Invalid direction %d\n", dir);
		return -1;
	}
	if ((mode != KuFtpLib::ascii) && (mode != KuFtpLib::image))
	{
		sprintf(nControl->response, "Invalid mode %c\n", mode);
		return -1;
	}
	l = sizeof(sin);

	if (getsockname(nControl->handle, &sin.sa, &l) < 0)
	{
		//perror("getsockname error");
		return -1;
	}

	sData = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
	if (sData == -1)
	{
		//perror("socket error");
		return -1;
	}
	if (setsockopt(sData,SOL_SOCKET,SO_REUSEADDR, SETSOCKOPT_OPTVAL_TYPE &on,sizeof(on)) == -1)
	{
		//perror("setsockopt error");
		net_close(sData);
		return -1;
	}
	if (setsockopt(sData,SOL_SOCKET,SO_LINGER, SETSOCKOPT_OPTVAL_TYPE &lng,sizeof(lng)) == -1)
	{
		//perror("setsockopt error");
		net_close(sData);
		return -1;
	}

	sin.in.sin_port = 0;
	if (bind(sData, &sin.sa, sizeof(sin)) == -1)
	{
		//perror("bind error");
		net_close(sData);
		return -1;
	}
	if (listen(sData, 1) < 0)
	{
		//perror("listen error");
		net_close(sData);
		return -1;
	}
	if (getsockname(sData, &sin.sa, &l) < 0) return 0;
	sprintf(buf, "PORT %hhu,%hhu,%hhu,%hhu,%hhu,%hhu",
		(unsigned char) sin.sa.sa_data[2],
		(unsigned char) sin.sa.sa_data[3],
		(unsigned char) sin.sa.sa_data[4],
		(unsigned char) sin.sa.sa_data[5],
		(unsigned char) sin.sa.sa_data[0],
		(unsigned char) sin.sa.sa_data[1]);
	if (!FtpSendCmd(buf,'2',nControl))
	{
		net_close(sData);
		return -1;
	}

	if (mp_ftphandle->offset != 0)
	{
		char buf[256];
		sprintf(buf,"REST %lld", mp_ftphandle->offset);
		if (!FtpSendCmd(buf,'3',nControl))
		{
			net_close(sData);
			return 0;
		}
	}

	ctrl = static_cast<ftphandle*>(calloc(1,sizeof(ftphandle)));
	if (ctrl == NULL)
	{
		//perror("calloc error");
		net_close(sData);
		return -1;
	}
	if ((mode == 'A') && ((ctrl->buf = static_cast<char*>(malloc(FTPLIB_BUFSIZ))) == NULL))
	{
		//perror("calloc error");
		net_close(sData);
		free(ctrl);
		return -1;
	}

	if (!FtpSendCmd(cmd, '1', nControl))
	{
		FtpClose(*nData);
		*nData = NULL;
		return -1;
	}

	ctrl->handle = sData;
	ctrl->dir = dir;
	ctrl->ctrl = (nControl->cmode == KuFtpLib::pasv) ? nControl : NULL;
	ctrl->idletime = nControl->idletime;
	ctrl->cbarg = nControl->cbarg;
	ctrl->xfered = 0;
	ctrl->xfered1 = 0;
	ctrl->cbbytes = nControl->cbbytes;
	if (ctrl->idletime.tv_sec || ctrl->idletime.tv_usec) 
		ctrl->idlecb = nControl->idlecb;
	else 
		ctrl->idlecb = NULL;
	if (ctrl->cbbytes )
	       	ctrl->xfercb = nControl->xfercb;
	else 
		ctrl->xfercb = NULL;
	*nData = ctrl;

	return 1;
}

/*
 * FtpOpenPasv - Establishes a PASV connection for data transfer
 *
 * return 1 if successful, -1 otherwise
 */
int KuFtpLib::FtpOpenPasv(ftphandle *nControl, ftphandle **nData, transfermode mode, int dir, char *cmd)
{
	int sData;
	union {
		struct sockaddr sa;
		struct sockaddr_in in;
	} sin;
	struct linger lng = { 0, 0 };
	unsigned int l;
	int on=1;
	ftphandle *ctrl;
	char *cp;
	char *cp2;
	unsigned char v[6];
	int ret;

	if (nControl->dir != FTPLIB_CONTROL) 
		return -1;
	if ((dir != FTPLIB_READ) && (dir != FTPLIB_WRITE))
	{
		sprintf(nControl->response, "Invalid direction %d\n", dir);
		return -1;
	}
	if ((mode != KuFtpLib::ascii) && (mode != KuFtpLib::image))
	{
		sprintf(nControl->response, "Invalid mode %c\n", mode);
		return -1;
	}
	l = sizeof(sin);

	memset(&sin, 0, l);
	sin.in.sin_family = AF_INET;
	if (!FtpSendCmd("PASV",'2',nControl)) 
		return -1;
	cp = strchr(nControl->response,'(');
	if (cp == NULL) 
		return -1;
	cp++;
	cp2=new char[strlen(cp)+1];
	memcpy(cp2,cp,strlen(cp));
	cp2[strlen(cp)]=0;
	sscanf(cp,"%hhu,%hhu,%hhu,%hhu,%hhu,%hhu",&v[2],&v[3],&v[4],&v[5],&v[0],&v[1]);
	if (nControl->correctpasv) if (!CorrectPasvResponse(v)) 
		return -1;
	sin.sa.sa_data[2] = v[2];
	sin.sa.sa_data[3] = v[3];
	sin.sa.sa_data[4] = v[4];
	sin.sa.sa_data[5] = v[5];
	sin.sa.sa_data[0] = v[0];
	sin.sa.sa_data[1] = v[1];

	if (mp_ftphandle->offset != 0)
	{
		char buf[256];
		sprintf(buf,"REST %lld",mp_ftphandle->offset);
		if (!FtpSendCmd(buf,'3',nControl)) 
			return 0;
	}

	sData = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
	if (sData == -1)
	{
		//perror("socket error");
		return -1;
	}
	if (setsockopt(sData,SOL_SOCKET,SO_REUSEADDR, SETSOCKOPT_OPTVAL_TYPE &on,sizeof(on)) == -1)
	{
		//perror("setsockopt error");
		net_close(sData);
		return -1;
	}
	if (setsockopt(sData,SOL_SOCKET,SO_LINGER, SETSOCKOPT_OPTVAL_TYPE &lng,sizeof(lng)) == -1)
	{
		//perror("setsockopt error");
		net_close(sData);
		return -1;
	}

	if (nControl->dir != FTPLIB_CONTROL) 
		return -1;
	sprintf(cmd,"%s\r\n",cmd);
	if (nControl->tlsctrl)
	       	ret = SSL_write(nControl->ssl,cmd,strlen(cmd));
	else 
		ret = net_write(nControl->handle,cmd,strlen(cmd));
	if (mp_ftphandle->logcb != NULL)
		mp_ftphandle->logcb(cmd, mp_ftphandle->cbarg, INLOG);
	if (ret <= 0)
	{
		//perror("write error");
		return -1;
	}

	if(mp_ftphandle->useProxy==false)
	{
		if (connect(sData, &sin.sa, sizeof(sin.sa)) == -1)
		{
			//perror("connect error");
			net_close(sData);
			return -1;
		}
	}
	else//use proxy
	{
		int ip_1=0;
		int ip_2=0;
		int ip_3=0;
		int ip_4=0;
		unsigned short port_1=0;
		unsigned short  port_2=0;
		sscanf(cp2,"%d,%d,%d,%d,%hu,%hu)\n",&ip_1,&ip_2,&ip_3,&ip_4,&port_1,&port_2);
		char ftpPasvHost[16];
		memset(ftpPasvHost,0,16);
		sprintf(ftpPasvHost,"%d",ip_1);
		strcat(ftpPasvHost,".");
		sprintf(&ftpPasvHost[strlen(ftpPasvHost)],"%d",ip_2);
		strcat(ftpPasvHost,".");
		sprintf(&ftpPasvHost[strlen(ftpPasvHost)],"%d",ip_3);
		strcat(ftpPasvHost,".");
		sprintf(&ftpPasvHost[strlen(ftpPasvHost)],"%d",ip_4);
		unsigned short ftpPasvPort=0;
		ftpPasvPort=port_1*256+port_2;

		struct sockaddr_in sin1;
		struct hostent *phe;
		memset(&sin1,0,sizeof(sin1));
	        sin1.sin_family=AF_INET;
	        sin1.sin_port=htons(mp_ftphandle->proxyPort);

		if(inet_aton(mp_ftphandle->proxyHost, &sin1.sin_addr)==0)
		{
			if ((phe=gethostbyname(mp_ftphandle->proxyHost)) == NULL)
			{
				//perror("gethostbyname error");
				return 0;
			}
			memcpy((char *)&sin1.sin_addr,phe->h_addr,phe->h_length);
		}
		//connect to proxy
		if (connect(sData,(struct sockaddr *)&sin1,sizeof(sin1))==-1)
		{
			//perror("connect error");
			net_close(sData);
			return 0;
		}
		if(mp_ftphandle->proxyType==HTTP_PROXY)
		{
			if(!DoHttpProxy(mp_ftphandle->proxyUser,mp_ftphandle->proxyPass,ftpPasvHost,ftpPasvPort,sData))
			{
				net_close(sData);
				return 0;
			}
		}
		else if(mp_ftphandle->proxyType==SOCKS4_PROXY)
		{
			if(!DoSocks4Proxy(ftpPasvHost,ftpPasvPort,sData))
			{
				net_close(sData);
				return 0;
			}
		}
		else
		{
			if(!DoSocks5Proxy(mp_ftphandle->proxyUser,mp_ftphandle->proxyPass,ftpPasvHost,ftpPasvPort,sData))
			{
				net_close(sData);
				return 0;
			}
		}
	}
	if (!readresp('1', nControl))
	{
		net_close(sData);
		return -1;
	}
	ctrl = static_cast<ftphandle*>(calloc(1,sizeof(ftphandle)));
	if (ctrl == NULL)
	{
		//perror("calloc error");
		net_close(sData);
		return -1;
	}
	if ((mode == 'A') && ((ctrl->buf = static_cast<char*>(malloc(FTPLIB_BUFSIZ))) == NULL))
	{
		//perror("calloc error");
		net_close(sData);
		free(ctrl);
		return -1;
	}
	ctrl->handle = sData;
	ctrl->dir = dir;
	ctrl->ctrl = (nControl->cmode == KuFtpLib::pasv) ? nControl : NULL;
	ctrl->idletime = nControl->idletime;
	ctrl->cbarg = nControl->cbarg;
	ctrl->xfered = 0;
	ctrl->xfered1 = 0;
	ctrl->cbbytes = nControl->cbbytes;
	if (ctrl->idletime.tv_sec || ctrl->idletime.tv_usec) 
		ctrl->idlecb = nControl->idlecb;
	else 
		ctrl->idlecb = NULL;
	if (ctrl->cbbytes )
	       	ctrl->xfercb = nControl->xfercb;
	else 
		ctrl->xfercb = NULL;
	*nData = ctrl;

	return 1;
}

/*
 * FtpClose - close a data connection
 */
int KuFtpLib::FtpClose(ftphandle *nData)
{
	ftphandle *ctrl;

	if (nData->dir == FTPLIB_WRITE)
	{
		if (nData->buf != NULL)
		       	writeline(NULL, 0, nData);
	}
	else if (nData->dir != FTPLIB_READ) 
		return 0;
	if (nData->buf)
	       	free(nData->buf);
	shutdown(nData->handle,2);
	SleepMicroseconds(1000*100);
	net_close(nData->handle);

	ctrl = nData->ctrl;
	SSL_free(nData->ssl);
	free(nData);
	if (ctrl)
	       	return readresp('2', ctrl);
	return 1;
}

/*
 * FtpRead - read from a data connection
 */
int KuFtpLib::FtpRead(void *buf, int max, ftphandle *nData)
{
	int i;

	if (nData->dir != FTPLIB_READ)
		return 0;
	if (nData->buf) 
		i = readline(static_cast<char*>(buf), max, nData);
	else
	{
		i = socket_wait(nData);
		if (i != 1) 
			return 0;
		if (nData->tlsdata) 
			i = SSL_read(nData->ssl, buf, max);
		else 
			i = net_read(nData->handle,buf,max);
	}
	if (i == -1) 
		return 0;
	nData->xfered += i;
	if (nData->xfercb && nData->cbbytes)
	{
		nData->xfered1 += i;
		if (nData->xfered1 > nData->cbbytes)
		{
			if (nData->xfercb(nData->xfered, nData->cbarg) == 0) 
				return 0;
			nData->xfered1 = 0;
		}
	}
	return i;
}

/*
 * FtpWrite - write to a data connection
 */
int KuFtpLib::FtpWrite(void *buf, int len, ftphandle *nData)
{
	int i;

	if (nData->dir != FTPLIB_WRITE) 
		return 0;
	if (nData->buf)
	       	i = writeline(static_cast<char*>(buf), len, nData);
	else
	{
		socket_wait(nData);
		if (nData->tlsdata) 
			i = SSL_write(nData->ssl, buf, len);
		else 
			i = net_write(nData->handle, buf, len);
	}
	if (i == -1) 
		return 0;
	nData->xfered += i;

	if (nData->xfercb && nData->cbbytes)
	{
		nData->xfered1 += i;
		if (nData->xfered1 > nData->cbbytes)
		{
			if (nData->xfercb(nData->xfered, nData->cbarg) == 0) 
				return 0;
			nData->xfered1 = 0;
		}
	}
	return i;
}

/*
 * FtpSite - send a SITE command
 *
 * return 1 if command successful, 0 otherwise
 */
int KuFtpLib::Site(const char *cmd)
{
	char buf[256];

	if ((strlen(cmd) + 7) > sizeof(buf)) 
		return 0;
	sprintf(buf,"SITE %s",cmd);
	if (!FtpSendCmd(buf,'2',mp_ftphandle)) 
		return 0;
	return 1;
}

/*
 * FtpRaw - send a raw string string
 *
 * return 1 if command successful, 0 otherwise
 */

int KuFtpLib::Raw(const char *cmd)
{
	char buf[256];
	strncpy(buf, cmd, 256);
	if (!FtpSendCmd(buf,'2',mp_ftphandle))
		return 0;
	return 1;
}

/*
 * FtpSysType - send a SYST command
 *
 * Fills in the user buffer with the remote system type.  If more
 * information from the response is required, the user can parse
 * it out of the response buffer returned by FtpLastResponse().
 *
 * return 1 if command successful, 0 otherwise
 */
int KuFtpLib::SysType(char *buf, int max)
{
	int l = max;
	char *b = buf;
	char *s;
	if (!FtpSendCmd("SYST",'2',mp_ftphandle)) 
		return 0;
	s = &mp_ftphandle->response[4];
	while ((--l) && (*s != ' '))
	       	*b++ = *s++;
	*b++ = '\0';
	return 1;
}

int KuFtpLib::Feat()
{
	if (!FtpSendCmd("FEAT",'2',mp_ftphandle)) 
		return 0;
	return 1;
}
/*
 * FtpMkdir - create a directory at server
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::Mkdir(const char *path)
{
	char buf[256];

	if ((strlen(path) + 6) > sizeof(buf)) 
		return 0;
	sprintf(buf,"MKD %s",path);
	if (!FtpSendCmd(buf,'2', mp_ftphandle))
	       	return 0;
	return 1;
}

/*
 * FtpChdir - change path at remote
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::Chdir(const char *path)
{
	char buf[256];

	if ((strlen(path) + 6) > sizeof(buf)) 
		return 0;
	sprintf(buf,"CWD %s",path);
	if (!FtpSendCmd(buf,'2',mp_ftphandle))
	       	return 0;
	return 1;
}

/*
 * FtpCDUp - move to parent directory at remote
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::Cdup()
{
	if (!FtpSendCmd("CDUP",'2',mp_ftphandle)) 
		return 0;
	return 1;
}

/*
 * FtpRmdir - remove directory at remote
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::Rmdir(const char *path)
{
	char buf[256];

	if ((strlen(path) + 6) > sizeof(buf)) 
		return 0;
	sprintf(buf,"RMD %s",path);
	if (!FtpSendCmd(buf,'2',mp_ftphandle)) 
		return 0;
	return 1;
}

/*
 * FtpPwd - get working directory at remote
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::Pwd(char *path, int max)
{
	int l = max;
	char *b = path;
	char *s;

 	if (!FtpSendCmd("PWD",'2',mp_ftphandle))
	       	return 0;
	s = strchr(mp_ftphandle->response, '"');
	if (s == NULL) 
		return 0;
   s++;
	while ((--l) && (*s) && (*s != '"'))
	       	*b++ = *s++;
	*b++ = '\0';
	return 1;
}

/*
 * FtpXfer - issue a command and transfer data
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::FtpXfer(const char *localfile, const char *path, ftphandle *nControl, accesstype type, transfermode mode)
{
	int l,c;
	char *dbuf;
	FILE *local = NULL;
	ftphandle *nData;
	int rv=1; // 3.1-1

	if (localfile != NULL)
	{
		char ac[3] = "  ";
		if ((type == KuFtpLib::dir) || (type == KuFtpLib::dirverbose)) 
		{
		       	ac[0] = 'w'; 
			ac[1] = '\0'; 
		}
		if (type == KuFtpLib::fileread)
	       	{
		       	ac[0] = 'w'; 
			ac[1] = '\0';
	       	}
		if (type == KuFtpLib::filewriteappend) 
		{
		       	ac[0] = 'r'; 
			ac[1] = '\0'; 
		}
		if (type == KuFtpLib::filereadappend)
	       	{
		       	ac[0] = 'a'; 
			ac[1] = '\0'; 
		}
		if (type == KuFtpLib::filewrite)
	       	{
		       	ac[0] = 'r';
		       	ac[1] = '\0';
	       	}
		if (mode == KuFtpLib::image)
		       	ac[1] = 'b';

		local = fopen64(localfile, ac);
		if (local == NULL)
		{
			strncpy(nControl->response, strerror(errno), sizeof(nControl->response));
			char log[256];
			sprintf(log,"Open %s Failed. System says:\r\n",localfile);
			if (mp_ftphandle->logcb != NULL)
		                mp_ftphandle->logcb(log,mp_ftphandle->cbarg,COMMONLOG);
			mp_ftphandle->logcb(nControl->response,mp_ftphandle->cbarg,INLOG);
			return 0;
		}
		if (type == KuFtpLib::filewriteappend) 
			fseeko64(local,mp_ftphandle->offset,SEEK_SET);
	}
	if (local == NULL) 
		local = ((type == KuFtpLib::filewrite) || (type == KuFtpLib::filewriteappend)) ? stdin : stdout;
	if (!FtpAccess(path, type, mode, nControl, &nData))
	       	return 0;

	mp_ftphandle->stoptransfer=false;
	dbuf = static_cast<char*>(malloc(FTPLIB_BUFSIZ));
	if ((type == KuFtpLib::filewrite) || (type == KuFtpLib::filewriteappend))
	{
		int i=10;
		timeval start;
		timeval end;
		gettimeofday(&start,NULL);
		while ((l = fread(dbuf, 1, FTPLIB_BUFSIZ, local)) > 0)
		{
			if ((c = FtpWrite(dbuf, l, nData)) < l)
			{
				//printf("short write: passed %d, wrote %d\n", l, c);
				rv = 0;
				break;
			}
			if(mp_ftphandle->stoptransfer)
				break;
			if(mp_ftphandle->maxUploadSpeed>0&&type!=KuFtpLib::dirverbose)
			{
				if(--i==0)
				{
					gettimeofday(&end,NULL);
					off64_t timeUse=1000000*(end.tv_sec-start.tv_sec)+end.tv_usec-start.tv_usec;
					off64_t sleepTime=10*1000*1000/mp_ftphandle->maxUploadSpeed-timeUse;//microseconds
					if(sleepTime>0)
						SleepMicroseconds(sleepTime);
					gettimeofday(&start,NULL);
					i=10;
				}
			}
		}
	}
	else
	{
		int i=10;
		timeval start;
		timeval end;
		gettimeofday(&start,NULL);
		while ((l = FtpRead(dbuf, FTPLIB_BUFSIZ, nData)) > 0)
		{
			if (fwrite(dbuf, 1, l, local) <= 0)
			{
				//perror("localfile write error");
				break;
			}
			if(mp_ftphandle->stoptransfer)
				break;
			if(mp_ftphandle->maxDownloadSpeed>0&&type!=KuFtpLib::dirverbose)
			{
				if(--i==0)
				{
					gettimeofday(&end,NULL);
					off64_t timeUse=1000000*(end.tv_sec-start.tv_sec)+end.tv_usec-start.tv_usec;
					off64_t sleepTime=10*1000*1000/mp_ftphandle->maxDownloadSpeed-timeUse;//microseconds
					if(sleepTime>0)
						SleepMicroseconds(sleepTime);
					gettimeofday(&start,NULL);
					i=10;
				}
			}
		}
	}
	free(dbuf);
	fflush(local);
	if (localfile != NULL)
		fclose(local);
	return FtpClose(nData);
	return rv;
}
void KuFtpLib::SleepMicroseconds(off64_t usecs)
{
	timeval timeout;
	memset(&timeout,0,sizeof(struct timeval));
	timeout.tv_sec=(long)(usecs/1000/1000);
	timeout.tv_usec=(long)(usecs%(1000*1000));
	select(0,NULL,NULL,NULL,&timeout);
}
/*
 * FtpNlst - issue an NLST command and write response to output
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::Nlst(const char *outputfile, const char *path)
{
	mp_ftphandle->offset = 0;
	return FtpXfer(outputfile, path, mp_ftphandle, KuFtpLib::dir, KuFtpLib::ascii);
}

/*
 * FtpDir - issue a LIST command and write response to output
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::Dir(const char *outputfile, const char *path)
{
	mp_ftphandle->offset = 0;
	return FtpXfer(outputfile, path, mp_ftphandle, KuFtpLib::dirverbose, KuFtpLib::ascii);
}

/*
 * FtpSize - determine the size of a remote file
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::Size(const char *path, int *size, transfermode mode)
{
       	char cmd[256];
     	int resp,sz,rv=1;
     	if ((strlen(path) + 7) > sizeof(cmd))
     		return 0;
     	sprintf(cmd, "TYPE %c", mode);
     	if (!FtpSendCmd(cmd, '2', mp_ftphandle)) 
		return 0;
	sprintf(cmd,"SIZE %s",path);
     	if (!FtpSendCmd(cmd,'2',mp_ftphandle)) 
		rv = 0;
     	else
     	{
		if (sscanf(mp_ftphandle->response, "%d %d", &resp, &sz) == 2)
		       	*size = sz;
		else 
			rv = 0;
     	}
     	return rv;
}

/*
 * FtpModDate - determine the modification date of a remote file
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::ModDate(const char *path, char *dt, int max)
{
	char buf[256];
	int rv = 1;

	if ((strlen(path) + 7) > sizeof(buf)) 
		return 0;
	sprintf(buf,"MDTM %s",path);

     	if (!FtpSendCmd(buf,'2',mp_ftphandle)) 
		rv = 0;
	else 
		strncpy(dt, &mp_ftphandle->response[4], max);
	return rv;
}

/*
 * FtpGet - issue a GET command and write received data to output
 *
 * return 1 if successful, 0 otherwise
 */

int KuFtpLib::Get(const char *outputfile, const char *path, transfermode mode, off64_t offset)
{
	mp_ftphandle->offset = offset;
	if (offset == 0) 
		return FtpXfer(outputfile, path, mp_ftphandle, KuFtpLib::fileread, mode);
	else 
		return FtpXfer(outputfile, path, mp_ftphandle, KuFtpLib::filereadappend, mode);
}

/*
 * FtpPut - issue a PUT command and send data from input
 *
 * return 1 if successful, 0 otherwise
 */

int KuFtpLib::Put(const char *inputfile, const char *path, transfermode mode, off64_t offset)
{
	mp_ftphandle->offset = offset;
	if (offset == 0) 
		return FtpXfer(inputfile, path, mp_ftphandle, KuFtpLib::filewrite, mode);
	else 
		return FtpXfer(inputfile, path, mp_ftphandle, KuFtpLib::filewriteappend, mode);
}


int KuFtpLib::Rename(const char *src, const char *dst)
{
	char cmd[256];

	if (((strlen(src) + 7) > sizeof(cmd)) || ((strlen(dst) + 7) > sizeof(cmd))) 
		return 0;
	sprintf(cmd,"RNFR %s",src);
	if (!FtpSendCmd(cmd,'3',mp_ftphandle)) 
		return 0;
	sprintf(cmd,"RNTO %s",dst);
	if (!FtpSendCmd(cmd,'2',mp_ftphandle)) 
		return 0;

	return 1;
}

int KuFtpLib::Delete(const char *path)
{
	char cmd[256];

	if ((strlen(path) + 7) > sizeof(cmd)) 
		return 0;
   	sprintf(cmd,"DELE %s",path);
   	if (!FtpSendCmd(cmd,'2', mp_ftphandle)) 
		return 0;
   	return 1;
}

/*
 * FtpQuit - disconnect from remote
 *
 * return 1 if successful, 0 otherwise
 */
int KuFtpLib::Quit()
{
	if (mp_ftphandle->dir != FTPLIB_CONTROL) 
		return 0;
	if (mp_ftphandle->handle == 0)
	{
		strcpy(mp_ftphandle->response, "Error: no anwser from server\n");
		return 0;
	}
	if (!FtpSendCmd("QUIT",'2',mp_ftphandle))
	{
		net_close(mp_ftphandle->handle);
		return 0;
	}
	else
	{
		net_close(mp_ftphandle->handle);
		return 1;
	}
}
void KuFtpLib::StopTransfer()
{
	mp_ftphandle->stoptransfer=true;
}
int KuFtpLib::Noop()
{
	if(!FtpSendCmd("NOOP",'2',mp_ftphandle))
		return 0;
	return 1;
}	
int KuFtpLib::Fxp(KuFtpLib* src, KuFtpLib* dst, const char *pathSrc, const char *pathDst, transfermode mode, fxpmethod method)
{
	char *cp;
	unsigned char v[6];
	char buf[256];
	int retval = 0;
	
	sprintf(buf, "TYPE %c", mode);
	if (!dst->FtpSendCmd(buf,'2',dst->mp_ftphandle))
	       	return -1;
	if (!src->FtpSendCmd(buf,'2',src->mp_ftphandle)) 
		return -1;

	if (method == KuFtpLib::defaultfxp)
	{
		// PASV dst
		if (!dst->FtpSendCmd("PASV",'2',dst->mp_ftphandle)) 
			return -1;
		cp = strchr(dst->mp_ftphandle->response,'(');
		if (cp == NULL) 
			return -1;
		cp++;
		sscanf(cp,"%hhu,%hhu,%hhu,%hhu,%hhu,%hhu",&v[2],&v[3],&v[4],&v[5],&v[0],&v[1]);
		if (dst->mp_ftphandle->correctpasv) if (!dst->CorrectPasvResponse(v)) 
			return -1;
		// PORT src
		sprintf(buf, "PORT %d,%d,%d,%d,%d,%d", v[2],v[3],v[4],v[5],v[0],v[1]);
		if (!src->FtpSendCmd(buf,'2',src->mp_ftphandle)) 
			return -1;
		// RETR src
		strcpy(buf,"RETR");
		if (pathSrc != NULL)
		{
			int i = strlen(buf);
			buf[i++] = ' ';
			if ((strlen(pathSrc) + i) >= sizeof(buf)) 
				return 0;
			strcpy(&buf[i],pathSrc);
		}
		if (!src->FtpSendCmd(buf, '1', src->mp_ftphandle)) 
			return 0;
		// STOR dst
		strcpy(buf,"STOR");
		if (pathDst != NULL)
		{
			int i = strlen(buf);
			buf[i++] = ' ';
			if ((strlen(pathDst) + i) >= sizeof(buf)) 
				return 0;
			strcpy(&buf[i],pathDst);
		}
		if (!dst->FtpSendCmd(buf, '1', dst->mp_ftphandle))
		{
			/* this closes the data connection, to abort the RETR on
			the source ftp. all hail pftp, it took me several
			hours and i was absolutely clueless, playing around with
			ABOR and whatever, when i desperately checked the pftp
			source which gave me this final hint. thanks dude(s). */
		
			dst->FtpSendCmd("PASV", '2', dst->mp_ftphandle);
			src->readresp('4', src->mp_ftphandle);
			return 0;
		}
		retval = (src->readresp('2', src->mp_ftphandle)) & (dst->readresp('2', dst->mp_ftphandle));
	}
	else 
	{
		// PASV src
		if (!src->FtpSendCmd("PASV",'2',src->mp_ftphandle)) 
			return -1;
		cp = strchr(src->mp_ftphandle->response,'(');
		if (cp == NULL) 
			return -1;
		cp++;
		sscanf(cp,"%hhu,%hhu,%hhu,%hhu,%hhu,%hhu",&v[2],&v[3],&v[4],&v[5],&v[0],&v[1]);
		if (src->mp_ftphandle->correctpasv) 
			if (!src->CorrectPasvResponse(v)) 
				return -1;
		// PORT dst
		sprintf(buf, "PORT %d,%d,%d,%d,%d,%d", v[2],v[3],v[4],v[5],v[0],v[1]);
		if (!dst->FtpSendCmd(buf,'2',dst->mp_ftphandle)) 
			return -1;
		// STOR dest
		strcpy(buf,"STOR");
		if (pathDst != NULL)
		{
			int i = strlen(buf);
			buf[i++] = ' ';
			if ((strlen(pathDst) + i) >= sizeof(buf)) 
				return 0;
			strcpy(&buf[i],pathDst);
		}
		if (!dst->FtpSendCmd(buf, '1', dst->mp_ftphandle)) 
			return 0;
		// RETR src
		strcpy(buf,"RETR");
		if (pathSrc != NULL)
		{
			int i = strlen(buf);
			buf[i++] = ' ';
			if ((strlen(pathSrc) + i) >= sizeof(buf)) 
				return 0;
			strcpy(&buf[i],pathSrc);
		}
		if (!src->FtpSendCmd(buf, '1', src->mp_ftphandle))
		{
			src->FtpSendCmd("PASV", '2', src->mp_ftphandle);
			dst->readresp('4', dst->mp_ftphandle);
			return 0;
		}
		// wait til its finished!
		retval = (src->readresp('2', src->mp_ftphandle)) & (dst->readresp('2', dst->mp_ftphandle));
	}

	return retval;
}

int KuFtpLib::SetDataEncryption(dataencryption enc)
{
	if (!mp_ftphandle->tlsctrl) 
		return 0;
	if (!FtpSendCmd("PBSZ 0",'2',mp_ftphandle)) 
		return 0;
	switch(enc)
	{
		case KuFtpLib::unencrypted:
			mp_ftphandle->tlsdata = 0;
			if (!FtpSendCmd("PROT C",'2',mp_ftphandle)) 
				return 0;
			break;
		case KuFtpLib::secure:
			mp_ftphandle->tlsdata = 1;
			if (!FtpSendCmd("PROT P",'2',mp_ftphandle)) 
				return 0;
			break;
		default:
			return 0;
	}
	return 1;
}

int KuFtpLib::NegotiateEncryption()
{
	int ret;
	if (!FtpSendCmd("AUTH TLS",'2',mp_ftphandle))
	       	return 0;
	mp_ftphandle->sbio = BIO_new_socket(mp_ftphandle->handle, BIO_NOCLOSE);
	SSL_set_bio(mp_ftphandle->ssl,mp_ftphandle->sbio,mp_ftphandle->sbio);

	ret = SSL_connect(mp_ftphandle->ssl);
	if (ret == 1) 
		mp_ftphandle->tlsctrl = 1;
	if (mp_ftphandle->certcb != NULL)
	{
		X509 *cert = SSL_get_peer_certificate(mp_ftphandle->ssl);
		if (!mp_ftphandle->certcb(mp_ftphandle->cbarg, cert)) 
			return 0;
	}
	if (ret < 1) 
		return 0;
	return 1;
}

void KuFtpLib::SetCallbackCertFunction(FtpCallbackCert pointer)
{
	mp_ftphandle->certcb = pointer;
}


void KuFtpLib::SetCallbackIdleFunction(FtpCallbackIdle pointer)
{
	mp_ftphandle->idlecb = pointer;
}

void KuFtpLib::SetCallbackXferFunction(FtpCallbackXfer pointer)
{
	mp_ftphandle->xfercb = pointer;
}

void KuFtpLib::SetCallbackLogFunction(FtpCallbackLog pointer)
{
	mp_ftphandle->logcb = pointer;
}

void KuFtpLib::SetCallbackArg(void *arg)
{
	mp_ftphandle->cbarg = arg;
}

void KuFtpLib::SetCallbackBytes(off64_t bytes)
{
	mp_ftphandle->cbbytes = bytes;
}

void KuFtpLib::SetCallbackIdletime(int time)
{
	mp_ftphandle->idletime.tv_sec = time / 1000;
	mp_ftphandle->idletime.tv_usec = (time % 1000) * 1000;
}

void KuFtpLib::SetConnmode(connmode mode)
{
	mp_ftphandle->cmode = mode;
}
void KuFtpLib::SetFtpUseProxy(bool i)
{
	mp_ftphandle->useProxy=i;
}
void KuFtpLib::SetCheckTimeout(bool i)
{
	mp_ftphandle->checkTimeout=i;
}
void KuFtpLib::ClearHandle()
{
	mp_ftphandle->dir = FTPLIB_CONTROL;
	mp_ftphandle->ctrl = NULL;
	mp_ftphandle->cmode = KuFtpLib::pasv;
	mp_ftphandle->idlecb = NULL;
	mp_ftphandle->idletime.tv_sec = mp_ftphandle->idletime.tv_usec = 0;
	mp_ftphandle->cbarg = NULL;
	mp_ftphandle->xfered = 0;
	mp_ftphandle->xfered1 = 0;
	mp_ftphandle->cbbytes = 0;
	mp_ftphandle->tlsctrl = 0;
	mp_ftphandle->tlsdata = 0;
	mp_ftphandle->certcb = NULL;
	mp_ftphandle->offset = 0;
	mp_ftphandle->handle = 0;
	mp_ftphandle->logcb = NULL;
	mp_ftphandle->xfercb = NULL;
	mp_ftphandle->correctpasv = false;
	mp_ftphandle->stoptransfer= false;
	mp_ftphandle->useProxy=false;
	mp_ftphandle->maxDownloadSpeed=0;
	mp_ftphandle->maxUploadSpeed=0;
	mp_ftphandle->checkTimeout=false;
	mp_ftphandle->timeout=false;
}

int KuFtpLib::CorrectPasvResponse(unsigned char *v)
{
	struct sockaddr ipholder;
	socklen_t ipholder_size = sizeof(ipholder);

	if (getpeername(mp_ftphandle->handle, &ipholder, &ipholder_size) == -1)
	{
		//perror("getpeername error");
		net_close(mp_ftphandle->handle);
		return 0;
	}
	for (int i = 2; i < 6; i++) 
		v[i] = ipholder.sa_data[i];
	return 1;
}

ftphandle* KuFtpLib::RawOpen(const char *path, accesstype type, transfermode mode)
{
	int ret;
	ftphandle* datahandle;
	ret = FtpAccess(path, type, mode, mp_ftphandle, &datahandle); 
	if (ret) 
		return datahandle;
	else 
		return NULL;
}

int KuFtpLib::RawClose(ftphandle* handle)
{
	return FtpClose(handle);
} 

int KuFtpLib::RawWrite(void* buf, int len, ftphandle* handle)
{
	return FtpWrite(buf, len, handle);
}

int KuFtpLib::RawRead(void* buf, int max, ftphandle* handle)
{
	return FtpRead(buf, max, handle);
}


