/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/* original copyright notes */
/***************************************************************************
    copyright            : (C) 2003 by mkulke
    email                : sikor_sxe@radicalapproach.de
*****************************************************************************
    ftplib.c - callable ftp access routines
    Copyright (C) 1996, 1997, 1998 Thomas Pfau, pfau@cnj.digex.net
    73 Catherine Street, South Bound Brook, NJ, 08880
*****************************************************************************/
 
#ifndef KUFTPLIB_H
#define KUFTPLIB_H

#include <sys/time.h>
#include <unistd.h>

#include <openssl/ssl.h>

#define off64_t __off64_t

using namespace std;

typedef int (*FtpCallbackXfer)(off64_t xfered, void *arg);
typedef int (*FtpCallbackIdle)(void *arg);
typedef void (*FtpCallbackLog)(char *str, void* arg, int out);
typedef bool (*FtpCallbackCert)(void *arg, X509 *cert);

struct ftphandle {
	char *cput,*cget;
	int handle;
	int cavail,cleft;
	char *buf;
	int dir;
	ftphandle *ctrl;
	int cmode;
	struct timeval idletime;
	FtpCallbackXfer xfercb;
	FtpCallbackIdle idlecb;
	FtpCallbackLog logcb;
	void *cbarg;
	off64_t xfered;
	off64_t cbbytes;
	off64_t xfered1;
	char response[256];
	SSL* ssl;
	SSL_CTX* ctx;
	BIO* sbio;
	int tlsctrl;
	int tlsdata;
	FtpCallbackCert certcb;
	off64_t offset;
	bool correctpasv;
	bool stoptransfer;
	bool useProxy;
	int maxUploadSpeed;  // maxUploadSpeed KB/s
	int maxDownloadSpeed;
	bool checkTimeout; // session idle and be kicked
	bool timeout; // network timeout
	char *proxyHost;
	unsigned short proxyPort;
	char *proxyUser;
	char *proxyPass;
	int proxyType;
};

class KuFtpLib
{
	public:
		enum accesstype
		{
			dir = 1,
			dirverbose,
			fileread,
			filewrite,
			filereadappend,
			filewriteappend
		}; 
		enum transfermode
		{
			ascii = 'A',
			image = 'I'
		};
		enum connmode
		{
			pasv = 1,
			port
		};
		enum fxpmethod
		{
			defaultfxp = 0,
	        	alternativefxp
		};
	    	enum dataencryption
	    	{
			unencrypted = 0,
			secure
	    	};
		KuFtpLib();
		~KuFtpLib();
	    	char* LastResponse();
	    	int Connect(const char *host,unsigned short port);// ftpserver
	    	int Connect(const char *host,unsigned short port,const char *proxyHost,unsigned short proxyPort,const char *proxyUser,const char *proxyPass,int proxyType);
	    	int Login(const char *user, const char *pass);
	    	int Site(const char *cmd);
	    	int Raw(const char *cmd);
	    	int SysType(char *buf, int max);
	    	int Feat();
	    	int Mkdir(const char *path);
	    	int Chdir(const char *path);
	    	int Cdup();
	    	int Rmdir(const char *path);
	    	int Pwd(char *path, int max);
	    	int Nlst(const char *outputfile, const char *path);
	    	int Dir(const char *outputfile, const char *path);
	    	int Size(const char *path, int *size, transfermode mode);
	    	int ModDate(const char *path, char *dt, int max);
	    	int Get(const char *outputfile, const char *path, transfermode mode, off64_t offset = 0);
	    	int Put(const char *inputfile, const char *path, transfermode mode, off64_t offset = 0);
	    	int Rename(const char *src, const char *dst);
	    	int Delete(const char *path);
		int Noop();
		int Timeout();
		int ConnectionLost();
		void SetFtpUseProxy(bool);
		void SetCheckTimeout(bool);
		int SetDataEncryption(dataencryption enc);
		int NegotiateEncryption();
		void SetCallbackCertFunction(FtpCallbackCert pointer);
	    	int Quit();
		void StopTransfer();
		void SetMaxUploadSpeed(int i){mp_ftphandle->maxUploadSpeed=i;}
		void SetMaxDownloadSpeed(int i){mp_ftphandle->maxDownloadSpeed=i;}
	    	void SetCallbackIdleFunction(FtpCallbackIdle pointer);
	    	void SetCallbackLogFunction(FtpCallbackLog pointer);
		void SetCallbackXferFunction(FtpCallbackXfer pointer);
		void SetCallbackArg(void *arg);
	    	void SetCallbackBytes(off64_t bytes);
		void SetCorrectPasv(bool b) { mp_ftphandle->correctpasv = b; };
	    	void SetCallbackIdletime(int time);
	    	void SetConnmode(connmode mode);
	    	static int Fxp(KuFtpLib *src,KuFtpLib *dst,const char *pathSrc,const char *pathDst,transfermode mode,fxpmethod method);
		ftphandle* RawOpen(const char *path, accesstype type, transfermode mode);
		int RawClose(ftphandle* handle); 
		int RawWrite(void* buf, int len, ftphandle* handle);
		int RawRead(void* buf, int max, ftphandle* handle); 
	private:
	    	ftphandle* mp_ftphandle;
		void base64Encode(const char *string,char *encoded);
		int ReadCrLfLine(int fd,void *buf,int size);
		void LogHttpProxyReturn(int fd);
		void LogSocks5ProxyError(int error);
		void SleepMicroseconds(off64_t t);
		int DoHttpProxy(const char *proxyUser,const char *proxyPass,const char *ftpHost,unsigned short ftpPort,int fd=0);
		int DoSocks4Proxy(const char *ftpHost,unsigned short ftpPort,int fd=0);
		int DoSocks5Proxy(const char *proxyUser,const char *proxyPass,const char *ftpHost,unsigned short ftpPort,int fd=0);
	    	int FtpXfer(const char *localfile,const char *path,ftphandle *nControl,accesstype type,transfermode mode);
	    	int FtpOpenPasv(ftphandle *nControl, ftphandle **nData, transfermode mode, int dir, char *cmd);
	    	int FtpSendCmd(const char *cmd, char expresp, ftphandle *nControl);
	    	int FtpAcceptConnection(ftphandle *nData, ftphandle *nControl);
	    	int FtpOpenPort(ftphandle *nControl, ftphandle **nData, transfermode mode, int dir, char *cmd);
	    	int FtpRead(void *buf, int max, ftphandle *nData);
	    	int FtpWrite(void *buf, int len, ftphandle *nData);
	    	int FtpAccess(const char *path, accesstype type, transfermode mode, ftphandle *nControl, ftphandle **nData);
	    	int FtpClose(ftphandle *nData);
		int socket_wait(ftphandle *ctl);
	    	int readline(char *buf,int max,ftphandle *ctl);
	    	int writeline(char *buf, int len, ftphandle *nData);
	    	int readresp(char c, ftphandle *nControl);
		void ClearHandle();
		int CorrectPasvResponse(unsigned char *v);
};

#endif


