/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>

#include <qlistview.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qsplitter.h>
#include <qaction.h>
#include <qheader.h>
#include <qtextcodec.h>
#include <qcstring.h>
#include <qspinbox.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qdom.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qsignalmapper.h>
#include <qdatetime.h>
#include <qmutex.h>
#include <qpoint.h>
#include <qregexp.h>
#include <qtimer.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qcheckbox.h>
#include <qfileinfo.h>
#include <qptrlist.h>
#include <qregexp.h>
#include <qvaluelist.h>
#include <qcursor.h>
#include <qlistview.h>

#include <kglobal.h>
#include <kiconloader.h>
#include <ktabwidget.h>
#include <klineedit.h>
#include <klistview.h>
#include <kapplication.h>
#include <kcombobox.h>
#include <klocale.h>
#include <kcharsets.h>
#include <ktextedit.h>
#include <kcursor.h>
#include <kfiledialog.h>
#include <ktoolbar.h>
#include <ktoolbarbutton.h>
#include <ksystemtray.h>
#include <kstatusbar.h>
#include <kprogress.h>
#include <kpopupmenu.h>
#include <kmessagebox.h>
#include <kpassdlg.h>

#include "kuftp.h"
#include "kuftpmain.h"
#include "kuftpcore.h"
#include "kuftpqueue.h"
#include "kuftpconnectdialog.h"
#include "kuftpeditnamedialog.h"
#include "kuftpsettings.h"
#include "kuftpballoon.h"
#include "kuftpaddbookmarkdialog.h"

#define CURRENTTIME  QString(QTime::currentTime().toString("hh:mm:ss"))

extern KTabWidget KuFtp::*p_tabwidget;
extern KuFtpMain KuFtp::*currentCW;
extern KuFtpSettings kuftpGlobalSettings;
extern KSystemTray KuFtp::*systemTray;

int countOfSelectedListViewItems(QListView *listview);
QString formatFileSizeString(off64_t size);

KuFtpMain::KuFtpMain(QWidget *parent):QWidget(parent)
{
	createAction();
        hSplitter=new QSplitter(QSplitter::Vertical,parent );
	topVSplitter=new QSplitter(hSplitter);

        leftBox=new QWidget(topVSplitter);
        leftLayout=new QGridLayout(leftBox);
	leftToolBar=new KToolBar(leftBox);
	leftToolBar->insertWidget(0,QString(i18n("  Local Browser  ")).length(),new QLabel(i18n("  Local Browser  "),leftToolBar));
	leftToolBar->insertButton("up",1,SIGNAL(clicked()),this,SLOT(SLOT_cdup()),true,i18n("Cd up"));
	leftToolBar->insertButton("gohome",2,SIGNAL(clicked()),this,SLOT(SLOT_gohome()),true,i18n("Home folder"));
	leftToolBar->insertButton("reload",3,SIGNAL(clicked()),this,SLOT(SLOT_reload()),true,i18n("Reload"));
        leftLabel=new QLabel(i18n("Location:"),leftBox);
        leftLineEdit=new KLineEdit(leftBox);
        connect(leftLineEdit,SIGNAL(returnPressed(const QString &)),this,SLOT(SLOT_updateLeftListView(const QString &)));
        leftListView=new KListView(leftBox);
	leftListView->setSelectionMode(QListView::Extended);
        leftListView->addColumn(i18n("Name"));
        leftListView->addColumn(i18n("Size"));
        leftListView->addColumn(i18n("Date"));
	connect(leftListView->header(),SIGNAL(clicked(int)),this,SLOT(SLOT_sortLeftListView(int)));
	connect(leftListView,SIGNAL(executed(QListViewItem *)),this,SLOT(SLOT_leftListViewClicked(QListViewItem *)));
	connect(leftListView,SIGNAL(returnPressed(QListViewItem *)),this,SLOT(SLOT_leftListViewClicked(QListViewItem *)));
	connect(leftListView,SIGNAL(rightButtonClicked(QListViewItem *,const QPoint &,int)),this,SLOT(SLOT_leftListViewRightClicked(QListViewItem *)));
	leftStatusLabel=new QLabel(leftBox);
	leftStatusBar=new KStatusBar(leftBox);
	leftStatusBar->addWidget(leftStatusLabel,1);
	leftStatusLabel->setText(i18n("Total"));
	leftStatusLabel->setAlignment( AlignHCenter );
	leftLayout->addMultiCellWidget(leftToolBar,0,0,0,1);
        leftLayout->addWidget(leftLabel,1,0);
	leftLayout->addWidget(leftLineEdit,1,1);
	leftLayout->addMultiCellWidget(leftListView,2,2,0,1);
	leftLayout->addMultiCellWidget(leftStatusBar,3,3,0,1);

        rightBox=new QWidget(topVSplitter);
        rightLayout=new QGridLayout(rightBox);
	connectPopupMenu=new KPopupMenu(rightBox);
	connectPopupMenu->insertSeparator();
	quickConnectAct=new QAction(i18n("Quick Connect"),0,rightBox);
	quickConnectAct->addTo(connectPopupMenu);
	connect(quickConnectAct,SIGNAL(activated()),this,SLOT(SLOT_connectDialog()));
	connectPopupMenu->insertSeparator();
	bookmarksSubPopupMenu=new KPopupMenu(connectPopupMenu);
	connectPopupMenu->insertItem(KGlobal::iconLoader()->loadIconSet("bookmark_folder",KIcon::Small),i18n("Bookmarks"),bookmarksSubPopupMenu);
	SLOT_updateBookmarksSubPopupMenu();
	connectPopupMenu->insertSeparator();
	settingsPopupMenu=new KPopupMenu(rightBox);
	showHideFilterAct=new QAction(i18n("Show/Hide Filter"),0,rightBox);
	showHideFilterAct->addTo(settingsPopupMenu);
	connect(showHideFilterAct,SIGNAL(activated()),this,SLOT(SLOT_showHideFilter()));
	showHideFilterAct->setEnabled(false);
	sendRawCmdAct=new QAction(KGlobal::iconLoader()->loadIconSet("launch",KIcon::Small),i18n("Send Raw Command"),0,rightBox);
	sendRawCmdAct->addTo(settingsPopupMenu);
	connect(sendRawCmdAct,SIGNAL(activated()),this,SLOT(SLOT_showHideRawCmdLine()));
	sendRawCmdAct->setEnabled(false);
	exportDirContentAct=new QAction(i18n("Export Directory Content..."),0,rightBox);
	exportDirContentAct->addTo(settingsPopupMenu);
	connect(exportDirContentAct,SIGNAL(activated()),this,SLOT(SLOT_exportDirContent()));
	exportDirContentAct->setEnabled(false);
	settingsPopupMenu->insertSeparator();
	charsetSubPopupMenu=new KPopupMenu(settingsPopupMenu);
	charsetSubPopupMenu->setCheckable(true);
	settingsPopupMenu->insertItem(KGlobal::iconLoader()->loadIconSet("charset",KIcon::Small),i18n("Change Encoding"),
			charsetSubPopupMenu);
	charsetSubPopupMenu->setEnabled(false);
	rightToolBar=new KToolBar(rightBox);
	rightToolBar->insertButton("connect_creating",0,connectPopupMenu,true,i18n("Quick Connect"));
	rightToolBar->insertButton("connect_no",1,SIGNAL(clicked()),this,SLOT(SLOT_disconnect()),true,i18n("Disconnect"));
	rightToolBar->insertButton("stop",2,SIGNAL(clicked()),this,SLOT(SLOT_rightCancel()),true,i18n("Cancel"));
	rightToolBar->insertButton("up",3,SIGNAL(clicked()),this,SLOT(SLOT_rightCdup()),true,i18n("Cd up"));
	rightToolBar->insertButton("reload",4,SIGNAL(clicked()),this,SLOT(SLOT_rightReload()),true,i18n("Reload"));
	rightToolBar->insertButton("configure",5,settingsPopupMenu,true,i18n("Configure"));

        rightLabel=new QLabel(i18n("Location:"),rightBox);
        rightLineEdit=new KLineEdit(rightBox);
        connect(rightLineEdit,SIGNAL(returnPressed(const QString &)),this,SLOT(SLOT_rightLineEditReturn(const QString &)));
        rightListView=new KListView(rightBox);
	rightListView->setSelectionMode(QListView::Extended);
        rightListView->addColumn(i18n("Name"));
        rightListView->addColumn(i18n("Size"));
        rightListView->addColumn(i18n("Date"));
        rightListView->addColumn(i18n("Permission"));
//        rightListView->addColumn(i18n("User"));
//      rightListView->addColumn(i18n("Group"));
	connect(rightListView->header(),SIGNAL(clicked(int)),this,SLOT(SLOT_sortRightListView(int)));
	connect(rightListView,SIGNAL(executed(QListViewItem *)),this,SLOT(SLOT_rightListViewClicked(QListViewItem *)));
	connect(rightListView,SIGNAL(returnPressed(QListViewItem *)),this,SLOT(SLOT_rightListViewClicked(QListViewItem *)));
	connect(rightListView,SIGNAL(rightButtonClicked(QListViewItem *,const QPoint &,int)),this,SLOT(SLOT_rightListViewRightClicked(QListViewItem *)));
	rightStatusLabel=new QLabel(this);
	rightStatusLabel->setText(i18n("Remote Total"));
	rightStatusLabel->setAlignment( AlignHCenter );
	rightStatusBar=new KStatusBar(rightBox);
	rightStatusBar->addWidget(rightStatusLabel,1);
	rightLayout->addMultiCellWidget(rightToolBar,0,0,0,1);
        rightLayout->addWidget(rightLabel,1,0);
	rightLayout->addWidget(rightLineEdit,1,1);
	rightLayout->addMultiCellWidget(rightListView,2,2,0,1);
	rightLayout->addMultiCellWidget(rightStatusBar,3,3,0,1);
	filterToolBar=new KToolBar(rightBox);
	filterToolBar->insertWidget(0,QString(i18n("  Filter:  ")).length(),new QLabel(i18n("  Filter:  "),filterToolBar));
	filterToolBar->insertLined(NULL,1,SIGNAL(returnPressed(const QString&)),this,SLOT(SLOT_filterRightListView(const QString&)));
	filterToolBar->insertButton("filter",2,SIGNAL(clicked()),this,SLOT(SLOT_filterClicked()),true,i18n("Do Filter"));
	filterToolBar->insertButton("button_cancel",3,SIGNAL(clicked()),this,SLOT(SLOT_showHideFilter()),true,i18n("Close"));
	filterToolBar->alignItemRight(3);
	filterToolBar->hide();

	bottomVSplitter =new QSplitter(hSplitter);
	queueListView=new KListView(bottomVSplitter);
	queueListView->setRootIsDecorated(true);
	queueListView->setSelectionMode(QListView::Extended);
	queueListView->setSorting(-1);
	queueListView->setAllColumnsShowFocus(true);
	connect(queueListView,SIGNAL(rightButtonClicked(QListViewItem *,const QPoint &,int)),this,SLOT(SLOT_queueListViewRightClicked(QListViewItem *)));
	queueListView->addColumn(i18n("Site"));
	queueListView->addColumn(i18n("Remote"));
	queueListView->addColumn(i18n("Local"));
	queueListView->addColumn(i18n("Size"));
	queueListView->addColumn(i18n("Status"));

	rightBottomBox=new QWidget(bottomVSplitter);
        rightBottomLayout=new QGridLayout(rightBottomBox);
        logWindow=new KTextEdit(rightBottomBox);
	//logWindow->setHScrollBarMode( QTextEdit::AlwaysOff );
	logWindow->setTextFormat( QTextEdit::PlainText );
	logWindow->setText(QString("[").append(CURRENTTIME).append(i18n("] KuFTP Started.")));
	logWindow->setReadOnly(true);
	logWindow->setMaxLogLines(5);
	rightBottomLayout->addWidget(logWindow,0,0);
	rawCmdToolBar=new KToolBar(rightBottomBox);
	rawCmdToolBar->insertWidget(0,QString(i18n("  Raw Cmd:  ")).length(),new QLabel(i18n("  Raw Cmd:  "),rawCmdToolBar));
	rawCmdToolBar->insertLined(NULL,1,SIGNAL(returnPressed(const QString&)),this,SLOT(SLOT_sendRawCmd(const QString&)));
	siteCheckBox=new QCheckBox(i18n("Prepend with SITE"),rawCmdToolBar);
	rawCmdToolBar->insertWidget(2,siteCheckBox->width(),siteCheckBox);
	rawCmdToolBar->insertButton("button_cancel",3,SIGNAL(clicked()),this,SLOT(SLOT_showHideRawCmdLine()),true,i18n("Close"));
	rawCmdToolBar->alignItemRight(3);
	rawCmdToolBar->hide();


	bottomStatusBar=new KStatusBar(parent);
	nameLabel=new QLabel(this);
	progressBar=new KProgress(this);
	speedLabel=new QLabel(" xxxxKb/s ",this);
        speedLabel->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
        speedLabel->setMinimumSize(speedLabel->sizeHint());
	elapsedLabel=new QLabel(i18n("xx:xx:xx Elapsed"),this);
        elapsedLabel->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
        elapsedLabel->setMinimumSize(elapsedLabel->sizeHint());
	remainLabel=new QLabel(i18n("xx:xx:xx Remain"),this);
        remainLabel->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
        remainLabel->setMinimumSize(remainLabel->sizeHint());
	bottomStatusBar->addWidget(nameLabel,1,true);
	bottomStatusBar->addWidget(progressBar,0,true);
	bottomStatusBar->addWidget(speedLabel,0,true);
	bottomStatusBar->addWidget(elapsedLabel,0,true);
	bottomStatusBar->addWidget(remainLabel,0,true);
	speedLabel->clear();
	elapsedLabel->clear();
	remainLabel->clear();

	core=new KuFtpCore();
	if(kuftpGlobalSettings.getIsEnableDownloadLimit())
	{
		setMaxDownloadSpeed(kuftpGlobalSettings.getMaxDownloadSpeed());
	}
	else
	{
		setMaxDownloadSpeed(0);
	}
	if(kuftpGlobalSettings.getIsEnableUploadLimit())
	{
		setMaxUploadSpeed(kuftpGlobalSettings.getMaxUploadSpeed());
	}
	else
	{
		setMaxUploadSpeed(0);
	}
	connect(core,SIGNAL(xferedChanged(off64_t)),this,SLOT(SLOT_updateCurrentQueue(off64_t)));
	connect(core,SIGNAL(logChanged(log_t)),this,SLOT(SLOT_setLogQueue(log_t)));
	connect(core,SIGNAL(signalFromCore(retData_t,type)),this,SLOT(SLOT_processCoreSignals(retData_t,type)));
	connect(core,SIGNAL(timeout()),this,SLOT(SLOT_timeout()));
	connect(core,SIGNAL(connectionLost()),this,SLOT(SLOT_connectionLost()));
	connect(this,SIGNAL(readyToTransfer()),SLOT(SLOT_startTransfer()));

	transferTime=new QTime();
	logTimer=new QTimer();
	statusBarTimer=new QTimer();
	idleTimer=new QTimer();
	reConnectTimer=new QTimer();

	connect(statusBarTimer,SIGNAL(timeout()),this,SLOT(SLOT_updateStatusBar()));
	connect(idleTimer,SIGNAL(timeout()),this,SLOT(SLOT_updateIdle()));
	connect(logTimer,SIGNAL(timeout()),this,SLOT(SLOT_updateLogWindow()));
	connect(reConnectTimer,SIGNAL(timeout()),this,SLOT(SLOT_updateReConnectLabel()));
	balloon=NULL;
	
	current.init=0;
	current.port=21;
	current.proxyIndex=-1;
	current.encryptMethod=0;
	current.pasv=true;
	current.correctPasv=false;
	current.lastSite=NULL;
	current.useProxy=false;
	current.connecting=false;
	current.established=false;
	current.transfering=false;
	current.listing=false;
	current.forcedisconnect=false;
	current.retryTimes=kuftpGlobalSettings.getRetryTimes();
	current.retryInterval=kuftpGlobalSettings.getRetryInterval();
	current.retryElapsed=current.retryInterval;
	current.retrying=false;
	current.connectionLost=false;
	current.noop=kuftpGlobalSettings.getIsEnableNoop();
	current.noopInterval=kuftpGlobalSettings.getNoopInterval();
	current.queue=NULL;
	current.queueItem=NULL;
	current.idle=0;
	current.logging=false;
	current.failedTransfer=0;
	current.lastXfered=0;
	current.existXfered=0;
	current.elapsedTime=0;
	current.updateStatusBarCount=0;
	current.transferFromQueue=false;
	current.maybeConnect=false;
	current.get=false;
	current.mirror=false;
	current.toConnectSite=NULL;
	current.stopTransfer=false;
	current.leftListViewSortAscend=true;
	current.leftListViewSortColumn=0;
	current.rightListViewSortAscend=true;
	current.rightListViewSortColumn=0;

	QValueList<int> vl_height;
	vl_height.append(KApplication::desktop()->size().height()*70/100);
	vl_height.append(KApplication::desktop()->size().height()*30/100);
	hSplitter->setSizes(vl_height);
	QValueList<int> vl_width;
	int w=KApplication::desktop()->size().width()/2;
	vl_width.append(w);
	vl_width.append(w);
	topVSplitter->setSizes(vl_width);
	bottomVSplitter->setSizes(vl_width);
	//leftListView->installEventFilter(this);
	//rightListView->installEventFilter(this);
	//queueListView->installEventFilter(this);
	SLOT_updateLeftListView(kuftpGlobalSettings.getDefaultDownloadDir());
	leftListView->setColumnWidth(0,KApplication::desktop()->size().width()/2*65/100);
	leftListView->setColumnWidth(1,KApplication::desktop()->size().width()/2*10/100);
	leftListView->setColumnWidth(2,KApplication::desktop()->size().width()/2*20/100);
}
/*
bool KuFtpMain::eventFilter(QObject *obj,QEvent *e)
{
	if(obj==leftListView&&e->type()==QEvent::Resize)
	{
		leftListView->setColumnWidth(0,leftListView->width()*65/100);
		leftListView->setColumnWidth(1,leftListView->width()*10/100);
		leftListView->setColumnWidth(2,leftListView->width()*20/100);
		return true;
	}else if(obj==rightListView&&e->type()==QEvent::Resize)
	{
		rightListView->setColumnWidth(0,rightListView->width()*50/100);
		rightListView->setColumnWidth(1,rightListView->width()*10/100);
		rightListView->setColumnWidth(2,rightListView->width()*20/100);
		rightListView->setColumnWidth(3,rightListView->width()*18/100);
		return true;
	}
	else if(obj==queueListView&&e->type()==QEvent::Resize)
	{
		queueListView->setColumnWidth(0,queueListView->width()*20/100);
		queueListView->setColumnWidth(1,queueListView->width()*30/100);
		queueListView->setColumnWidth(2,queueListView->width()*30/100);
		queueListView->setColumnWidth(3,queueListView->width()*8/100);
		queueListView->setColumnWidth(4,queueListView->width()*10/100);
		return true;
	}
	else
		return false;
}*/

void KuFtpMain::createAction()
{
	localTransferAct=new QAction(i18n("Transfer"),0,this);
	localQueueAct=new QAction(i18n("Queue"),0,this);
	localMkdirAct=new QAction(i18n("New Folder"),0,this);
	localRenameAct=new QAction(i18n("Rename"),0,this);
	localRemoveAct=new QAction(i18n("Remove"),0,this);
	localRefreshAct=new QAction(i18n("Refresh"),0,this);
	connect(localTransferAct,SIGNAL(activated()),this,SLOT(SLOT_put()));
	connect(localQueueAct,SIGNAL(activated()),this,SLOT(SLOT_putQueue()));
	connect(localMkdirAct,SIGNAL(activated()),this,SLOT(SLOT_mkdir()));
	connect(localRenameAct,SIGNAL(activated()),this,SLOT(SLOT_rename()));
	connect(localRemoveAct,SIGNAL(activated()),this,SLOT(SLOT_remove()));
	connect(localRefreshAct,SIGNAL(activated()),this,SLOT(SLOT_reload()));
	
	remoteTransferAct=new QAction(i18n("Transfer"),0,this);
	remoteQueueAct=new QAction(i18n("Queue"),0,this);
	remoteMkdirAct=new QAction(i18n("New Folder"),0,this);
	remoteRenameAct=new QAction(i18n("Rename"),0,this);
	remoteRmdirAct=new QAction(i18n("Remove Folder"),0,this);
	remoteDelAct=new QAction(i18n("Delete"),0,this);
	remoteRefreshAct=new QAction(i18n("Refresh"),0,this);
	addToBookmarksAct=new QAction(i18n("Add To Bookmarks"),0,this);
	connect(remoteTransferAct,SIGNAL(activated()),this,SLOT(SLOT_get()));
	connect(remoteQueueAct,SIGNAL(activated()),this,SLOT(SLOT_getQueue()));
	connect(remoteMkdirAct,SIGNAL(activated()),this,SLOT(SLOT_remoteMkdir()));
	connect(remoteRenameAct,SIGNAL(activated()),this,SLOT(SLOT_remoteRename()));
	connect(remoteRmdirAct,SIGNAL(activated()),this,SLOT(SLOT_remoteRmdir()));
	connect(remoteDelAct,SIGNAL(activated()),this,SLOT(SLOT_remoteDelete()));
	connect(remoteRefreshAct,SIGNAL(activated()),this,SLOT(SLOT_rightReload()));
	connect(addToBookmarksAct,SIGNAL(activated()),this,SLOT(SLOT_addToBookmarks()));

	queueTransferAct=new QAction(i18n("Transfer"),0,this);
	saveQueueAct=new QAction(i18n("Save Queue"),0,this);
	loadQueueAct=new QAction(i18n("Load Queue"),0,this);
	clearQueueAct=new QAction(i18n("Clear Queue"),0,this);
	removeQueueItemsAct=new QAction(i18n("Remove"),0,this);
	removeFailedQueueItemsAct=new QAction(i18n("Remove Failed"),0,this);
	resetFailedQueueItemsAct=new QAction(i18n("Reset Failed"),0,this);
	movetopAct=new QAction(i18n("Move Top"),0,this);
	moveupAct=new QAction(i18n("Move Up"),0,this);
	movedownAct=new QAction(i18n("Move Down"),0,this);
	movebottomAct=new QAction(i18n("Move Bottom"),0,this);
	connect(queueTransferAct,SIGNAL(activated()),this,SLOT(SLOT_queueTransfer()));
	connect(saveQueueAct,SIGNAL(activated()),this,SLOT(SLOT_saveQueue()));
	connect(loadQueueAct,SIGNAL(activated()),this,SLOT(SLOT_loadQueue()));
	connect(clearQueueAct,SIGNAL(activated()),this,SLOT(SLOT_clearQueue()));
	connect(removeQueueItemsAct,SIGNAL(activated()),this,SLOT(SLOT_removeQueueItems()));
	connect(removeFailedQueueItemsAct,SIGNAL(activated()),this,SLOT(SLOT_removeFailedQueueItems()));
	connect(resetFailedQueueItemsAct,SIGNAL(activated()),this,SLOT(SLOT_resetFailedQueueItems()));
	connect(movetopAct,SIGNAL(activated()),this,SLOT(SLOT_queueMovetop()));
	connect(moveupAct,SIGNAL(activated()),this,SLOT(SLOT_queueMoveup()));
	connect(movedownAct,SIGNAL(activated()),this,SLOT(SLOT_queueMovedown()));
	connect(movebottomAct,SIGNAL(activated()),this,SLOT(SLOT_queueMovebottom()));
}

void KuFtpMain::SLOT_updateLeftListView(const QString & path)
{
	QString cdpath=path;
	cdpath=cdpath.simplifyWhiteSpace();

	QDir cwd(cdpath);
	if (cdpath != "")
	       	if (!cwd.cd(cdpath)) 
			return;
	if (!cwd.exists()) return;
	if(!cwd.isReadable())
		return;
	if(cdpath.endsWith("/")&&cdpath.length()!=1)
		cdpath=cdpath.left(cdpath.length()-1);
	current.localPath=cdpath;
	leftLineEdit->setText(QDir::cleanDirPath(cdpath));

	QListViewItem *tmp;
	while ( ( tmp = leftListView->firstChild()) ) delete tmp;

	int dirSort=0;
	if(current.leftListViewSortColumn==0)
		dirSort=QDir::Name;
	else if(current.leftListViewSortColumn==1)
		dirSort=QDir::Size;
	else
		dirSort=QDir::Time;
	if(current.leftListViewSortAscend)
        	cwd.setSorting( dirSort | QDir::Reversed);
	else
        	cwd.setSorting( dirSort );


	QFileInfo *fi;
	QString filedate;
	off64_t totalsize=0;
	int filenums=0;
	int dirnums=0;

	cwd.setFilter(QDir::Files | QDir::NoSymLinks | QDir::Hidden);
	const QFileInfoList *filelist = cwd.entryInfoList();
	QFileInfoListIterator fit(*filelist);
	fit.atFirst();
        while ( (fi=fit.current())!=0)
        {
		if(!kuftpGlobalSettings.getIsShowHidden())
			if(fi->fileName().startsWith("."))
			{
				++fit;
				continue;
			}
		filenums++;
		off64_t size=fi->size();
		totalsize+=size;
		filedate=fi->lastModified().toString("MMM dd yyyy");
		(new QListViewItem(leftListView,fi->fileName(),formatFileSizeString(size),filedate))->setPixmap(0, KGlobal::iconLoader()->loadIcon("files",KIcon::Small));
                ++fit;
        }
//	leftListView->setSorting(0);
//	leftListView->sort();
	leftListView->setSorting(-1);

	cwd.setFilter(  QDir::Dirs | QDir::NoSymLinks | QDir::Hidden);
	if(current.leftListViewSortAscend)
        	cwd.setSorting( dirSort | QDir::Reversed);
	else
        	cwd.setSorting( dirSort );
        //cwd.setSorting( QDir::Name | QDir::Reversed);
	const QFileInfoList *dirlist = cwd.entryInfoList();
	QFileInfoListIterator dit(*dirlist);
	dit.atFirst();
	while( (fi=dit.current())!=0)
	{
		filedate=fi->lastModified().toString("MM dd yyyy");
		if(fi->fileName()=="."||fi->fileName()=="..")
	       	{
			++dit;
			continue;
		}
		else 
		{
			if(!kuftpGlobalSettings.getIsShowHidden())
				if(fi->fileName().startsWith("."))
				{
					++dit;
					continue;
				}
			dirnums++;
			(new QListViewItem(leftListView,fi->fileName(),"-",filedate))->setPixmap(0, KGlobal::iconLoader()->loadIcon("folder",KIcon::Small));
			++dit;
		}
	}
	QListViewItem* dirup = new QListViewItem(leftListView, "..");
        dirup->setPixmap(0, KGlobal::iconLoader()->loadIcon("folder",KIcon::Small));
        dirup->setSelectable(false);
	leftStatusLabel->setText(QString(i18n("%1 files %2 dirs total %3")).arg(filenums).arg(dirnums).arg(formatFileSizeString(totalsize)));
	leftListView->setColumnWidth(0,leftListView->width()*65/100);
	leftListView->setColumnWidth(1,leftListView->width()*10/100);
	leftListView->setColumnWidth(2,leftListView->width()*20/100);
}
void KuFtpMain::SLOT_cdup()
{
	QString cdpath=current.localPath;
	if(cdpath=="/")
		return;
	else if(cdpath.contains('/')==1)
		SLOT_updateLeftListView("/");
	else
		SLOT_updateLeftListView(cdpath.section('/',0,cdpath.contains('/')-1));
}
void KuFtpMain::SLOT_gohome()
{
	SLOT_updateLeftListView(QDir::homeDirPath());
}
void KuFtpMain::SLOT_reload()
{
	SLOT_updateLeftListView(current.localPath);
}
void KuFtpMain::SLOT_sortLeftListView(int section)
{
	current.leftListViewSortAscend^=true;
	current.leftListViewSortColumn=section;
	SLOT_updateLeftListView(current.localPath);
	current.leftListViewSortColumn=0;
}
void KuFtpMain::SLOT_leftListViewClicked(QListViewItem *item)
{
	QString cdpath=current.localPath;
	if(item->text(0)=="..")
	{
		if(cdpath=="/")
			return;
		if(cdpath.contains('/')==1)
			SLOT_updateLeftListView("/");
		else
			SLOT_updateLeftListView(cdpath.section('/',0,cdpath.contains('/')-1));
	}
	else if(item->text(1)=="-")
	{
		if(cdpath=="/")
			SLOT_updateLeftListView(cdpath.append(item->text(0)));
		else
			SLOT_updateLeftListView(cdpath.append("/").append(item->text(0)));
	}
}
void KuFtpMain::SLOT_leftListViewRightClicked(QListViewItem *)
{
	KPopupMenu cm;

	localTransferAct->addTo(&cm);
	localQueueAct->addTo(&cm);
	cm.insertSeparator();
	localMkdirAct->addTo(&cm);
	localRenameAct->addTo(&cm);
	localRemoveAct->addTo(&cm);
	cm.insertSeparator();
	localRefreshAct->addTo(&cm);

	localTransferAct->setEnabled(false);
	localQueueAct->setEnabled(false);
	localRemoveAct->setEnabled(false);
	if(current.established)
	{
		localQueueAct->setEnabled(true);
		localTransferAct->setEnabled(true);
		if(current.transfering)
		{
			localTransferAct->setEnabled(false);
			localRenameAct->setEnabled(false);
		}
	}
	cm.exec(QCursor::pos());
}
void KuFtpMain::SLOT_put()
{
	SLOT_putQueue();
	current.stopTransfer=false;
	emit readyToTransfer();
}
void KuFtpMain::SLOT_putQueue()
{
	QListViewItemIterator it( leftListView );
	while ( it.current() ) 
	{
		if(it.current()->isSelected()&&it.current()->text(1)!="-") 
		{
			QString remote=current.remotePath;
			QString local=current.localPath;
			local.append("/").append(it.current()->text(0));
			if(remote=="/")
				remote.append(it.current()->text(0));
			else
				remote.append("/").append(it.current()->text(0));
			QFileInfo f(local);
			off64_t size=f.size();
			off64_t xfered=0;
			if(kuftpGlobalSettings.getIsResumeExistFile())
				xfered=getSizeByCache(remote); // not use ftp size cmd
			current.queue->pl_queueItem.append(new KuFtpQueueItem(KuFtpQueueItem::upload,
						remote,local,xfered,size));
		}
		else if(it.current()->isSelected()&&it.current()->text(1)=="-"&&it.current()->text(0)!="..") //scan dir
		{
			QString remote=current.remotePath;
			if(remote=="/")
				remote.append(it.current()->text(0));
			else
				remote.append("/").append(it.current()->text(0));
			current.queue->mkdirList.append(remote);
			scanLocalDir(it.current()->text(0));
		}
		++it;
	}
	if(pl_queue.count()==0)
		pl_queue.append(current.queue);
	SLOT_updateQueueListView();
}
void KuFtpMain::scanLocalDir(QString name)
{
	QString s;
	if(QString(current.localPath)=="/")
		s=QString("/"+QString(name));
	else
		s=QString(current.localPath)+"/"+QString(name);
	QDir dir(s);
	QFileInfoList list;
	list=*dir.entryInfoList();
	QFileInfoListIterator it(list);
	it.atFirst();
	while(it.current())
	{
		QString fileName=it.current()->fileName();
		if(fileName=="."||fileName=="..")
		{
			++it;
			continue;
		}
		QString local=QString(current.localPath).append("/").append(name).append("/").append(fileName);
		QString remote=current.remotePath;
		if(remote=="/")
			remote.append(QString(name)+"/"+fileName);
		else
			remote.append("/").append(QString(name)+"/"+fileName);
		if(it.current()->isDir())
		{
			current.queue->mkdirList.append(remote);
			scanLocalDir(QString(name)+"/"+fileName);
			++it;
		}
		else
		{
			QFileInfo f(local);
			off64_t size=f.size();
			off64_t xfered=0;
			if(kuftpGlobalSettings.getIsResumeExistFile())
				xfered=getSizeByCache(remote);
			current.queue->pl_queueItem.append(new KuFtpQueueItem(KuFtpQueueItem::upload,remote,local,xfered,
						size));
			++it;
		}
	}
}
void KuFtpMain::SLOT_mkdir()
{
	KuFtpEditNameDialog d;
	if(d.exec())
	{
		QString name=d.lineEdit->text().simplifyWhiteSpace();
		if(name.isEmpty())
			return;
		QDir dir;
		QString local=current.localPath;
		if(local=="/")
			local.append(name);
		else
			local.append("/").append(name);
		dir.mkdir(local);
		SLOT_reload();
	}
}
void KuFtpMain::SLOT_rename()
{
	QDir dir(current.localPath);
       	QListViewItemIterator it( leftListView );
	while ( it.current() )
	{
		if ( it.current()->isSelected() )
		{
			KuFtpEditNameDialog d(it.current()->text(0));
			if(d.exec())
			{
				QString newname=d.lineEdit->text().simplifyWhiteSpace();
				if(newname.isEmpty())
					return;
				dir.rename(it.current()->text(0),newname);
			}
		}
		++it;
	}
	SLOT_reload();
}

void KuFtpMain::SLOT_remove()
{
	
	/*
	QDir dir;
       	QListViewItemIterator it( leftListView );
	while ( it.current() )
	{
		if ( it.current()->isSelected() )
		{
		}
		++it;
	}
	SLOT_reload();
	*/
}

void KuFtpMain::SLOT_updateRightListView(const QString &remotePath)
{
	if(!current.established)
		return;
	if(current.transfering)
		return;
	current.remotePath=remotePath;
	rightLineEdit->setText(current.remotePath);
	for(QValueList<dirCache_t>::iterator t=vl_dirCache.begin();t!=vl_dirCache.end();++t)
	{
		if((*t).remotePath==remotePath)
		{
			current.dirCache.remotePath=(*t).remotePath;
			current.dirCache.pl_cacheInfo=(*t).pl_cacheInfo;
			current.dirCache.fileNums=(*t).fileNums;
			current.dirCache.dirNums=(*t).dirNums;
			current.dirCache.totalSize=(*t).totalSize;
			break;
		}
	}
	QRegExp filter(current.filter);

	QListViewItem *tmp;
	while((tmp=rightListView->firstChild())) 
		delete tmp;

	for(KuFtpCacheInfo *t=current.dirCache.pl_cacheInfo.first();t;t=current.dirCache.pl_cacheInfo.next())
	{
		if(!t->getIsDir())
		{
			if(!kuftpGlobalSettings.getIsShowHidden())
				if(t->getName().startsWith("."))
					continue;
			if(!filter.isEmpty())
			{
				if(filter.search(t->getName())!=-1) // matched
					(new QListViewItem(rightListView,t->getName(),formatFileSizeString(t->getSize()),t->getDate(),t->getPermission()))->setPixmap(0, KGlobal::iconLoader()->loadIcon("files",KIcon::Small));
			}
			else
				(new QListViewItem(rightListView,t->getName(),formatFileSizeString(t->getSize()),t->getDate(),t->getPermission()))->setPixmap(0, KGlobal::iconLoader()->loadIcon("files",KIcon::Small));
		}
	}
	rightListView->setSorting(0);
	rightListView->sort();
	rightListView->setSorting(-1);
	for(KuFtpCacheInfo *t=current.dirCache.pl_cacheInfo.last();t;t=current.dirCache.pl_cacheInfo.prev())
	{
		if(t->getIsDir())
		{
			if(!kuftpGlobalSettings.getIsShowHidden())
				if(t->getName().startsWith("."))
					continue;
			if(!filter.isEmpty())
			{
				if(filter.search(t->getName())!=-1) // matched
					(new QListViewItem(rightListView,t->getName(),"-",t->getDate(),t->getPermission()))->setPixmap(0, KGlobal::iconLoader()->loadIcon("folder",KIcon::Small));
			}
			else
				(new QListViewItem(rightListView,t->getName(),"-",t->getDate(),t->getPermission()))->setPixmap(0, KGlobal::iconLoader()->loadIcon("folder",KIcon::Small));
		}
	}
	QListViewItem *dirup=new QListViewItem(rightListView,"..","-","-");
	dirup->setPixmap(0, KGlobal::iconLoader()->loadIcon("folder",KIcon::Small));
	dirup->setSelectable(false);
	rightStatusLabel->setText(QString(i18n("%1 files %2 dirs total %3")).arg(current.dirCache.fileNums).arg(current.dirCache.dirNums).arg(formatFileSizeString(current.dirCache.totalSize)));
	current.filter="";
	rightListView->setColumnWidth(0,rightListView->width()*50/100);
	rightListView->setColumnWidth(1,rightListView->width()*10/100);
	rightListView->setColumnWidth(2,rightListView->width()*20/100);
	rightListView->setColumnWidth(3,rightListView->width()*18/100);
}

void KuFtpMain::SLOT_updateBookmarksSubPopupMenu()
{
	bookmarksSubPopupMenu->clear();
	pl_bmi=KuFtpBookmarks::pl_bmi;
	if(pl_bmi.isEmpty())
		return;
	
	bookmarkActMapper=new QSignalMapper(this);
	connect(bookmarkActMapper,SIGNAL(mapped(int)),this,SLOT(SLOT_connectFromBookmarks(int)));
	
	int i=0;
	pl_bookmarksAct.setAutoDelete(true);
	KuFtpBookmarksItem *topItem;
        for(topItem=pl_bmi.first();topItem;topItem=pl_bmi.next())
	{
		if(topItem->getIsGroup())
		{
			KPopupMenu *topGroupPopupMenu=new KPopupMenu(this);
			KuFtpBookmarksItem *subItem;
			for(subItem=topItem->pl_child.first();subItem;subItem=topItem->pl_child.next())
			{
				QAction *subItemAct=new	QAction(subItem->getSiteName(),0,this);
				subItemAct->setIconSet(KGlobal::iconLoader()->loadIconSet("bookmark",KIcon::Small));
				subItemAct->addTo(topGroupPopupMenu);
				i++;
				connect( subItemAct, SIGNAL( activated() ),bookmarkActMapper,SLOT(map()));
				bookmarkActMapper->setMapping(subItemAct,i);
				bookmarksAct_t *tmp=new bookmarksAct_t;
				tmp->i=i;
				tmp->ptr=subItem;
				pl_bookmarksAct.append(tmp);
			}
			bookmarksSubPopupMenu->insertItem(KGlobal::iconLoader()->loadIconSet("bookmark_folder",KIcon::Small),topItem->getSiteName(),topGroupPopupMenu);
		}
		else
		{
			QAction *topItemAct=new QAction(topItem->getSiteName(),0,this);
			topItemAct->setIconSet(KGlobal::iconLoader()->loadIconSet("bookmark",KIcon::Small));
			topItemAct->addTo(bookmarksSubPopupMenu);
			i++;
			connect( topItemAct, SIGNAL( activated() ),bookmarkActMapper,SLOT(map()));
			bookmarkActMapper->setMapping(topItemAct,i);
			bookmarksAct_t *tmp=new bookmarksAct_t;
			tmp->i=i;
			tmp->ptr=topItem;
			pl_bookmarksAct.append(tmp);
		}
	}
}
void KuFtpMain::SLOT_showHideFilter()
{
	if(filterToolBar->isHidden())
	{
		rightLayout->remove(rightStatusBar);
		rightLayout->addMultiCellWidget(filterToolBar,3,3,0,1);
		rightLayout->addMultiCellWidget(rightStatusBar,4,4,0,1);
		filterToolBar->show();
		filterToolBar->setFocus();
	}
	else
	{
		filterToolBar->hide();
	}
}
void KuFtpMain::SLOT_filterClicked()
{
	filterToolBar->setLinedText(1,NULL);
	current.filter=filterToolBar->getLinedText(1);
	SLOT_updateRightListView(current.remotePath);
}
void KuFtpMain::SLOT_filterRightListView(const QString &string)
{
	filterToolBar->setLinedText(1,NULL);
	current.filter=string;
	SLOT_updateRightListView(current.remotePath);
}
void KuFtpMain::SLOT_showHideRawCmdLine()
{
	if(rawCmdToolBar->isHidden())
	{
		rawCmdToolBar->setLinedText(1,NULL);
		rightBottomLayout->addWidget(rawCmdToolBar,1,0);
		rawCmdToolBar->show();
	}
	else
	{
		rawCmdToolBar->hide();
	}
}
void KuFtpMain::SLOT_exportDirContent()
{
	QString file=KFileDialog::getSaveFileName(QDir::homeDirPath());
	if(file.isEmpty())
		return;
	dirCache_t dirCache;
	for(QValueList<dirCache_t>::iterator t=vl_dirCache.begin();t!=vl_dirCache.end();++t)
	{
		if((*t).remotePath==current.remotePath)
		{
			dirCache.pl_cacheInfo=(*t).pl_cacheInfo;
			break;
		}
	}

	QFile f(file);
	if(f.open(IO_WriteOnly))
	{
		QTextStream stream(&f);
		for(KuFtpCacheInfo *t=dirCache.pl_cacheInfo.first();t;t=dirCache.pl_cacheInfo.next())
		{
			QString permission=QString("%1\t").arg((*t).getPermission());
			QString user=QString("%1\t").arg((*t).getUser());
			QString group=QString("%1\t").arg((*t).getGroup());
			QString size=QString("%1\t").arg((*t).getSize());
			QString date=QString("%1\t").arg((*t).getDate());
			QString filename=QString("%1\t\n").arg((*t).getName());
			stream<<permission<<user<<group<<size<<date<<filename;
		}
		f.close();
	}
}
void KuFtpMain::SLOT_sendRawCmd(const QString &string)
{
	rawCmdToolBar->setLinedText(1,NULL);
	if(siteCheckBox->isChecked())
		core->sendSiteCmd(string);
	else
		core->sendRawCmd(string);
}
//not use
void KuFtpMain::SLOT_forceDisconnect()
{
	if(!current.established)
		return;
	if(kuftpGlobalSettings.getIsConfirmExit())
		if(current.transfering)
			if(!askToConfirm(this,i18n("There's active transfering \nAre you sure to disconnect?"),i18n("Confirm Disconnect-- KuFTP")))
				return;
	current.stopTransfer=true;
	core->stopTransfer();
	core->clearThreadQueue();
	core->stopCheckTimeout();
	rightListView->clear();
	rightListView->setEnabled(true);
	rightLineEdit->clear();
	rightStatusLabel->setText(i18n("Remote Total"));
	idleTimer->stop();
	logTimer->stop();
	nameLabel->clear();
	progressBar->reset();
	speedLabel->clear();
	elapsedLabel->clear();
	remainLabel->clear();
	charsetSubPopupMenu->setEnabled(false);
	showHideFilterAct->setEnabled(false);
	sendRawCmdAct->setEnabled(false);
	exportDirContentAct->setEnabled(false);
	clearDirCache();
	current.transfering=false;
	current.established=false;
	current.queue->setActive(false);
	SLOT_updateQueueListView();
	log_t log={QString("[").append(CURRENTTIME).append(i18n("] Disconnected")),log_t::emer};
	vl_log.append(log);
	SLOT_updateLogWindow();
	emit mainStatusChanged();
}
void KuFtpMain::SLOT_rightCancel()
{
	if(current.connecting)
	{
		logTimer->stop();
		stopRetry();
		logWindow->setColor(Qt::red);
		logWindow->append(QString("[").append(CURRENTTIME).append(i18n("] User Canceled")));
		core->abort();
		current.connecting=false;
		return;
	}
	if(current.retrying)
	{
		logTimer->stop();
		stopRetry();
		logWindow->setColor(Qt::red);
		logWindow->append(QString("[").append(CURRENTTIME).append(i18n("] Retry Canceled")));
		core->abort();
		return;
	}
	if(!current.established)
		return;
	if(!current.transfering&&!current.listing)
		return;
	core->clearThreadQueue();
	current.stopTransfer=true;
	core->stopTransfer();
	nameLabel->clear();
	progressBar->reset();
	speedLabel->clear();
	elapsedLabel->clear();
	remainLabel->clear();
	statusBarTimer->stop();
	current.transfering=false;
	current.listing=false;
	logWindow->setColor(Qt::red);
	logWindow->append(QString("[").append(CURRENTTIME).append(i18n("] Transfer Canceled")));
	if(!rightListView->isEnabled())
	{
		rightListView->setEnabled(true);
	}
}


void KuFtpMain::SLOT_rightCdup()
{
	if(!current.established)
		return;
	if(current.transfering)
		return;
	rightListView->setEnabled(false);
	core->cdup();
	core->pwd();
	current.listing=true;
	core->dir();
}
void KuFtpMain::SLOT_rightReload()
{
	if(!current.established)
		return;
	if(current.transfering)
		return;
	current.remoteCdPath=current.remotePath;
	// remove the dircache so that will do a ftp list
	for(QValueList<dirCache_t>::iterator t=vl_dirCache.begin();t!=vl_dirCache.end();++t)
	{
		if(current.remoteCdPath==(*t).remotePath)
		{
			vl_dirCache.remove(t);
			break;
		}
	}
	rightListView->setEnabled(false);
	core->pwd();
	current.listing=true;
	core->dir();
}
void KuFtpMain::SLOT_charsetChanged(int i)
{
	QStringList charsets = KGlobal::charsets()->descriptiveEncodingNames();
	QString s=KGlobal::charsets()->encodingForName(charsets[i-1]);

	QString t=QString("[").append(CURRENTTIME).append("] ");
	QString s1=QString(i18n("Change Remote Server Encoding to %1")).arg(charsets[i-1]);
//	QString s2=QString(i18n("This change will take effect in next operations"));

	if(s=="iso-10646-ucs-2"||s=="ucs2"||s=="utf16"||s=="utf7")
	{
		QString s3=QString(i18n("Sorry %1 not support yet.")).arg(charsets[i-1]);
		QString s4=QString(i18n("Server Encoding not changed"));
		log_t log1={t+s1,log_t::emer};
		log_t log2={t+s1,log_t::emer};
		vl_log.append(log1);
		vl_log.append(log2);
		return;
	}
	else
	{
		current.descriptiveEncoding=charsets[i-1];
		current.encoding=s;
	}
	int count=charsets.count();
	for(int n=1;n<=count;n++)
		charsetSubPopupMenu->setItemChecked(n,false);
	charsetSubPopupMenu->setItemChecked(i-1,true);
	log_t log={t+s1,log_t::emer};
	vl_log.append(log);
	core->setFtpEncoding(current.encoding);
	clearDirCache();
	SLOT_rightReload();
}
void KuFtpMain::SLOT_remoteMkdir()
{
	if(!current.established)
		return;
	if(current.transfering)
		return;
	KuFtpEditNameDialog d;
	if(d.exec())
	{
		QString name=d.lineEdit->text().simplifyWhiteSpace();
		if(name.isEmpty())
			return;
		core->mkdir(name);
		SLOT_rightReload();
	}
}
void KuFtpMain::SLOT_remoteRmdir()
{
	if(!current.established)
		return;
	if(current.transfering)
		return;
	if(kuftpGlobalSettings.getIsConfirmDeleteFile())
		if(current.transfering||current.established)
			if(!askToConfirm(this,i18n("Do you want to delete?"),i18n("Confirm Delete -- KuFTP")))
				return;
       	QListViewItemIterator it( rightListView );
	while ( it.current() )
	{
		if ( it.current()->isSelected()&&it.current()->text(1)=="-" )
		{
			core->rmdir(it.current()->text(0));
			return;
		}
		++it;
	}
	SLOT_rightReload();
}	
void KuFtpMain::SLOT_remoteRename()
{
	if(!current.established)
		return;
	if(current.transfering)
		return;
	KuFtpEditNameDialog d;
	if(d.exec())
	{
		QString newname=d.lineEdit->text().simplifyWhiteSpace();
		if(newname.isEmpty())
			return;
	       	QListViewItemIterator it( rightListView );
		while ( it.current() )
		{	
			if ( it.current()->isSelected() )
			{
				core->rename(it.current()->text(0),newname);
				SLOT_rightReload();
				return;
			}
			++it;
		}
	}
}
void KuFtpMain::SLOT_remoteDelete()
{
	if(!current.established)
		return;
	if(current.transfering)
		return;
	if(kuftpGlobalSettings.getIsConfirmDeleteFile())
		if(current.transfering||current.established)
			if(!askToConfirm(this,i18n("Do you want to delete?"),i18n("Confirm Delete -- KuFTP")))
				return;
	QListViewItemIterator it(rightListView);
	while( it.current() )
	{
		if(it.current()->isSelected()&&it.current()->text(1)!="-" )
		{
			core->del(it.current()->text(0));
			return;
		}
		++it;
	}
	SLOT_rightReload();
}
void KuFtpMain::SLOT_sortRightListView(int )
{
	// use filter
	/*current.rightListViewSortAscend^=true;
	current.rightListViewSortColumn=section;
	SLOT_updateRightListView(current.remotePath);
	current.rightListViewSortColumn=0;*/
}
void KuFtpMain::SLOT_rightLineEditReturn(const QString &path)
{
	if(!current.established)
		return;
	if(current.transfering)
		return;
	core->chdir(path);
	core->pwd();
	current.listing=true;
	core->dir();
}
void KuFtpMain::SLOT_rightListViewClicked(QListViewItem *item)
{
	if(!current.established)
		return;
	if(current.transfering)
		return;
	if(item->text(0)=="..")
	{
		SLOT_rightCdup();
		return;
	}
	if(item->text(1)!="-")
		return;
	else
	{
		rightListView->setEnabled(false);
		core->chdir(item->text(0));
		core->pwd();
		current.listing=true;
		core->dir();
	}
}
void KuFtpMain::SLOT_rightListViewRightClicked(QListViewItem *)
{
	KPopupMenu cm;

	remoteTransferAct->addTo(&cm);
	remoteQueueAct->addTo(&cm);
	cm.insertSeparator();
	remoteMkdirAct->addTo(&cm);
	remoteRenameAct->addTo(&cm);
	remoteRmdirAct->addTo(&cm);
	remoteDelAct->addTo(&cm);
	cm.insertSeparator();
	addToBookmarksAct->addTo(&cm);
	cm.insertSeparator();
	remoteRefreshAct->addTo(&cm);
	if(!current.established)
	{
		remoteTransferAct->setEnabled(false);
		remoteQueueAct->setEnabled(false);
		remoteMkdirAct->setEnabled(false);
		remoteRenameAct->setEnabled(false);
		remoteDelAct->setEnabled(false);
		remoteRmdirAct->setEnabled(false);
		remoteRefreshAct->setEnabled(false);
		addToBookmarksAct->setEnabled(false);
	}
	else
	{
		remoteQueueAct->setEnabled(true);
		if(current.transfering)
		{
			remoteTransferAct->setEnabled(false);
			remoteMkdirAct->setEnabled(false);
			remoteRenameAct->setEnabled(false);
			remoteDelAct->setEnabled(false);
			remoteRmdirAct->setEnabled(false);
			remoteRefreshAct->setEnabled(false);
		}
		else
		{
			remoteTransferAct->setEnabled(true);
			remoteMkdirAct->setEnabled(true);
			remoteRenameAct->setEnabled(true);
			remoteDelAct->setEnabled(true);
			remoteRmdirAct->setEnabled(true);
			remoteRefreshAct->setEnabled(true);
			addToBookmarksAct->setEnabled(true);
		}
	}
	cm.exec(QCursor::pos());
}
void KuFtpMain::SLOT_get()
{
	current.get=true;
	current.stopTransfer=false;
       	SLOT_getQueue();
}
void KuFtpMain::SLOT_getQueue()
{
       	QListViewItemIterator it( rightListView );
	QStringList mirrorDirs;
	while ( it.current() ) 
	{
		if (it.current()->isSelected()&&it.current()->text(1)!="-" )
		{
			QString remote=current.remotePath;
			QString local=current.localPath;
			if(local=="/")
				local.append(it.current()->text(0));
			else
				local.append("/").append(it.current()->text(0));
			if(remote=="/")
				remote.append(it.current()->text(0));
			else
				remote.append("/").append(it.current()->text(0));
			off64_t xfered=0;
			if(kuftpGlobalSettings.getIsResumeExistFile())
				if(QFile::exists(local))
				{
					QFileInfo f(local);
					xfered=f.size();
				}
			off64_t size=getSizeByCache(remote);
			current.queue->pl_queueItem.append(new KuFtpQueueItem(KuFtpQueueItem::download,remote,local,xfered,size));
		}
		else if(it.current()->isSelected()&&it.current()->text(1)=="-")
		{
			if(current.transfering)
			{
				++it;
				continue;
			}
		//	QString remote=current.remotePath;
		//	if(remote=="/")
		//		remote.append(it.current()->text(0));
		//	else
		//		remote.append("/").append(it.current()->text(0));
			mirrorDirs.append(it.current()->text(0));
		}
		++it;
	}
	if(pl_queue.count()==0)
		pl_queue.append(current.queue);	
	if(mirrorDirs.count())
	{
		current.mirror=true;
		rightBox->setEnabled(false);
		queueListView->setEnabled(false);
		core->setFtpCurrentLocalPath(current.localPath);
		//core->setFtpCurrentRemotePath(current.remotePath);
		core->mirror(mirrorDirs);
		return;
	}
	if(current.get&&current.mirror==false)
	{
		SLOT_updateQueueListView();
		emit readyToTransfer();
		return;
	}
	SLOT_updateQueueListView();
}
void KuFtpMain::SLOT_addToBookmarks()
{
	if(!current.established)
		return;
	KuFtpAddBookmarkDialog d;
	if(d.exec())
	{
		QString name=d.nameLineEdit->text().simplifyWhiteSpace();
		QString groupName=d.groupComboBoxText;//...
		KuFtpBookmarksItem *siteItem=new KuFtpBookmarksItem(name,NULL,current.host,current.port,current.username,current.password,NULL,NULL,0,current.encryptMethod,current.pasv,current.correctPasv,current.descriptiveEncoding);
	       	siteItem->setGroup(false);
		if(groupName=="My Bookmarks")
		{
			(KuFtpBookmarks::pl_bmi).append(siteItem);
			KuFtpBookmarks::writeBookmarks();
			emit bookmarksChanged();
			return;
		}
		KuFtpBookmarksItem *item=NULL;
		for(item=(KuFtpBookmarks::pl_bmi).first();item;item=(KuFtpBookmarks::pl_bmi).next())
		{
			if(item->getIsGroup()&&item->getSiteName()==groupName)
			{
				item->pl_child.append(siteItem);
				KuFtpBookmarks::writeBookmarks();
				emit bookmarksChanged();
				break;
			}
		}
	}
}
void KuFtpMain::SLOT_updateCurrentQueue(off64_t i)
{
	if(current.queueItem==NULL)
		return;
	current.queueItem->setXfered(i);
	current.queueItem->setElapsedTime(transferTime->elapsed());
}
void KuFtpMain::SLOT_updateQueueListView()
{
	QListViewItem *tmp;
	while((tmp=queueListView->firstChild()))
		delete tmp;
	for(KuFtpQueue *p=pl_queue.last();p;p=pl_queue.prev())
	{
		if(p->getIsActive())
		{
			pl_queue.remove(p);
			pl_queue.prepend(p);
			break;
		}
	}
	for(KuFtpQueue *p=pl_queue.last();p;p=pl_queue.prev())
	{
		if(p->pl_queueItem.count()==0)
			continue;
		QString site=QString(p->site->getHost()).append(":%1").arg(p->site->getPort());
		KListViewItem *top=new KListViewItem(queueListView,site,NULL,NULL,NULL,"Queue");
		p->ptr=top;
		if(p==current.queue)
			top->setOpen(true);
		if(p->getIsActive())
			top->setText(4,"Active");
		top->setExpandable(true);
		for(KuFtpQueueItem *i=p->pl_queueItem.last();i;i=p->pl_queueItem.prev())
		{
			if(i->status==KuFtpQueueItem::download)
			{
				KListViewItem *item=new KListViewItem(top,i18n("Download"),i->getRemotePath(),
						i->getLocalPath(),formatFileSizeString(i->getSize()),NULL);
				item->setPixmap(0, KGlobal::iconLoader()->loadIcon("2downarrow",KIcon::Small));
				i->ptr=item;
			}
			else if(i->status==KuFtpQueueItem::upload)
			{
				KListViewItem *item=new KListViewItem(top,i18n("Upload"),i->getRemotePath(),
						i->getLocalPath(),formatFileSizeString(i->getSize()),NULL);
				item->setPixmap(0, KGlobal::iconLoader()->loadIcon("2uparrow",KIcon::Small));
				i->ptr=item;
			}
			if(i->failed)
			{
				i->ptr->setPixmap(0, KGlobal::iconLoader()->loadIcon("remove",KIcon::Small));
				i->ptr->setText(4,i18n("Failed"));
			}
			if(i->transfering)
				i->ptr->setText(4,i18n("Transfering"));
		}
	}
	/*queueListView->setColumnWidth(0,queueListView->width()*20/100);
	queueListView->setColumnWidth(1,queueListView->width()*30/100);
	queueListView->setColumnWidth(2,queueListView->width()*30/100);
	queueListView->setColumnWidth(3,queueListView->width()*8/100);
	queueListView->setColumnWidth(4,queueListView->width()*10/100);*/
}
void KuFtpMain::SLOT_queueListViewRightClicked(QListViewItem *)
{
	KPopupMenu cm;
	queueTransferAct->addTo(&cm);
	cm.insertSeparator();
	saveQueueAct->addTo(&cm);
	loadQueueAct->addTo(&cm);
	cm.insertSeparator();
	KPopupMenu move;
	movetopAct->addTo(&move);
	moveupAct->addTo(&move);
	movedownAct->addTo(&move);
	movebottomAct->addTo(&move);
	cm.insertItem(i18n("Move"),&move);
	cm.insertSeparator();
	removeQueueItemsAct->addTo(&cm);
	removeFailedQueueItemsAct->addTo(&cm);
	resetFailedQueueItemsAct->addTo(&cm);
	clearQueueAct->addTo(&cm);

	movetopAct->setEnabled(true);
	moveupAct->setEnabled(true);
	movedownAct->setEnabled(true);
	movebottomAct->setEnabled(true);

       	QListViewItemIterator it( queueListView );
	QListViewItem *item=NULL;
	bool hasActiveItemSelected=false;
	while( it.current() )
	{
		if ( it.current()->isSelected() )
		{
			item=it.current();
			if((item->parent()==0&&item->text(4)=="Active")||(item->parent()&&item->parent()->text(4)=="Active"))
			{
				hasActiveItemSelected=true;
				break;
			}
		}
		++it;
	}
	if(current.established)
		queueTransferAct->setEnabled(hasActiveItemSelected);
	else
		queueTransferAct->setEnabled(true);
	if(current.transfering)
	{
		queueTransferAct->setEnabled(false);
		clearQueueAct->setEnabled(false);
		removeQueueItemsAct->setEnabled(false);
	}

	if(countOfSelectedListViewItems(queueListView)!=1)
	{
		movetopAct->setEnabled(false);
		moveupAct->setEnabled(false);
		movedownAct->setEnabled(false);
		movebottomAct->setEnabled(false);
	}
	cm.exec(QCursor::pos());
}
void KuFtpMain::SLOT_queueTransfer()
{
	if(current.transfering)
		return;
	current.stopTransfer=false;
	if(!current.established)
	{
		current.transferFromQueue=true;
		for(KuFtpQueue *t=pl_queue.first();t;t=pl_queue.next())
		{
			if((int)t->pl_queueItem.count()==t->getFailedQueueItems())
				continue;
			maybeConnectTo(t->site);
			return;
		}
	}
	else
		SLOT_startTransfer();
}

void KuFtpMain::SLOT_saveQueue()
{
	QString queuefile=KFileDialog::getSaveFileName(QDir::homeDirPath().append("/.kuftp/saved_queues"));
	saveQueue(queuefile);
}
void KuFtpMain::saveQueue(QString filename)
{   
	QDomDocument doc( "KuFtpQueue" );
        QDomElement root = doc.createElement( "Queue" );
        doc.appendChild( root );

	QDomText queueInfoText;
	QDomElement queueInfoElement;
        for(KuFtpQueue *p=pl_queue.first();p;p=pl_queue.next())
        {
		QDomElement queueElement=doc.createElement("queue");

                queueInfoElement=doc.createElement("host");
                queueElement.appendChild( queueInfoElement );
                queueInfoText = doc.createTextNode(p->site->getHost());
                queueInfoElement.appendChild(queueInfoText);

                queueInfoElement=doc.createElement("port");
                queueElement.appendChild( queueInfoElement );
                queueInfoText = doc.createTextNode(QString("%1").arg(p->site->getPort()));
                queueInfoElement.appendChild(queueInfoText);

                queueInfoElement=doc.createElement("username");
                queueElement.appendChild( queueInfoElement );
                queueInfoText = doc.createTextNode(p->site->getUsername());
                queueInfoElement.appendChild(queueInfoText);

                queueInfoElement=doc.createElement("password");
                queueElement.appendChild( queueInfoElement );
                queueInfoText = doc.createTextNode(p->site->getPassword());
                queueInfoElement.appendChild(queueInfoText);

                queueInfoElement=doc.createElement("proxyindex");
                queueElement.appendChild( queueInfoElement );
                queueInfoText = doc.createTextNode(QString("%1").arg(p->site->getProxyIndex()));
                queueInfoElement.appendChild(queueInfoText);

                queueInfoElement=doc.createElement("encryptmethod");
                queueElement.appendChild( queueInfoElement );
                queueInfoText = doc.createTextNode(QString("%1").arg(p->site->getEncryptMethod()));
                queueInfoElement.appendChild(queueInfoText);

                queueInfoElement=doc.createElement("pasv");
                queueElement.appendChild( queueInfoElement );
                queueInfoText = doc.createTextNode(p->site->getIsPasv()?"true":"false");
                queueInfoElement.appendChild(queueInfoText);

                queueInfoElement=doc.createElement("correctpasv");
                queueElement.appendChild( queueInfoElement );
                queueInfoText = doc.createTextNode(p->site->getIsCorrectPasv()?"true":"false");
                queueInfoElement.appendChild(queueInfoText);

                queueInfoElement=doc.createElement("encoding");
                queueElement.appendChild( queueInfoElement );
                queueInfoText = doc.createTextNode(p->site->getEncoding());
                queueInfoElement.appendChild(queueInfoText);

		for(KuFtpQueueItem *i=p->pl_queueItem.first();i;i=p->pl_queueItem.next())
		{
	               	queueInfoElement=doc.createElement("queueitem");
        	        queueElement.appendChild( queueInfoElement );

	                QDomText queueItemInfoText;
	                QDomElement queueItemInfoElement;
	
	                queueItemInfoElement=doc.createElement("remote");
	                queueInfoElement.appendChild( queueItemInfoElement);
	                queueItemInfoText=doc.createTextNode(i->getRemotePath());
	                queueItemInfoElement.appendChild(queueItemInfoText);
	
	                queueItemInfoElement=doc.createElement("local");
	                queueInfoElement.appendChild( queueItemInfoElement);
	                queueItemInfoText=doc.createTextNode(i->getLocalPath());
	                queueItemInfoElement.appendChild(queueItemInfoText);
	
	                queueItemInfoElement=doc.createElement("status");
	                queueInfoElement.appendChild( queueItemInfoElement);
			if(i->status==KuFtpQueueItem::download)
		                queueItemInfoText=doc.createTextNode("download");
			else if(i->status==KuFtpQueueItem::upload)
				queueItemInfoText=doc.createTextNode("upload");
	                queueItemInfoElement.appendChild(queueItemInfoText);

	                queueItemInfoElement=doc.createElement("size");
	                queueInfoElement.appendChild( queueItemInfoElement);
	                queueItemInfoText=doc.createTextNode(QString("%1").arg(i->getSize()));
	                queueItemInfoElement.appendChild(queueItemInfoText);
		}
		root.appendChild(queueElement);
        }
	if(filename.isEmpty())
		filename=QDir::homeDirPath()+"/.kuftp/saved_queues/"+QString(current.host)+"_"+CURRENTTIME+".xml";
	queuelist.append(filename);
        QFile file(filename);
        if( !file.open( IO_WriteOnly ) )
                return;
        QTextStream stream( &file );
        stream << doc.toString();
        file.close();
}

void KuFtpMain::SLOT_loadQueue()
{
	QString queuefile=KFileDialog::getOpenFileName(QDir::homeDirPath().append("/.kuftp/saved_queues"));
        if(queuefile.isEmpty())
                return;

        QDomDocument doc( "KuFtpQueue" );
        QFile file(queuefile);
        if( !file.open( IO_ReadOnly ) ) return ;
        if( !doc.setContent( &file ) )
        {
                file.close();
                return ;
        }
        file.close();
	QDir d;
	d.remove(queuefile);

        QString host;
        unsigned int port=21;
        QString username;
        QString password;
	int proxyIndex=-1;
	int encryptMethod=0;
	bool pasv=true;
	bool correctPasv=false;
	QString encoding;
	QString remote;
	QString local;
	off64_t size=0;
	int status=0;

        QDomElement root = doc.documentElement();
        if( root.tagName() != "Queue" ) return ;
        QDomNode topNode = root.firstChild();

        while( !topNode.isNull() )
        {
                QDomElement topElement = topNode.toElement();
                if( !topElement.isNull() )
                {
			KuFtpQueue *q=new KuFtpQueue;
                        if(topElement.tagName()=="queue")
                        {
                                QDomNode queueNode = topElement.firstChild();
                                while( !queueNode.isNull() )
                                {
                                        QDomElement queueElement=queueNode.toElement();
                                        if(!queueElement.isNull())
                                        {
                                                if(queueElement.tagName()=="host")
                                                        host=queueElement.text();
                                                if(queueElement.tagName()=="port")
                                                {
                                                        port=queueElement.text().toUInt();
                                                        if(port<=0||port>65535)
                                                                port=21;
                                                }
                                                if(queueElement.tagName()=="username")
                                                        username=queueElement.text();
                                                if(queueElement.tagName()=="password")
                                                        password=queueElement.text();
                                                if(queueElement.tagName()=="proxyindex")
                                                        proxyIndex=queueElement.text().toUInt();
                                                if(queueElement.tagName()=="encryptmethod")
                                                        encryptMethod=queueElement.text().toInt();
                                                if(queueElement.tagName()=="pasv")
                                                        pasv=queueElement.text()=="true"?1:0;
                                                if(queueElement.tagName()=="correctpasv")
                                                        correctPasv=queueElement.text()=="true"?1:0;
						if(queueElement.tagName()=="encoding")
							encoding=queueElement.text();
						if(queueElement.tagName()=="queueitem")
						{
							QDomNode queueItemNode=queueElement.firstChild();
							while( !queueItemNode.isNull() )
							{
								QDomElement queueItemInfoElement=queueItemNode.toElement();
								if(!queueItemInfoElement.isNull())
								{
									if(queueItemInfoElement.tagName() == "remote")
										remote=queueItemInfoElement.text();
									if(queueItemInfoElement.tagName() == "local")
										local=queueItemInfoElement.text();
									if(queueItemInfoElement.tagName() == "status")
										status=queueItemInfoElement.text()==
											"download"?0:1;
									if(queueItemInfoElement.tagName() == "size")
										size=(off64_t)queueItemInfoElement.text().
											toLongLong();
								}
								queueItemNode=queueItemNode.nextSibling();
							}
							KuFtpQueueItem *i=new KuFtpQueueItem((KuFtpQueueItem::status_t)
									status,remote,local,0,size);
							q->pl_queueItem.append(i);
						}
                                        }
                                        queueNode=queueNode.nextSibling();
                                }
				q->setSite(new KuFtpSiteInfo(NULL,host,port,username,password,NULL,NULL,proxyIndex,encryptMethod,pasv,correctPasv,encoding));
				pl_queue.append(q);
                        }
                }
                topNode = topNode.nextSibling();
        }
	SLOT_updateQueueListView();
}

void KuFtpMain::SLOT_queueMovetop()
{
	QListViewItem *item=NULL;
       	QListViewItemIterator it( queueListView );
	while( it.current() )
	{
		if ( it.current()->isSelected() )
		{
			item=it.current();
			break;
		}
		++it;
	}
	KuFtpQueue *p=NULL;
	if(item->parent()==NULL)//top item
	{
		if(pl_queue.count()==1)
			return;
		for(p=pl_queue.first();p;p=pl_queue.next())
		{
			if(p->ptr==item) 
			{
				pl_queue.remove(p);
				pl_queue.prepend(p);
				item->moveItem(queueListView->firstChild());// do a screen
				queueListView->firstChild()->moveItem(item);// update
				return;
			}
		}
	}
	else //queue items
	{
		for(p=pl_queue.first();p;p=pl_queue.next())
		{
			for(KuFtpQueueItem *i=p->pl_queueItem.first();i;i=p->pl_queueItem.next())
			{
				if(i->ptr==item)
				{
					if(p->pl_queueItem.count()==1)
						return;
					p->pl_queueItem.remove(i);
					p->pl_queueItem.prepend(i);
					item->moveItem(p->ptr->firstChild());
					p->ptr->firstChild()->moveItem(item);
					return;
				}
			}
		}
	}
}
void KuFtpMain::SLOT_queueMoveup()
{
	QListViewItem *item=NULL;
       	QListViewItemIterator it( queueListView );
	while( it.current() )
	{
		if ( it.current()->isSelected() )
		{
			item=it.current();
			break;
		}
		++it;
	}
	/*volatile*/ int index=0;
	if(item->parent()==NULL)//top item
	{
		for(KuFtpQueue *p=pl_queue.first();p;p=pl_queue.next())
		{
			if(p->ptr==item&&index==0)
				return;
			else if(p->ptr==item&&index==1)
			{
				pl_queue.remove(p);
				pl_queue.prepend(p);
				queueListView->firstChild()->moveItem(item);
				return;
			}
			else if(p->ptr==item)
			{
				pl_queue.remove(p);
				pl_queue.insert(index-1,p);
				item->moveItem(item->itemAbove()->itemAbove());
				return;
			}
			index++;
		}
	}
	else
	{
		for(KuFtpQueue *p=pl_queue.first();p;p=pl_queue.next())
		{
			index=0;
			for(KuFtpQueueItem *i=p->pl_queueItem.first();i;i=p->pl_queueItem.next())
			{
				if(i->ptr==item&&index==0)
					return;
				else if(i->ptr==item&&index==1)
				{
					p->pl_queueItem.remove(i);
					p->pl_queueItem.prepend(i);
					p->ptr->firstChild()->moveItem(item);
					return;
				}
				else if(i->ptr==item)
				{
					p->pl_queueItem.remove(i);
					p->pl_queueItem.insert(index-1,i);
					item->moveItem(item->itemAbove()->itemAbove());
					return;
				}
				index++;
			}
		}
	}
}
void KuFtpMain::SLOT_queueMovedown()
{
	QListViewItem *item=NULL;
       	QListViewItemIterator it( queueListView );
	while( it.current() )
	{
		if ( it.current()->isSelected() )
		{
			item=it.current();
			break;
		}
		++it;
	}
	int n=pl_queue.count()-1;
	/*volatile*/ int index=n;
	if(item->parent()==NULL)//top item
	{
		for(KuFtpQueue *p=pl_queue.last();p;p=pl_queue.prev())
		{
			if(p->ptr==item&&index==n)
				return;
			else if(p->ptr==item&&index==n-1)
			{
				pl_queue.remove(p);
				pl_queue.append(p);
				item->moveItem(item->itemBelow());
				return;
			}
			else if(p->ptr==item)
			{
				pl_queue.remove(p);
				pl_queue.insert(index+1,p);
				item->moveItem(item->itemBelow());
				return;
			}
			index--;
		}
	}
	else
	{
		for(KuFtpQueue *p=pl_queue.first();p;p=pl_queue.next())
		{
			index=p->pl_queueItem.count();
			for(KuFtpQueueItem *i=p->pl_queueItem.last();i;i=p->pl_queueItem.prev())
			{
				if(i->ptr==item&&index==n)
					return;
				else if(i->ptr==item&&index==n-1)
				{
					p->pl_queueItem.remove(i);
					p->pl_queueItem.append(i);
					item->moveItem(item->itemBelow());
					return;
				}
				else if(i->ptr==item)
				{
					p->pl_queueItem.remove(i);
					p->pl_queueItem.insert(index+1,i);
					item->moveItem(item->itemBelow());
					return;
				}
				index--;
			}
		}
	}
}
void KuFtpMain::SLOT_queueMovebottom()
{
	QListViewItem *item=NULL;
       	QListViewItemIterator it( queueListView );
	while( it.current() )
	{
		if ( it.current()->isSelected() )
		{
			item=it.current();
			break;
		}
		++it;
	}
	KuFtpQueue *p=NULL;
	if(item->parent()==NULL)//top item
	{
		if(pl_queue.count()==1)
			return;
		for(p=pl_queue.first();p;p=pl_queue.next())
		{
			if(p->ptr==item) 
			{
				pl_queue.remove(p);
				pl_queue.append(p);
				item->moveItem(queueListView->lastItem()->parent());
				return;
			}
		}
	}
	else //queue items
	{
		for(p=pl_queue.first();p;p=pl_queue.next())
		{
			for(KuFtpQueueItem *i=p->pl_queueItem.first();i;i=p->pl_queueItem.next())
			{
				if(i->ptr==item)
				{
					if(p->pl_queueItem.count()==1)
						return;
					p->pl_queueItem.remove(i);
					p->pl_queueItem.append(i);
					QListViewItem *t=p->ptr->firstChild();
					QListViewItem *last=t;
					while((t=t->nextSibling()))
						last=t;
					item->moveItem(last);
					return;
				}
			}
		}
	}
}
void KuFtpMain::SLOT_removeQueueItems()
{
	if(kuftpGlobalSettings.getIsConfirmDeleteQueueItem())
		if(current.transfering||current.established)
			if(!askToConfirm(this,i18n("Do you want to delete?"),i18n("Confirm Delete -- KuFTP")))
				return;
       	QListViewItemIterator it( queueListView );
	while( it.current() )
	{
		if ( it.current()->isSelected() )
		{
			for(KuFtpQueue *p=pl_queue.first();p;p=pl_queue.next())
			{
				if(p->ptr==it.current())
				{
					p->pl_queueItem.clear();
					pl_queue.remove(p);
					break;
				}
				for(KuFtpQueueItem *i=p->pl_queueItem.first();i;i=p->pl_queueItem.next())
				{
					if(i->ptr==it.current())
					{
						p->pl_queueItem.remove(i);
						break;
					}
				}
			}
		}
		++it;
	}
	SLOT_updateQueueListView();
}
void KuFtpMain::SLOT_removeFailedQueueItems()
{
	for(KuFtpQueue *p=pl_queue.first();p;p=pl_queue.next())
	{
		for(KuFtpQueueItem *i=p->pl_queueItem.first();i;i=p->pl_queueItem.next())
		{
			if(i->failed)
				p->pl_queueItem.remove(i);
		}
		if(p->pl_queueItem.count()==0)
			pl_queue.remove(p);
	}
	SLOT_updateQueueListView();
}
void KuFtpMain::SLOT_resetFailedQueueItems()
{
	for(KuFtpQueue *p=pl_queue.first();p;p=pl_queue.next())
	{
		for(KuFtpQueueItem *i=p->pl_queueItem.first();i;i=p->pl_queueItem.next())
		{
			if(i->failed)
				i->failed=false;
		}
	}
	SLOT_updateQueueListView();
}
void KuFtpMain::SLOT_clearQueue()
{
	if(kuftpGlobalSettings.getIsConfirmClearQueue())
		if(current.transfering||current.established)
			if(!askToConfirm(this,i18n("Do you want to clear?"),i18n("Confirm Clear Queue -- KuFTP")))	
				return;
	queueListView->clear();
	for(KuFtpQueue *p=pl_queue.first();p;p=pl_queue.next())
		p->pl_queueItem.clear();
	pl_queue.clear();
	current.failedTransfer=0;
}

void KuFtpMain::SLOT_setLogQueue(log_t log)
{
	vl_log.append(log);
}

void KuFtpMain::SLOT_updateLogWindow()
{
//	QMutex mutex;
//	mutex.lock();
	if(current.logging)
		return;
	current.logging=true;
	while(!vl_log.empty())
  	{
		log_t t=vl_log.front();
		vl_log.pop_front();
		if(t.type==log_t::out)
			logWindow->setColor(kuftpGlobalSettings.getInColor());
		else if(t.type==log_t::in)
			logWindow->setColor(kuftpGlobalSettings.getOutColor());
		else if(t.type==log_t::emer)
			logWindow->setColor(Qt::red);
		else
			logWindow->setColor(Qt::black);
		QString s(t.string);
		logWindow->append(s);
	}
	current.logging=false;
//	mutex.unlock();
}
void KuFtpMain::SLOT_connectDialog()
{
	stopRetry();
	if(kuftpGlobalSettings.getIsConfirmExit())
		if(current.transfering||current.established)
			if(!askToConfirm(this,i18n("Transfering Or Established Connection\nDiscard This and Connect?"),i18n("Confirm Connect -- KuFTP")))
				return;
	if(current.established)
		SLOT_disconnect();
	KuFtpConnectDialog kcd;
	if(kcd.exec())
	{		
		QString host=kcd.hostLineEdit->text();
		int port=kcd.portSpinBox->value();
		QString username=kcd.usernameLineEdit->text();
		QString password=kcd.passwordEdit->password();
		QString remotePath=kcd.remotePathLineEdit->text();
		int proxyIndex=kcd.proxyComboBox->currentItem();
		bool pasv=kcd.modeCheckBox->isChecked();
		bool correctPasv=kcd.correctPasvCheckBox->isChecked();
		int encryptMethod=kcd.getEncryptMethod();
		QString encoding=kcd.codecComboBox->currentText();
		KuFtpSiteInfo *site=new KuFtpSiteInfo(NULL,host,port,username,password,remotePath,NULL,proxyIndex,encryptMethod, pasv,correctPasv,encoding);
		connectTo(site);
	}
}
void KuFtpMain::SLOT_connectFromBookmarks(int i)
{
	stopRetry();
	KuFtpBookmarksItem *t=NULL;
	for(bookmarksAct_t *tmp=pl_bookmarksAct.first();tmp;tmp=pl_bookmarksAct.next())
		if(tmp->i==i)
		{
			t=tmp->ptr;
			break;
		}
	maybeConnectTo(new KuFtpSiteInfo(NULL,t->getHost(),t->getPort(),t->getUsername(),t->getPassword(),t->getRemotePath(),t->getLocalPath(),t->getProxyIndex(),t->getEncryptMethod(),t->getIsPasv(), t->getIsCorrectPasv(),t->getEncoding()));
}
void KuFtpMain::SLOT_reConnect()
{
	stopRetry();
	if(current.lastSite==NULL)
		return;
	maybeConnectTo(current.lastSite);
}
void KuFtpMain::maybeConnectTo(KuFtpSiteInfo *site)
{
	if(site==NULL)
		return;
	if((!current.retrying)&&(!current.transferFromQueue)&&kuftpGlobalSettings.getIsConfirmExit())
		if(current.transfering||current.established)
			if(!askToConfirm(this,i18n("Transfering Or Established Connection\nDiscard This and Connect?"),i18n("Confirm Connect -- KuFTP")))
				return;
	current.toConnectSite=site;
	if(current.established)
	{
		current.maybeConnect=true;
		SLOT_disconnect();
	}
	else
		connectTo(site);
}
void KuFtpMain::connectTo(KuFtpSiteInfo *site)
{
	if(site==NULL)
		return;

	current.host=site->getHost();
	current.port=site->getPort();
	if(current.port<1||current.port>65535)
		current.port=21;
	current.username=site->getUsername();
	if(current.username.stripWhiteSpace().isEmpty())
		current.username=QString("anonymous");
	current.password=site->getPassword();
	if(current.password.stripWhiteSpace().isEmpty())
		current.password=kuftpGlobalSettings.getEmail();
	current.remoteMaybeCdPath=site->getRemotePath();
	current.proxyIndex=site->getProxyIndex();
	current.encryptMethod=site->getEncryptMethod();
	current.pasv=site->getIsPasv();
	current.correctPasv=site->getIsCorrectPasv();
	QString s=KGlobal::charsets()->encodingForName(site->getEncoding());
	if(s=="iso-10646-ucs-2"||s=="ucs2"||s=="utf16"||s=="utf7")
	{
		current.descriptiveEncoding="Unicode ( utf8 )";
		current.encoding="utf8";
	}
	else
	{
		current.descriptiveEncoding=site->getEncoding();
		current.encoding=s;
	}
	current.lastSite=site;
	if(current.init==true)
		connectPopupMenu->removeItemAt(0);
	connectPopupMenu->insertItem(QString(i18n("ReConnect ")).append(current.host),this,SLOT(SLOT_reConnect()),0,-1,0);
	current.init=true;
	core->setFtpEncoding(current.encoding);
	core->setEncryptMethod(current.encryptMethod);
	core->setNetworkTimeout(kuftpGlobalSettings.getNetworkTimeout());
	proxy_t *proxy=NULL;
	current.connecting=true;
	logTimer->start(100);
	if(current.proxyIndex==0||(current.proxyIndex==1&&kuftpGlobalSettings.pl_proxy.count()==0))
	{
		log_t log={QString("[").append(CURRENTTIME).append(i18n("] Connecting %1...\n")).arg(current.host),log_t::common};
		vl_log.append(log);
		core->setFtpUseProxy(false);
		core->connect(current.host,current.port);
	}
	else 
	{
		if(current.proxyIndex==1)//use default proxy
		{
			for(proxy=kuftpGlobalSettings.pl_proxy.first();proxy;proxy=kuftpGlobalSettings.pl_proxy.next())
			{
				if(proxy->isDefault)
					break;
			}
			if(proxy==NULL)
				proxy=kuftpGlobalSettings.pl_proxy.first();
		}
		else
		{
			proxy=kuftpGlobalSettings.pl_proxy.at(current.proxyIndex-2);
		}
		log_t log={QString("[").append(CURRENTTIME).append(i18n("] Connecting %1 through proxy %2:%3...\n")).arg(current.host).arg(proxy->host).arg(proxy->port),log_t::common};
		vl_log.append(log);
		core->setFtpUseProxy(true);
		core->connect(current.host,current.port,proxy);
	}
	setQueue(site);
	core->setFtpCurrentQueue(current.queue);
	core->setFtpDirCacheValueList(&vl_dirCache);
}

void KuFtpMain::afterEstablished()
{
	core->systype();
	if(!current.pasv)
		core->setConnMode(port);
	if(current.correctPasv)
		core->correctPasv();

	emit mainStatusChanged();
	rightListView->setEnabled(false);
//	current.remoteCdPath="/";
//	current.remotePath="/";
	if(!current.remoteMaybeCdPath.stripWhiteSpace().isEmpty())
	{
		core->chdir(current.remoteMaybeCdPath.stripWhiteSpace());
	}
	core->pwd();
	current.listing=true;
	core->dir();

	charsetSubPopupMenu->setEnabled(true);
	charsetSubPopupMenu->clear();
	showHideFilterAct->setEnabled(true);
	sendRawCmdAct->setEnabled(true);
	exportDirContentAct->setEnabled(true);
	QStringList charsets = KGlobal::charsets()->descriptiveEncodingNames();
	int count = 0;
	for (QStringList::iterator i=charsets.begin();i!=charsets.end();++i)
	{
		charsetSubPopupMenu->insertItem(*i,this,SLOT(SLOT_charsetChanged(int)),0,++count);
		if(KGlobal::charsets()->encodingForName(*i)==current.encoding)
			charsetSubPopupMenu->setItemChecked(count,true);
	}
	SLOT_updateQueueListView();
	if(current.transferFromQueue)//if start transfer from queue and no connection established,then after this 
		SLOT_startTransfer();//establish transfer will be called
}
void KuFtpMain::SLOT_timeout()
{
	//SLOT_disconnect();
	afterQuit();
}
void KuFtpMain::SLOT_connectionLost()
{
	current.connectionLost=true;
	current.maybeConnect=false;
	log_t log={QString("[").append(CURRENTTIME).append(i18n("] Network Timeout.")),log_t::emer};
	vl_log.append(log);
	afterQuit();
	if(current.queue!=NULL&&(int)current.queue->pl_queueItem.count()!=current.queue->getFailedQueueItems())
	{
		current.transferFromQueue=true;
		if(current.queueItem!=NULL)
		{
			current.queueItem->transfering=false;
		}
		SLOT_reConnect();
	}
	current.connectionLost=false;
}
void KuFtpMain::afterQuit()
{
	//core->abort();
	core->clearThreadQueue();
	core->stopCheckTimeout();
	rightListView->clear();
	rightListView->setEnabled(true);
	rightLineEdit->clear();
	rightStatusLabel->setText(i18n("Remote Total"));
	idleTimer->stop();
	logTimer->stop();
	nameLabel->clear();
	progressBar->reset();
	speedLabel->clear();
	elapsedLabel->clear();
	remainLabel->clear();
	charsetSubPopupMenu->setEnabled(false);
	showHideFilterAct->setEnabled(false);
	sendRawCmdAct->setEnabled(false);
	exportDirContentAct->setEnabled(false);
	clearDirCache();
	current.transfering=false;
	current.established=false;
	current.queue->setActive(false);
	SLOT_updateQueueListView();
	log_t log={QString("[").append(CURRENTTIME).append(i18n("] Disconnected")),log_t::emer};
	vl_log.append(log);
	SLOT_updateLogWindow();

	if(current.maybeConnect)
	{
		current.maybeConnect=false;
		connectTo(current.toConnectSite);
		return;
	}
	emit mainStatusChanged();
}
void KuFtpMain::SLOT_disconnect()
{
	if(!current.established)
		return;
	if(kuftpGlobalSettings.getIsConfirmExit())
		if(current.transfering)
			if(!askToConfirm(this,i18n("There's active transfering \nAre you sure to disconnect?"),i18n("Confirm Disconnect-- KuFTP")))
				return;
	current.stopTransfer=true;
	core->stopTransfer();
	core->clearThreadQueue();
	core->quit();
}
void KuFtpMain::stopRetry()
{
	reConnectTimer->stop();
	nameLabel->clear();
	current.retryInterval=kuftpGlobalSettings.getRetryInterval();
	current.retryTimes=kuftpGlobalSettings.getRetryTimes();
	current.retryElapsed=current.retryInterval;
	current.retrying=false;
}
void KuFtpMain::setMaxDownloadSpeed(int i)
{
	core->setMaxDownloadSpeed(i);
}
void KuFtpMain::setMaxUploadSpeed(int i)
{
	core->setMaxUploadSpeed(i);
}
void KuFtpMain::SLOT_startTransfer()
{
	if(current.transfering)
		return;
	idleTimer->stop();
	for(KuFtpQueueItem *t=current.queue->pl_queueItem.first();t;t=current.queue->pl_queueItem.next())
	{
		current.queueItem=t;
		if(t->failed)
			continue;
		else if(t->status==KuFtpQueueItem::download)
		{
			progressBar->setTotalSteps(t->getSize());//remote size
			progressBar->setProgress(t->getXfered());//local size
			current.existXfered=t->getXfered();
			transferTime->restart();
			statusBarTimer->start(1000);
			t->ptr->setText(4,i18n("Transfering"));
			t->transfering=true;

			QDir dir("/");
			QString local=t->getLocalPath();
			int contains=QString(local).contains("/");
			int i=1;
			while(i!=contains)
			{
				QString t=QString(local).section("/",0,i);
				if(dir.exists(t))
				{
					i++;
					continue;
				}
				else
				{
					dir.mkdir(t);
					i++;
					continue;
				}
			}
			QString remote=t->getRemotePath();
			remote.remove(0,current.remotePath.length()==1?1:current.remotePath.length()+1);
			current.transfering=true;
			core->get(t->getLocalPath(),remote,image,t->getXfered());
			return;
		}
		else if(t->status==KuFtpQueueItem::upload)
		{
			progressBar->setTotalSteps(t->getSize());//local size
			progressBar->setProgress(t->getXfered());//remote size
			current.existXfered=t->getXfered();
			transferTime->restart();
			statusBarTimer->start(1000);
			t->ptr->setText(4,i18n("Transfering"));
			t->transfering=true;

			current.queue->mkdirList.sort();
			QStringList sl(current.queue->mkdirList);
			current.queue->mkdirList.clear();
			for(QStringList::Iterator i=sl.begin();i!=sl.end();++i)
			{
				if(QString(t->getRemotePath()).startsWith(*i))
				{
					core->mkdir(*i);
				}
				else
					current.queue->mkdirList.append(*i);
			}
			QString remote=t->getRemotePath();
			remote.remove(0,current.remotePath.length()==1?1:current.remotePath.length()+1);
			current.transfering=true;
			core->put(t->getLocalPath(),remote,image,t->getXfered());
			return;
		}
	}
}
void KuFtpMain::SLOT_processCoreSignals(retData_t ret,type tasktype)
{
	int success=ret.success;
	QString string=ret.string;
	switch(tasktype)
	{
		case FTP_CONNECT:
		case FTP_CONNECT_USE_PROXY:
			if(success)
			{
				current.connecting=false;
				log_t log={QString("[").append(CURRENTTIME).append(i18n("] Connected with server.")),log_t::common};
				vl_log.append(log);
				reConnectTimer->stop();
				if(current.encryptMethod>0)
					core->auth();
				else
					core->login(current.username,current.password);
			}
			else				
			{
				log_t log={QString("[").append(CURRENTTIME).append(i18n("] Cannot connect to %1.")).arg(current.host),log_t::emer};
				vl_log.append(log);
				if(current.retryTimes-->=0)
				{
					QString s(QString("[").append(CURRENTTIME).append("] ").append(i18n("ReConnect %1 after %2 seconds(times %3)").arg(current.host).arg(current.retryInterval).arg(kuftpGlobalSettings.getRetryTimes()-current.retryTimes-1)));
					log_t log={s,log_t::common};
					vl_log.append(log);
					current.retryElapsed=current.retryInterval;
					current.retrying=true;
					setReConnectLabel();
				}
				else
				{
					stopRetry();
					log_t log={QString("[").append(CURRENTTIME).append("] ").append(i18n("Give up ReConnect.Not connected")),log_t::emer};
					vl_log.append(log);
					core->abort();
				}
			}
			break;
		case FTP_AUTH:
			if(success)
				core->login(current.username,current.password);
			else
			{
				log_t log={QString("[").append(CURRENTTIME).append(i18n("] Some error in negotiation with %1.")).arg(current.host),log_t::emer};
				vl_log.append(log);
				core->abort();
			}
			break;
		case FTP_LOGIN:
			if(success)
			{
				if(kuftpGlobalSettings.getIsShowBalloonWhenConnectionEstablished())
				{
					if((KuFtp::systemTray)->isShown()&&(!kapp->mainWidget()->isActiveWindow()))
					{
						QString s("<qt><nobr><b>KuFtp</b><br>"+current.host+i18n(" Connected")+
								"</nobr></qt>");
						if(balloon)
						{
							balloon->hide();
							delete balloon;
						}
						balloon=new KuFtpBalloon(s,"info");
						connect(balloon,SIGNAL(signalBalloonClicked()),this,SLOT(SLOT_hideBalloon()));
						balloon->setAnchor((KuFtp::systemTray)->mapToGlobal((KuFtp::systemTray)->pos()));
						balloon->show();
						QTimer::singleShot(4000,this,SLOT(SLOT_hideBalloon()));
					}
				}
				current.retrying=false;
				stopRetry();
				current.established=true;
				afterEstablished();
			}
			else
			{
				if(current.retryTimes-->=0)
				{
					QString s(QString("[").append(CURRENTTIME).
						append("] ").append(i18n("ReConnect %1 after %2 seconds(times %3)").
							arg(current.host).arg(current.retryInterval).
							arg(kuftpGlobalSettings.getRetryTimes()-current.retryTimes-1)));
					log_t log={s,log_t::common};
					vl_log.append(log);
					current.retryElapsed=current.retryInterval;
					current.retrying=true;
					setReConnectLabel();
				}
				else
				{
					stopRetry();
					nameLabel->clear();
					log_t log={QString("[").append(CURRENTTIME).append("] ").append(i18n("Give up ReConnect.Not connected")),log_t::emer};
					vl_log.append(log);
					core->abort();
				}
			}
			break;
		case FTP_PWD:
			if(success)
			{
				current.remotePath=string;
			}
			break;
		case FTP_SYSTYPE:
		case FTP_CHDIR:
		case FTP_CDUP:
		case FTP_RMDIR:
		case FTP_MKDIR:
		case FTP_SIZE:
		case FTP_FEAT:
		case FTP_RENAME:
		case FTP_DEL:
		case FTP_CONNECTIONLOST:
		case FTP_TIMEOUT:
		case FTP_XFERED:
		case FTP_LOG:
			break;
		case FTP_NOOP:
		case FTP_RAW:
		case FTP_SITE:
			setIdle();
			break;
		case FTP_QUIT:
			afterQuit();
			break;
		case FTP_DIR:
			current.listing=false;
			SLOT_updateRightListView(current.remotePath);
			setIdle();
			rightListView->setEnabled(true);
			break;
		case FTP_MIRROR:
			SLOT_updateQueueListView();
			rightBox->setEnabled(true);
			queueListView->setEnabled(true);
			current.mirror=false;
			if(current.get)
			{
				current.get=false;
				emit readyToTransfer();
			}
			else
			{
				setIdle();
			}
			break;
		case FTP_GET:
		case FTP_PUT:
			if(kuftpGlobalSettings.getIsShowBalloonWhenTransferFinished())
			{
				if((KuFtp::systemTray)->isShown()&&(!kapp->mainWidget()->isActiveWindow()))
				{
					QString s("<qt><nobr><b>KuFtp</b><br>");
					if(current.queueItem->status==KuFtpQueueItem::download)
					{
						s.append(QString(current.queueItem->getRemotePath()).section('/',-1));
						if(success)
						{
							s.append(i18n(" Download Succeed"));
						}
						else
						{
							s.append(i18n(" Download Failed"));
						}
					}
					else if(current.queueItem->status==KuFtpQueueItem::upload)
					{
						s.append(QString(current.queueItem->getLocalPath()).section('/',-1));
						if(success)
						{
							s.append(i18n(" Upload Succeed"));
						}
						else
						{
							s.append(i18n(" Upload Failed"));
						}
					}
					if(balloon)
					{
						balloon->hide();
						delete balloon;
					}
					balloon=new KuFtpBalloon(s.append("</nobr></qt>"),"info");
					connect(balloon,SIGNAL(signalBalloonClicked()),this,SLOT(SLOT_hideBalloon()));
					balloon->setAnchor((KuFtp::systemTray)->mapToGlobal((KuFtp::systemTray)->pos()));
					balloon->show();
					QTimer::singleShot(4000,this,SLOT(SLOT_hideBalloon()));
				}
			}
			elapsedLabel->clear();
			progressBar->setProgress(0);
			nameLabel->clear();
			speedLabel->clear();
			remainLabel->clear();
			statusBarTimer->stop();
			current.updateStatusBarCount=0;
			current.get=false;
			setIdle();
			if(success)
			{		
				if(current.queue->pl_queueItem.remove(current.queueItem))
				{
					current.transfering=false;
					if(current.queue->pl_queueItem.count()!=current.failedTransfer)
						emit readyToTransfer();
				}
				SLOT_updateQueueListView();
				SLOT_reload();
			}
			else
			{
			/*	if(tasktype==FTP_GET)
				{
					logWindow->setColor(Qt::red);
					logWindow->append(QString("[").append(CURRENTTIME).
							append("] ").append(QString("Download %1 Failed. System says:").
								arg(current.queueItem->getRemotePath())));
				}*/
				current.queueItem->failed=true;
				current.queueItem->transfering=false;
				current.failedTransfer++;
				SLOT_updateQueueListView();
				current.transfering=false;
				if((!current.stopTransfer)&&current.queue->pl_queueItem.count()-current.failedTransfer!=0)
					emit readyToTransfer();
			}
			current.queue->setFailedQueueItems(current.failedTransfer);
			if(current.stopTransfer==false)
			{
				if(current.queue->pl_queueItem.count()==current.failedTransfer&&current.transferFromQueue)
				{
					for(KuFtpQueue *t=pl_queue.first();t;t=pl_queue.next())
					{
						if((int)t->pl_queueItem.count()==t->getFailedQueueItems())
							continue;
						maybeConnectTo(t->site);
						break;
					}
				}
			}
			break;
	}
}

void KuFtpMain::SLOT_updateIdle()
{
	current.idle++;
	nameLabel->setText(QString(current.host).append(i18n(" Idle: ")).append(QTime().addSecs(current.idle).toString(QString("hh:mm:ss")).section(':',1,2)));
	if(current.noop==true)
		if(current.idle==current.noopInterval)
		{
			idleTimer->stop();
			nameLabel->clear();
			current.idle=0;
			core->noop();
		}
}
void KuFtpMain::SLOT_updateReConnectLabel()
{
	nameLabel->setText(i18n("ReConnect %1 after %2 seconds").arg(current.host).arg(current.retryElapsed--));
	if(current.retryElapsed<0)
	{
		reConnectTimer->stop();
		nameLabel->clear();
		maybeConnectTo(current.lastSite);
	}
}
void KuFtpMain::SLOT_updateStatusBar()
{
	QTime t;
	elapsedLabel->setText(t.addSecs(++current.updateStatusBarCount).toString(QString("hh:mm:ss")).append(i18n("Elapsed")));
	off64_t xfered=current.queueItem->getXfered();
	off64_t thisXfered=current.queueItem->getXfered()-current.lastXfered; // Bytes
	if(thisXfered==0)
	{
		speedLabel->setText(i18n("Stalled"));
		return;
		thisXfered++;
	}
	int elapsedTime=current.queueItem->getElapsedTime();
	int thisTime=current.queueItem->getElapsedTime()-current.elapsedTime; // milliseconds
	if(thisTime==0)
	{
		speedLabel->setText(i18n("Stalled"));
		return;
		thisTime++;
	}
	off64_t speed=thisXfered*1024/thisTime;  // about Bytes/S
	if(speed==0)
		speed++;
	off64_t total=current.queueItem->getSize();
	off64_t remainToXfer=total-xfered-current.existXfered; 
	off64_t remainTime=remainToXfer/speed; //seconds
	current.lastXfered=xfered;
	current.elapsedTime=elapsedTime;

	QString s=current.queueItem->getRemotePath();
	QString filename=s.section('/',s.contains('/'),s.contains('/'));
	if(filename.length()>50)
		filename=filename.left(47).append("...");
	QString perc=QString("%1/%2").arg(formatFileSizeString(xfered+current.existXfered)).arg(formatFileSizeString(total));
	nameLabel->setText(QString(filename).append("   %1").arg(perc));
	progressBar->setProgress(xfered+current.existXfered);
	QString speedString;
	if(speed<1000)
		speedString=QString(i18n("%1 Bytes/s")).arg(speed);
	else if(speed<1000*1000)
		speedString=QString(i18n("%1 KB/s")).arg(speed/1000);
	else if(speed<1000*1000*1000)
		speedString=QString(i18n("%1 MB/s")).arg(speed/1000/1000);
	else
		speedString=QString(i18n("%1 GB/s")).arg(speed/1000/1000/1000);
	speedLabel->setText(speedString);
	current.speedString=speedString;
	t=QTime(0,0,0,0);
	QString remainTimeString=t.addSecs(remainTime).toString(QString("hh:mm:ss"));
	remainLabel->setText(QString(remainTimeString).append(i18n("Remain")));
	current.remainTimeString=remainTimeString;
}


bool KuFtpMain::askToConfirm(QWidget *parent,QString text,QString caption)
{
	bool i=(KMessageBox::questionYesNoCancel(parent,text,caption)==KMessageBox::Yes);
	return i;
}

void KuFtpMain::setIdle()
{
	current.idle=0;
	nameLabel->setText(QString(current.host).append(i18n(" Idle: 00:00")));
	core->startCheckTimeout();
	idleTimer->start(1000);
}
void KuFtpMain::setQueue(KuFtpSiteInfo *site)
{
	for(KuFtpQueue *p=pl_queue.first();p;p=pl_queue.next())
	{
		if(p->site->getHost()==site->getHost()&&p->site->getPort()==site->getPort()
				&&p->site->getUsername()==site->getUsername())
		{
			if(current.queue)
				current.queue->setActive(false);
			current.queue=p;
			current.queue->setActive(true);
			return;
		}
	}
	KuFtpQueue *q=new KuFtpQueue(site);
	q->setActive(true);
	if(current.queue)
		current.queue->setActive(false);
	pl_queue.append(q);
	current.queue=q;
}
void KuFtpMain::setReConnectLabel()
{
	nameLabel->setText(i18n("ReConnect %1 after %2 seconds").arg(current.host).arg(current.retryElapsed--));
	reConnectTimer->start(1000);
}
void KuFtpMain::SLOT_hideBalloon()
{
	balloon->hide();
}
/*void KuFtpMain::sortListView(KListView *listView,int section,bool ascend)
{
	QListViewItem *top=listView->firstChild();
        listView->takeItem(top);
        listView->setSorting(section,ascend);
        listView->sort();
        listView->setSorting(-1);
        listView->insertItem(top);
}*/
void KuFtpMain::clearDirCache()
{
	for(QValueList<dirCache_t>::iterator t=vl_dirCache.begin();t!=vl_dirCache.end();++t)
		(*t).pl_cacheInfo.clear();
	vl_dirCache.clear();
}
off64_t KuFtpMain::getSizeByCache(QString file)
{
	QString remotePath;
	if(QString(file).contains('/')==1)
		remotePath="/";
	else
		remotePath=QString(file).section('/',0,file.contains('/')-1);
	QString name=QString(file).section('/',-1);
	for(QValueList<dirCache_t>::iterator t=vl_dirCache.begin();t!=vl_dirCache.end();++t)
	{
		if((*t).remotePath==remotePath)
		{
			for(KuFtpCacheInfo *p=(*t).pl_cacheInfo.first();p;p=(*t).pl_cacheInfo.next())
				if(p->getName()==name)
					return p->getSize();
		}
	}
	return 0;
}
int countOfSelectedListViewItems(QListView *listview)
{
	int i=0;
       	QListViewItemIterator it( listview );
	while ( it.current() )
	{
		if ( it.current()->isSelected() )
			i++;
		++it;
	}
	return i;
}

QString formatFileSizeString(off64_t size)
{
	if(size<1024)
		return QString("%1").arg(size);
	else if(size<1024*1024)
		return QString("%1K").arg(size*1.0/1024,0,'f',2);
	else if(size<1024*1024*1024)
		return QString("%1M").arg(size*1.0/1024/1024,0,'f',2);
	else 
		return QString("%1").arg(size*1.0/1024/1024/1024,0,'f',2).append("G");
}

