/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KUFTPMAIN_H
#define KUFTPMAIN_H

#include <qwidget.h>

#include "kuftpqueue.h"
#include "bookmarks/kuftpbookmarks.h"
//#include "kuftp.h"
#include "kuftpthread.h"
#include "kuftpcacheinfo.h"

class QWidget;
class QSplitter;
class QGridLayout;
class QLabel;
class QSignalMapper;
class QAction;
class QListViewItem;
class QListView;
class QStringList;
class QTimer;
template<class type>
class QPtrList;
template<class type>
class QValueList;
class QEvent;

class KLineEdit;
class KListView;
class KTextEdit;
class KToolBar;
class KStatusBar;
class KProgress;
class KPopupMenu;

class KuFtpCore;
class KuFtpSettings;
class KuFtpBalloon;
class KuFtpSiteInfo;

struct session_t 
{
	bool init;
//	KuFtpSiteInfo *site;
	QString host;
	unsigned int port;
	QString username;
	QString password;
	QString remotePath;
	QString localPath;
	int proxyIndex;
	int encryptMethod;
	bool pasv;
	bool correctPasv;
	QString encoding;
	QString descriptiveEncoding;

	KuFtpSiteInfo *lastSite;
//	KuFtpSettings *settings;
	bool useProxy;
	bool connecting;
	bool established;
	bool transfering;
	bool listing;
	bool forcedisconnect;
	int retryTimes;
	int retryInterval;
	int retryElapsed;
	bool retrying;
	bool connectionLost;
	bool noop;
	int noopInterval;
	KuFtpQueue *queue;
	KuFtpQueueItem *queueItem;
	int idle;
	int netwokTimeout;
	bool logging;
	unsigned int failedTransfer;
	off64_t lastXfered;
	off64_t existXfered;
	int elapsedTime;
	int updateStatusBarCount;
	bool transferFromQueue;
	dirCache_t dirCache;
	QString remoteCdPath;
	QString remoteMaybeCdPath;
	bool maybeConnect;
	bool get;
	bool mirror;
	KuFtpSiteInfo *toConnectSite;
	bool stopTransfer;
	bool leftListViewSortAscend;
	bool rightListViewSortAscend;
	int leftListViewSortColumn;
	int rightListViewSortColumn;
	QString filter;
	QString remainTimeString;
	QString speedString;
};
class KuFtpMain:public QWidget
{
	Q_OBJECT
public:
	KuFtpMain(QWidget *parent=0);
	KuFtpCore *core;
        KTextEdit *logWindow;
	session_t current;
	QString cacheFile;
	QStringList queuelist;
	void updateConnectPopupMenu();
	bool askToConfirm(QWidget *parent,QString text,QString caption);
	void stopRetry();
	void clearDirCache();
	void connectTo(KuFtpSiteInfo *site);
	void maybeConnectTo(KuFtpSiteInfo *site);
	void setMaxDownloadSpeed(int i);
	void setMaxUploadSpeed(int i);
//protected:
//	bool eventFilter(QObject *,QEvent *);
private:
	QPtrList<KuFtpBookmarksItem> pl_bmi;
	QPtrList<KuFtpQueue> pl_queue;
	QValueList<log_t> vl_log;
	QValueList<dirCache_t> vl_dirCache;
private:
	QAction *localTransferAct;
	QAction *localQueueAct;
	QAction *localMkdirAct;
	QAction *localRenameAct;
	QAction *localRemoveAct;
	QAction *localRefreshAct;

	QAction *remoteTransferAct;
	QAction *remoteQueueAct;
	QAction *remoteMkdirAct;
	QAction *remoteRenameAct;
	QAction *remoteRmdirAct;
	QAction *remoteDelAct;
	QAction *remoteRefreshAct;
	QAction *addToBookmarksAct;
	
	QAction *queueTransferAct;
	QAction *saveQueueAct;
	QAction *loadQueueAct;
	QAction *clearQueueAct;
	QAction *removeQueueItemsAct;
	QAction *removeFailedQueueItemsAct;
	QAction *resetFailedQueueItemsAct;
	QAction *movetopAct;
	QAction *moveupAct;
	QAction *movedownAct;
	QAction *movebottomAct;

	QAction *showHideFilterAct;
	QAction *sendRawCmdAct;
	QAction *exportDirContentAct;

        QSplitter *hSplitter;
	QSplitter *topVSplitter;
	QSplitter *bottomVSplitter;

        QWidget *leftBox;
        QGridLayout *leftLayout;
	KToolBar *leftToolBar;
        QLabel *leftLabel;
        KLineEdit *leftLineEdit;
        KListView *leftListView;
	QLabel *leftStatusLabel;
	KStatusBar *leftStatusBar;

        QWidget *rightBox;
        QGridLayout *rightLayout;
	KPopupMenu *connectPopupMenu;
	QAction *reConnectAct;
	QAction *quickConnectAct;
	KPopupMenu *bookmarksSubPopupMenu;
	QSignalMapper *bookmarkActMapper;
	QPtrList<bookmarksAct_t> pl_bookmarksAct;
	KToolBar *rightToolBar;
        QLabel *rightLabel;
        KLineEdit *rightLineEdit;
	KPopupMenu *settingsPopupMenu;
	KPopupMenu *charsetSubPopupMenu;

        KListView *rightListView;
	KToolBar *filterToolBar;
	QLabel *rightStatusLabel;
	KStatusBar *rightStatusBar;

	KListView *queueListView;

        QWidget *rightBottomBox;
        QGridLayout *rightBottomLayout;
	KToolBar *rawCmdToolBar;
	QCheckBox *siteCheckBox;

	KStatusBar *bottomStatusBar;
	QLabel *nameLabel;
	KProgress *progressBar;
	QLabel *speedLabel;
	QLabel *elapsedLabel;
	QLabel *remainLabel;
	QTime *transferTime;
	QTimer *logTimer;
	QTimer *statusBarTimer;
	QTimer *idleTimer;
	QTimer *reConnectTimer;
	KuFtpBalloon *balloon;
	void updateLogWindow();
	void setIdle();
	void setReConnectLabel();
	void setQueue(KuFtpSiteInfo *);
	void scanLocalDir(QString name);
	off64_t getSizeByCache(QString file);
private slots:
	void createAction();
	void SLOT_updateLeftListView(const QString &);
	void SLOT_leftListViewClicked(QListViewItem *);
	void SLOT_leftListViewRightClicked(QListViewItem *);
	void SLOT_cdup();
	void SLOT_gohome();
	void SLOT_reload();
	
	void SLOT_updateRightListView(const QString &);
	void SLOT_rightLineEditReturn(const QString &);
	void SLOT_rightListViewClicked(QListViewItem *);
	void SLOT_rightListViewRightClicked(QListViewItem *);
	void SLOT_connectDialog();
	void SLOT_reConnect();
	void SLOT_connectFromBookmarks(int i);
	void SLOT_forceDisconnect();
	void SLOT_rightCancel();
	void SLOT_rightCdup();
	void SLOT_rightReload();
	void SLOT_showHideFilter();
	void SLOT_filterClicked();
	void SLOT_filterRightListView(const QString &);
	void SLOT_charsetChanged(int);

	void SLOT_updateQueueListView();
	void SLOT_queueListViewRightClicked(QListViewItem *);
	void SLOT_queueTransfer();
	void SLOT_saveQueue();
	void saveQueue(QString filename);
	void SLOT_loadQueue();
	void SLOT_queueMovetop();
	void SLOT_queueMoveup();
	void SLOT_queueMovedown();
	void SLOT_queueMovebottom();
	void SLOT_removeQueueItems();
	void SLOT_removeFailedQueueItems();
	void SLOT_resetFailedQueueItems();
	void SLOT_clearQueue();

	void SLOT_setLogQueue(log_t);
	void SLOT_updateLogWindow();
	void SLOT_showHideRawCmdLine();
	void SLOT_exportDirContent();
	void SLOT_sendRawCmd(const QString&);
	void SLOT_updateIdle();
	void SLOT_updateReConnectLabel();
	void SLOT_hideBalloon();
	void SLOT_sortLeftListView(int section);
	void SLOT_sortRightListView(int section);
public slots:
	void SLOT_disconnect();
	void SLOT_addToBookmarks();
	void SLOT_updateBookmarksSubPopupMenu();

private slots:
	void SLOT_put();
	//void SLOT_putTo()
	void SLOT_putQueue();
	void SLOT_mkdir();
	void SLOT_rename();
	void SLOT_remove();
	
	void SLOT_get();
	//void SLOT_getTo();
	void SLOT_getQueue();
	void SLOT_startTransfer();
	void SLOT_remoteMkdir();
	void SLOT_remoteRmdir();
	void SLOT_remoteRename();
	void SLOT_remoteDelete();

	void SLOT_updateCurrentQueue(off64_t);
	void SLOT_processCoreSignals(retData_t,type);
	void SLOT_timeout();
	void SLOT_connectionLost();
	void SLOT_updateStatusBar();
	void afterEstablished();
	void afterQuit();
signals:
	void readyToTransfer();
	void bookmarksChanged();
	void mainStatusChanged();
};
#endif

