/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qstring.h>
#include <qdir.h>
#include <qdom.h>
#include <qfile.h>
#include <qtextstream.h>

#include <klocale.h>
#include <kglobal.h>
#include <kcharsets.h>

#include "kuftpsettings.h"

extern KuFtpSettings kuftpGlobalSettings;

KuFtpSettings::KuFtpSettings()
{
	email="user@kuftp.sf.net";
	defaultDownloadDir=QDir::homeDirPath();
	enableLogFile=false;
	minimizedToSystray=true;
	retryInterval=30;
	retryTimes=6;
	enableNoop=true;
	noopInterval=15;
	enableDownloadLimit=false;
	enableUploadLimit=false;
	maxDownloadSpeed=0;
	maxUploadSpeed=0;
	networkTimeout=60;

	confirmExit=false;
	confirmDeleteFile=false;
	confirmDeleteQueueItem=false;
	confirmClearQueue=false;
	resumeExistFile=true;

	useProxy=false;

	inColor=Qt::blue;
	outColor=Qt::green;
	showSystray=true;
	showHidden=true;
	showBalloonWhenConnectionEstablished=true;
	showBalloonWhenTransferFinished=true;

	defaultEncoding="Unicode ( utf8 )";
}
QString KuFtpSettings::getEncoding()
{
	QString encoding=QString(KGlobal::charsets()->encodingForName(defaultEncoding));
	if(encoding=="iso-10646-ucs-2"||encoding=="ucs2"||encoding=="utf16")
		encoding="utf8";
	return encoding;
}
QString KuFtpSettings::getLocalEncoding()
{
	return QString(KGlobal::locale()->encoding());
}
void KuFtpSettings::readConfig()
{
	QDomDocument doc( "KuFtpConfig" );
	QFile file(QDir::homeDirPath().append("/.kuftp/config.xml"));
	if( !file.open( IO_ReadOnly ) ) 
		return ;
	if( !doc.setContent( &file ) )
	{
		file.close();
		return ;
	}
	file.close();

	int i=0;

	QDomElement root = doc.documentElement();
	if( root.tagName() != "Settings" ) return ;
	QDomNode configNode = root.firstChild();
	while( !configNode.isNull() )
	{
		QDomElement configElement = configNode.toElement();
		if( !configElement.isNull() )
		{
			if( configElement.tagName() == "proxy" )
			{
				QDomNode proxyNode = configElement.firstChild();
				proxy_t *proxy=new proxy_t;
				while( !proxyNode.isNull() )
				{
					QDomElement proxyElement=proxyNode.toElement();
					if(!proxyElement.isNull())
					{
						if(proxyElement.tagName()=="name")
							proxy->name=proxyElement.text();
						else if(proxyElement.tagName()=="type")
						{
							if(proxyElement.text()=="socks4")
								proxy->type=socks4;
							else if(proxyElement.text()=="socks5")
								proxy->type=socks5;
							else
								proxy->type=http;
						}
						else if(proxyElement.tagName() == "host") 
							proxy->host=proxyElement.text();
						else if(proxyElement.tagName() == "port")
						{
							proxy->port=proxyElement.text().toUInt();
							if(proxy->port<0||proxy->port>65535)
								proxy->port=8080;
						}
						else if(proxyElement.tagName() == "username")
							proxy->username=proxyElement.text();
						else if(proxyElement.tagName() == "password")
							proxy->password=proxyElement.text();
						else if(proxyElement.tagName() == "isdefault")
							proxy->isDefault=(proxyElement.text()=="true"?true:false);
					}
					proxyNode=proxyNode.nextSibling();
				}
				proxy->index=i;
				pl_proxy.append(proxy);
				i++;
			}
			else if(configElement.tagName()=="email")
				email=configElement.text();
			else if(configElement.tagName() == "defaultfolder") 
				defaultDownloadDir=configElement.text();
			else if(configElement.tagName() == "enablelogfile")
				enableLogFile=(configElement.text()=="true"?true:false);
			else if(configElement.tagName() == "minimizedtosystray")
				minimizedToSystray=(configElement.text()=="true"?true:false);
			else if(configElement.tagName() == "retryinterval")
				retryInterval=configElement.text().toInt();
			else if(configElement.tagName() == "retrytimes")
				retryTimes=configElement.text().toInt();
			else if(configElement.tagName() == "enablenoop")
				enableNoop=(configElement.text()=="true"?true:false);
			else if(configElement.tagName()=="noopinterval")
				noopInterval=configElement.text().toInt();
			else if(configElement.tagName() == "enabledownloadlimit")
				enableDownloadLimit=(configElement.text()=="true"?true:false);
			else if(configElement.tagName()=="maxdownloadspeed")
				maxDownloadSpeed=configElement.text().toInt();
			else if(configElement.tagName() == "enableuploadlimit")
				enableUploadLimit=(configElement.text()=="true"?true:false);
			else if(configElement.tagName()=="maxuploadspeed")
				maxUploadSpeed=configElement.text().toInt();
			else if(configElement.tagName()=="networktimeout")
				networkTimeout=configElement.text().toInt();
			else if(configElement.tagName() == "confirmexit")
				confirmExit=(configElement.text()=="true"?true:false);
			else if(configElement.tagName() == "confirmdeletefile")
				confirmDeleteFile=(configElement.text()=="true"?true:false);
			else if(configElement.tagName() == "confirmdeletequeueitem")
				confirmDeleteQueueItem=(configElement.text()=="true"?true:false);
			else if(configElement.tagName() == "confirmclearqueue")
				confirmClearQueue=(configElement.text()=="true"?true:false);
			else if(configElement.tagName() == "resumeexistfile")
				resumeExistFile=(configElement.text()=="true"?true:false);
			else if(configElement.tagName()=="defaultencoding")
				defaultEncoding=configElement.text();
			else if(configElement.tagName()=="incolor")
			{
				QString color;
				int r=0;int g=0;int b=0;
				color=configElement.text();
				r=color.section(',',0,0).toInt();
				g=color.section(',',1,1).toInt();
				b=color.section(',',2,2).toInt();
				inColor.setRgb(r,g,b);
			}
			else if(configElement.tagName()=="outcolor")
			{
				QString color;
				int r=0;int g=0;int b=0;
				color=configElement.text();
				r=color.section(',',0,0).toInt();
				g=color.section(',',1,1).toInt();
				b=color.section(',',2,2).toInt();
				outColor.setRgb(r,g,b);
			}
			else if(configElement.tagName()=="showsystray")
				showSystray=(configElement.text()=="true"?1:0);
			else if(configElement.tagName()=="showhidden")
				showHidden=(configElement.text()=="true"?1:0);
			else if(configElement.tagName()=="showballoonwhenconnectionestablished")
				showBalloonWhenConnectionEstablished=(configElement.text()=="true"?1:0);
			else if(configElement.tagName()=="showballoonwhentransferfinished")
				showBalloonWhenTransferFinished=(configElement.text()=="true"?1:0);
		}
		configNode=configNode.nextSibling();
	}
}

void KuFtpSettings::writeConfig()
{
	QDomDocument doc( "KuFtpConfig" );
        QDomElement root= doc.createElement( "Settings" );
        doc.appendChild(root);

        QDomText configText;
        QDomElement configElement;

	configElement=doc.createElement("email");
	root.appendChild( configElement);
	configText=doc.createTextNode(email);
	configElement.appendChild(configText);

	configElement=doc.createElement("defaultfolder");
	root.appendChild(configElement);
	configText=doc.createTextNode(defaultDownloadDir);
	configElement.appendChild(configText);

	configElement=doc.createElement("enablelogfile");
	root.appendChild( configElement);
	configText=doc.createTextNode(enableLogFile?"true":"false");
	configElement.appendChild(configText);

	configElement=doc.createElement("minimizedtosystray");
	root.appendChild( configElement);
	configText=doc.createTextNode(minimizedToSystray?"true":"false");
	configElement.appendChild(configText);
	
	configElement=doc.createElement("retryinterval");
	root.appendChild( configElement);
	char i[10];
	sprintf(i,"%d",retryInterval);
	configText=doc.createTextNode(i);
	configElement.appendChild(configText);
		
	configElement=doc.createElement("retrytimes");
	root.appendChild( configElement);
	sprintf(i,"%d",retryTimes);
	configText=doc.createTextNode(i);
	configElement.appendChild(configText);
		
	configElement=doc.createElement("enablenoop");
	root.appendChild( configElement);
	configText=doc.createTextNode(enableNoop?"true":"false");
	configElement.appendChild(configText);
		
	configElement=doc.createElement("noopinterval");
	root.appendChild( configElement);
	sprintf(i,"%d",noopInterval);
	configText=doc.createTextNode(i);
	configElement.appendChild(configText);

	configElement=doc.createElement("enabledownloadlimit");
	root.appendChild( configElement);
	configText=doc.createTextNode(enableDownloadLimit?"true":"false");
	configElement.appendChild(configText);
		
	configElement=doc.createElement("maxdownloadspeed");
	root.appendChild( configElement);
	sprintf(i,"%d",maxDownloadSpeed);
	configText=doc.createTextNode(i);
	configElement.appendChild(configText);

	configElement=doc.createElement("enableuploadlimit");
	root.appendChild( configElement);
	configText=doc.createTextNode(enableUploadLimit?"true":"false");
	configElement.appendChild(configText);
		
	configElement=doc.createElement("maxuploadspeed");
	root.appendChild( configElement);
	sprintf(i,"%d",maxUploadSpeed);
	configText=doc.createTextNode(i);
	configElement.appendChild(configText);
		
	configElement=doc.createElement("networktimeout");
	root.appendChild( configElement);
	sprintf(i,"%d",networkTimeout);
	configText=doc.createTextNode(i);
	configElement.appendChild(configText);
		
	configElement=doc.createElement("confirmexit");
	root.appendChild( configElement);
	configText=doc.createTextNode(confirmExit?"true":"false");
	configElement.appendChild(configText);
		
	configElement=doc.createElement("confirmdeletefile");
	root.appendChild( configElement);
	configText=doc.createTextNode(confirmDeleteFile?"true":"false");
	configElement.appendChild(configText);
		
	configElement=doc.createElement("confirmdeletequeueitem");
	root.appendChild( configElement);
	configText=doc.createTextNode(confirmDeleteQueueItem?"true":"false");
	configElement.appendChild(configText);
		
	configElement=doc.createElement("confirmclearqueue");
	root.appendChild( configElement);
	configText=doc.createTextNode(confirmClearQueue?"true":"false");
	configElement.appendChild(configText);
		
	configElement=doc.createElement("resumeexistfile");
	root.appendChild( configElement);
	configText=doc.createTextNode(resumeExistFile?"true":"false");
	configElement.appendChild(configText);
		
	configElement=doc.createElement("defaultencoding");
	root.appendChild( configElement);
	configText=doc.createTextNode(defaultEncoding);
	configElement.appendChild(configText);
		
	configElement=doc.createElement("incolor");
	root.appendChild( configElement);
	int r=0;int g=0;int b=0;
	inColor.getRgb(&r,&g,&b);
	configText=doc.createTextNode(QString("%1,%2,%3").arg(r).arg(g).arg(b));
	configElement.appendChild(configText);
		
	configElement=doc.createElement("outcolor");
	root.appendChild( configElement);
	outColor.getRgb(&r,&g,&b);
	configText=doc.createTextNode(QString("%1,%2,%3").arg(r).arg(g).arg(b));
	configElement.appendChild(configText);

	configElement=doc.createElement("showsystray");
	root.appendChild( configElement);
	configText=doc.createTextNode(showSystray?"true":"false");
	configElement.appendChild(configText);

	configElement=doc.createElement("showhidden");
	root.appendChild( configElement);
	configText=doc.createTextNode(showHidden?"true":"false");
	configElement.appendChild(configText);

	configElement=doc.createElement("showballoonwhenconnectionestablished");
	root.appendChild( configElement);
	configText=doc.createTextNode(showBalloonWhenConnectionEstablished?"true":"false");
	configElement.appendChild(configText);

	configElement=doc.createElement("showballoonwhentransferfinished");
	root.appendChild( configElement);
	configText=doc.createTextNode(showBalloonWhenTransferFinished?"true":"false");
	configElement.appendChild(configText);

	for(proxy_t *proxy=pl_proxy.first();proxy;proxy=pl_proxy.next())
	{
		QDomElement proxyElement=doc.createElement("proxy");
		root.appendChild(proxyElement);
		QDomElement proxyInfoElement;
		QDomText proxyInfoText;

		proxyInfoElement=doc.createElement("name");
		proxyElement.appendChild( proxyInfoElement );
		proxyInfoText = doc.createTextNode(proxy->name);
		proxyInfoElement.appendChild(proxyInfoText);

		proxyInfoElement=doc.createElement("type");
		proxyElement.appendChild(proxyInfoElement);
		if(proxy->type==(proxyType_t)1)
			proxyInfoText=doc.createTextNode("http");
		else if(proxy->type==(proxyType_t)2)
			proxyInfoText=doc.createTextNode("socks4");
		else
			proxyInfoText=doc.createTextNode("socks5");
		proxyInfoElement.appendChild(proxyInfoText);

		proxyInfoElement=doc.createElement("host");
		proxyElement.appendChild( proxyInfoElement );
		proxyInfoText = doc.createTextNode(proxy->host);
		proxyInfoElement.appendChild(proxyInfoText);

		proxyInfoElement=doc.createElement("port");
		proxyElement.appendChild( proxyInfoElement);
		proxyInfoText=doc.createTextNode(QString("%1").arg(proxy->port));
		proxyInfoElement.appendChild(proxyInfoText);

		proxyInfoElement=doc.createElement("username");
		proxyElement.appendChild( proxyInfoElement);
		proxyInfoText=doc.createTextNode(proxy->username);
		proxyInfoElement.appendChild(proxyInfoText);

		proxyInfoElement=doc.createElement("password");
		proxyElement.appendChild( proxyInfoElement);
		proxyInfoText=doc.createTextNode(proxy->password);
		proxyInfoElement.appendChild(proxyInfoText);

		proxyInfoElement=doc.createElement("isdefault");
		proxyElement.appendChild(proxyInfoElement);
		proxyInfoText=doc.createTextNode(proxy->isDefault?"true":"false");
		proxyInfoElement.appendChild(proxyInfoText);
	}

	QFile file(QDir::homeDirPath().append("/.kuftp/config.xml"));
        if( !file.open( IO_WriteOnly ) ) 
		return;
        QTextStream stream( &file );
        stream << doc.toString();
        file.close();
}


