/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KUFTPSETTINGS_H
#define KUFTPSETTINGS_H

#include <qcolor.h>

class QString;
template<class type>
class QPtrList;

enum proxyType_t{http=1,socks4,socks5};
struct proxy_t
{
	QString name;
	proxyType_t type;
	QString host;
	int port;
	QString username;
	QString password;
	bool isDefault;
	int index;
};

class KuFtpSettings
{
	public:
		KuFtpSettings();
		void readConfig();
		void writeConfig();

		void setEmail(QString s){email=s;}
		QString getEmail(){return email;}
		void setDefaultDownloadDir(QString s){defaultDownloadDir=s;}
		QString getDefaultDownloadDir(){return defaultDownloadDir;}
		void setEnableLogFile(bool i){enableLogFile=i;}
		bool getIsEnableLogFile(){return enableLogFile;}
		void setMinimizedToSystray(bool i){minimizedToSystray=i;}
		bool getIsMinimizedToSystray(){return minimizedToSystray;}
		void setRetryInterval(int i){retryInterval=i;}
		int getRetryInterval(){return retryInterval;}
		void setRetryTimes(int i){retryTimes=i;}
		int getRetryTimes(){return retryTimes;}
		void setEnableNoop(bool i){enableNoop=i;}
		bool getIsEnableNoop(){return enableNoop;}
		void setNoopInterval(int i){noopInterval=i;}
		int getNoopInterval(){return noopInterval;}
		void setEnableDownloadLimit(bool i){enableDownloadLimit=i;}
		bool getIsEnableDownloadLimit(){return enableDownloadLimit;}
		void setMaxDownloadSpeed(int i){maxDownloadSpeed=i;}
		int getMaxDownloadSpeed(){return maxDownloadSpeed;}
		void setEnableUploadLimit(bool i){enableUploadLimit=i;}
		bool getIsEnableUploadLimit(){return enableUploadLimit;}
		void setMaxUploadSpeed(int i){maxUploadSpeed=i;}
		int getMaxUploadSpeed(){return maxUploadSpeed;}
		void setNetworkTimeout(int i){networkTimeout=i;}
		int getNetworkTimeout(){return networkTimeout;}

		void setConfirmExit(bool i){confirmExit=i;}
		bool getIsConfirmExit(){return confirmExit;}
		void setConfirmDeleteFile(bool i){confirmDeleteFile=i;}
		bool getIsConfirmDeleteFile(){return confirmDeleteFile;}
		void setConfirmDeleteQueueItem(bool i){confirmDeleteQueueItem=i;}
		bool getIsConfirmDeleteQueueItem(){return confirmDeleteQueueItem;}
		void setConfirmClearQueue(bool i){confirmClearQueue=i;}
		bool getIsConfirmClearQueue(){return confirmClearQueue;}
		void setResumeExistFile(bool i){resumeExistFile=i;}
		bool getIsResumeExistFile(){return resumeExistFile;}

		void setUseProxy(bool i){useProxy=i;}
		bool getIsUseProxy(){return useProxy;}

		void setInColor(QColor c){inColor=c;}
		QColor getInColor(){return inColor;}
		void setOutColor(QColor c){outColor=c;}
		QColor getOutColor(){return outColor;}
		void setShowSystray(bool i){showSystray=i;}
		bool getIsShowSystray(){return showSystray;}
		void setShowHidden(bool i){showHidden=i;}
		bool getIsShowHidden(){return showHidden;}
		void setShowBalloonWhenConnectionEstablished(bool i){showBalloonWhenConnectionEstablished=i;}
		bool getIsShowBalloonWhenConnectionEstablished(){return showBalloonWhenConnectionEstablished;}
		void setShowBalloonWhenTransferFinished(bool i){showBalloonWhenTransferFinished=i;}
		bool getIsShowBalloonWhenTransferFinished(){return showBalloonWhenTransferFinished;}
		void setDescriptiveEncoding(QString s){defaultEncoding=s;}
		QString getDescriptiveEncoding(){return defaultEncoding;}
		QString getEncoding();
		QString getLocalEncoding();


		QPtrList<proxy_t> pl_proxy;

	private:
		QString email;
		QString defaultDownloadDir;
		bool enableLogFile;
		bool minimizedToSystray;
		int retryInterval;
		int retryTimes;
		bool enableNoop;
		int noopInterval;
		bool enableDownloadLimit;
		bool enableUploadLimit;
		int maxDownloadSpeed;
		int maxUploadSpeed;
		int networkTimeout;

		bool confirmExit;
		bool confirmDeleteFile;
		bool confirmDeleteQueueItem;
		bool confirmClearQueue;
		bool resumeExistFile;
		QString defaultEncoding;

		bool useProxy;

		QColor inColor;
		QColor outColor;
		bool showSystray;
		bool showHidden;
		bool showBalloonWhenConnectionEstablished;
		bool showBalloonWhenTransferFinished;
};

#endif

