/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qstring.h>
#include <qdatetime.h>
#include <qregexp.h>
#include <qtimer.h>
#include <qapplication.h>
#include <qevent.h>
#include <qdir.h>
#include <qfile.h>
#include <qmemarray.h>
#include <qstringlist.h>
#include <qvaluelist.h>
#include <qcstring.h>
#include <qtextcodec.h>

#include <klocale.h>

#include "kuftpthread.h"
#include "kuftpqueue.h"

extern KuFtpSettings kuftpGlobalSettings;

KuFtpThread::KuFtpThread():QObject(),QThread()
{
	thread=new KuFtpLib();
	thread->SetCallbackArg(this);
	thread->SetCallbackBytes(BYTES_CALLBACK);
	thread->SetCallbackXferFunction(xferCallback);
	thread->SetCallbackLogFunction(logCallback);
//	thread->SetCallbackCertFunction(certCallback);
	timer=new QTimer;
	connect(timer,SIGNAL(timeout()),this,SLOT(SLOT_checkTimeout()));
	encoding=kuftpGlobalSettings.getEncoding().latin1();
}
KuFtpThread::~KuFtpThread()
{
	delete thread;
}
void KuFtpThread::setEventReceiver(QObject *receiver)
{
	eventreceiver=receiver;
}
void KuFtpThread::startCheckTimeout()
{
	timer->start(1000);
	thread->SetCheckTimeout(true);
}
void KuFtpThread::stopCheckTimeout()
{
	timer->stop();
	thread->SetCheckTimeout(false);
}
void KuFtpThread::Connect(const char * host,unsigned short port)
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Connect(host,port);
	eventToCore(FTP_CONNECT,rd);
}
void KuFtpThread::Connect(const char * host,unsigned short port,const char *proxyHost,unsigned short proxyPort,const char *proxyUser,const char *proxyPass,int proxyType)
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Connect(host,port,proxyHost,proxyPort,proxyUser,proxyPass,proxyType);
	eventToCore(FTP_CONNECT,rd);
}
void KuFtpThread::Auth()
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->NegotiateEncryption();
	eventToCore(FTP_AUTH,rd);
}
void KuFtpThread::ConnMode(connmode cmode)
{
	thread->SetConnmode((KuFtpLib::connmode)cmode);
}
void KuFtpThread::CorrectPasv()
{
	thread->SetCorrectPasv(true);
}
void KuFtpThread::Login(const char * user,const char * pass)
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Login(user,pass);
	eventToCore(FTP_LOGIN,rd);
}
void KuFtpThread::Dir()
{
	for(QValueList<dirCache_t>::iterator t=p_vl_dirCache->begin();t!=p_vl_dirCache->end();++t)
        {
                if((*t).remotePath==currentRemotePath) // we have dircache,so we dont really do a ftp Dir
                {
			retData_t *rd=new retData_t;
			rd->success=1;
			eventToCore(FTP_DIR,rd);
			log_t *log=new log_t;
			log->string=QString("[").append(QTime::currentTime().toString("hh:mm:ss")).append(i18n(("] Using cached directory list.")));
			log->type=log_t::common;
			eventToCore(FTP_LOG,log);
			return;
		}
	}
	retData_t *rd=new retData_t;
	rd->success=DirAndCache();
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_DIR,rd);
}
int KuFtpThread::DirAndCache()
{
	QString cacheFile=QString(QDir::homeDirPath().append("/.kuftp/cache.").append(QString::number(rand())));
	thread->SetCheckTimeout(false);
	if(encryptMethod>1)
	{
		if(!thread->SetDataEncryption(KuFtpLib::secure))
			thread->SetDataEncryption(KuFtpLib::unencrypted);
	}
	int success=thread->Dir(cacheFile.latin1(),"");
	if(success)
	{
		QFile file(cacheFile);
		dirCache_t dirCache;
		QRegExp sep("\\s+");
		if(file.open(IO_ReadOnly))
		{
			QTextStream stream( &file );
	        	QTextCodec *codec=QTextCodec::codecForName(encoding);
			stream.setCodec(codec);
			QString line;
			QString permission;
			QString user;
			QString group;
			QString name;
			off64_t size=0;
			off64_t totalSize=0;
			QString date;
			int fileNums=0;
			int dirNums=0;
			dirCache.pl_cacheInfo.clear();
			while ( !stream.atEnd() )
			{
				line = stream.readLine();
				permission=QString(line).section(sep,0,0);
				user=QString(line).section(sep,2,2);
				group=QString(line).section(sep,3,3);
				size=(off64_t)(QString(line).section(sep,4,4).toLong());
				date=QString(line).section(sep,5,7);
				name=QString(line).section(sep,8,QString(line).contains(sep));
				if(line.startsWith("-"))
				{
					fileNums++;
					totalSize+=size;
					KuFtpCacheInfo *t=new KuFtpCacheInfo(permission,user,group,name,size,date,false);
					dirCache.pl_cacheInfo.append(t);
				}
				if(line.startsWith("d"))
				{
					if(name=="."||name=="..")
					{
					       	continue;
					}
					else
					{
						dirNums++;
					}
					KuFtpCacheInfo *t=new KuFtpCacheInfo(permission,user,group,name,-1,date,true);
					dirCache.pl_cacheInfo.append(t);
				}
			}
			dirCache.remotePath=currentRemotePath;
			dirCache.fileNums=fileNums;
			dirCache.dirNums=dirNums;
			dirCache.totalSize=totalSize;
			p_vl_dirCache->append(dirCache);
			file.close();
		}
		file.remove();
	}
	return success;
}
void KuFtpThread::Mirror(QStringList mirrorDirs)
{
	for(QStringList::Iterator it=mirrorDirs.begin();it!=mirrorDirs.end();++it)
	{
		fullRemoteMirrorDir.truncate(0);
		Mirror1(*it);
	}
	retData_t *rd=new retData_t;
	rd->success=1;
	eventToCore(FTP_MIRROR,rd);
}
void KuFtpThread::Mirror1(QString remoteDir)
{
	QTextCodec *codec = QTextCodec::codecForName(encoding);
	QString cacheFile=QString(QDir::homeDirPath().append("/.kuftp/cache.").append(QString::number(rand())));
	QCString remote=codec->fromUnicode(remoteDir);
	Chdir(remote);
	if(ret.success)
	{
		int len=fullRemoteMirrorDir.length();
		if(fullRemoteMirrorDir.isEmpty())
			fullRemoteMirrorDir.append(remoteDir);
		else
			fullRemoteMirrorDir.append("/"+remoteDir);
		Pwd();
		if(DirAndCache())
		{
			if(thread->ConnectionLost())
			{
				eventToCore(FTP_CONNECTIONLOST,NULL);
				return;
			}
//			for(QValueList<dirCache_t>::iterator t=p_vl_dirCache->begin();t!=p_vl_dirCache->end();++t)
//		        {
//              		if((*t).remotePath==currentRemotePath) 
			dirCache_t t=p_vl_dirCache->last(); // 
			for(KuFtpCacheInfo *p=t.pl_cacheInfo.first();p;p=t.pl_cacheInfo.next())
			{
				if(!p->getIsDir())
				{
					QString remote;
					QString local;
					if(currentRemotePath=="/")
						remote=QString("/"+p->getName());
					else
						remote=QString(currentRemotePath)+"/"+p->getName();
					if(QString(currentLocalPath)=="/")
						local=QString("/")+QString(fullRemoteMirrorDir)+"/"+QString(p->getName());
					else
						local=QString(currentLocalPath)+"/"+QString(fullRemoteMirrorDir)+"/"+p->getName();
					off64_t xfered=0;
					if(kuftpGlobalSettings.getIsResumeExistFile())
					{
						if(QFile::exists(local))
						{
							QFileInfo f(local);
							xfered=f.size();
						}
					}
					off64_t size=p->getSize();
					currentQueue->pl_queueItem.append(new KuFtpQueueItem(KuFtpQueueItem::download,remote,local,xfered,size));
				}
				else
				{
					Mirror1(p->getName());
				}
			}
			Cdup();
			fullRemoteMirrorDir.truncate(len);
			Pwd();
		}
	}
}
void KuFtpThread::Systype()
{
	char *buf=(char *)malloc(BUF_MAX);
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->SysType(buf,BUF_MAX);
	free(buf);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_SYSTYPE,rd);
}
void KuFtpThread::Feat()
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Feat();
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_FEAT,rd);
}
void KuFtpThread::Get(const char * localpath,const char * remotepath,transfermode mode,off64_t offset)
{
	thread->SetCheckTimeout(false);
	if(encryptMethod>2)
	{
		if(!thread->SetDataEncryption(KuFtpLib::secure))
			thread->SetDataEncryption(KuFtpLib::unencrypted);
	}
	retData_t *rd=new retData_t;
	rd->success=thread->Get(localpath,remotepath,(KuFtpLib::transfermode)mode,offset);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_GET,rd);
}
void KuFtpThread::Put(const char * localpath,const char * remotepath,transfermode mode,off64_t offset)
{
	thread->SetCheckTimeout(false);
	if(encryptMethod>2)
	{
		if(!thread->SetDataEncryption(KuFtpLib::secure))
			thread->SetDataEncryption(KuFtpLib::unencrypted);
	}
	retData_t *rd=new retData_t;
	rd->success=thread->Put(localpath,remotepath,(KuFtpLib::transfermode)mode,offset);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_PUT,rd);
}
void KuFtpThread::Chdir(const char * remotepath)
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	ret.success=thread->Chdir(remotepath);
	rd->success=ret.success;
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_CHDIR,rd);
}
void KuFtpThread::Cdup()
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Cdup();
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_CDUP,rd);
}
void KuFtpThread::Pwd()
{
	char *buf=(char *)malloc(BUF_MAX);
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Pwd(buf,BUF_MAX);
	QTextCodec *codec=QTextCodec::codecForName(encoding);
	currentRemotePath=codec->toUnicode(QCString(buf));
	rd->string=currentRemotePath;
	free(buf);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_PWD,rd);
}
void KuFtpThread::Mkdir(const char * remotepath)
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Mkdir(remotepath);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_MKDIR,rd);
}
void KuFtpThread::Rmdir(const char * remotepath)
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Rmdir(remotepath);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_RMDIR,rd);
}
void KuFtpThread::Size(const char * remotepath,transfermode mode)
{
	int filesize;
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Size(remotepath,&filesize,(KuFtpLib::transfermode)mode);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_SIZE,rd);
}

void KuFtpThread::Rename(const char * remotesrc,const char * remotedst)
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Rename(remotesrc,remotedst);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_RENAME,rd);
}
void KuFtpThread::Del(const char * remotepath)
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Delete(remotepath);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_DEL,rd);
}
void KuFtpThread::Noop()
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Noop();
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_NOOP,rd);
}
void KuFtpThread::Raw(const char *cmd)
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Raw(cmd);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_RAW,rd);
}
void KuFtpThread::Site(const char *cmd)
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=thread->Site(cmd);
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_SITE,rd);
}
void KuFtpThread::Quit()
{
	thread->SetCheckTimeout(false);
	retData_t *rd=new retData_t;
	rd->success=1;
	thread->Quit();
	if(thread->ConnectionLost())
	{
		eventToCore(FTP_CONNECTIONLOST,NULL);
		return;
	}
	eventToCore(FTP_QUIT,rd);
	terminate();
}
void KuFtpThread::eventToCore(type t,void *ret)
{
	QCustomEvent *event=new QCustomEvent(t);
	event->setData(ret);
	qApp->postEvent(eventreceiver,event);
}

void KuFtpThread::run()
{
	while(!vl_task.empty())
	{
		currentTask=vl_task.front();
		vl_task.pop_front();
		QTextCodec *remoteCodec=QTextCodec::codecForName(encoding);
		QTextCodec *localCodec=QTextCodec::codecForName(kuftpGlobalSettings.getLocalEncoding());
		QCString remote=remoteCodec->fromUnicode(currentTask.remotepath); 
		QCString local=localCodec->fromUnicode(currentTask.localpath);
		QCString cmd=remoteCodec->fromUnicode(currentTask.cmd);
		switch(currentTask.tasktype)
		{
			case FTP_CONNECT:
				Connect(currentTask.host.latin1(),currentTask.port);
				break;
			case FTP_CONNECT_USE_PROXY:
				Connect(currentTask.host.latin1(),currentTask.port,currentTask.proxy->host.latin1(),currentTask.proxy->port,currentTask.proxy->username.latin1(),currentTask.proxy->password.latin1(),(int)currentTask.proxy->type);
				break;
			case FTP_AUTH:
				Auth();
				break;
			case FTP_LOGIN:
				Login(currentTask.user.latin1(),currentTask.pass.latin1());
				break;
			case FTP_SYSTYPE:
				Systype();
				break;
			case FTP_FEAT:
				Feat();
				break;
			case FTP_CHDIR:
				Chdir(remote.data());
				break;
			case FTP_CDUP:
				Cdup();
				break;
			case FTP_DIR:
				Dir();
				break;
			case FTP_MIRROR:
				Mirror(currentTask.mirrorDirs);
				break;
			case FTP_GET:
				Get(local.data(),remote.data(),currentTask.tmode,currentTask.offset);
				break;
			case FTP_PUT:
				Put(local.data(),remote.data(),currentTask.tmode,currentTask.offset);
				break;
			case FTP_PWD:
				Pwd();
				break;
			case FTP_MKDIR:
				Mkdir(remote.data());
				break;
			case FTP_RMDIR:
				Rmdir(remote.data());
				break;
			case FTP_SIZE:
				Size(remote.data(),currentTask.tmode);
				break;
			case FTP_RENAME:
				Rename(local.data(),remote.data());
				break;
			case FTP_DEL:
				Del(remote.data());
				break;
			case FTP_RAW:
				Raw(cmd.data());
				break;
			case FTP_SITE:
				Site(cmd.data());
				break;
			case FTP_NOOP:
				Noop();
				break;
			case FTP_QUIT:
				Quit();
				break;
			case FTP_CONNECTIONLOST:
			case FTP_TIMEOUT:
			case FTP_XFERED:
			case FTP_LOG:
				break;
		}
	}
}

void KuFtpThread::SLOT_checkTimeout()
{
	if(thread->Timeout())
	{
		eventToCore(FTP_TIMEOUT,NULL);
	}
}
void KuFtpThread::setFtpEncoding(QString s)
{
	if(s=="iso-10646-ucs-2"||s=="ucs2"||s=="utf16")
		return;
	encoding=s;
}
void KuFtpThread::SetNetworkTimeout(int seconds)
{
	if(seconds>0)
	{
		thread->SetCallbackIdletime(seconds*1000);
		thread->SetCallbackIdleFunction(idleCallback);
	}
}
void KuFtpThread::setFtpCurrentQueue(KuFtpQueue *q)
{
        currentQueue=q;
}
void KuFtpThread::setFtpCurrentLocalPath(QString s)
{
	currentLocalPath=s;
}
void KuFtpThread::setFtpCurrentRemotePath(QString s)
{
	currentRemotePath=s;
}
void KuFtpThread::setFtpDirCacheValueList(QValueList<dirCache_t> *p_vl_d)
{
        p_vl_dirCache=p_vl_d;
}
void KuFtpThread::SetFtpUseProxy(bool i)
{
	useProxy=i;
	thread->SetFtpUseProxy(i);
}
void KuFtpThread::StopTransfer()
{
	thread->StopTransfer();
}
void KuFtpThread::SetMaxDownloadSpeed(int i)
{
	thread->SetMaxDownloadSpeed(i);
}
void KuFtpThread::SetMaxUploadSpeed(int i)
{
	thread->SetMaxUploadSpeed(i);
}
int KuFtpThread::xferCallback(off64_t i,void *arg)
{
	off64_t *x=new off64_t;
	*x=i;
	(static_cast<KuFtpThread *>(arg))->eventToCore(FTP_XFERED,x);
	return 1;
}
int KuFtpThread::idleCallback(void *)
{
	/*log_t *log=new log_t;
	log->string=QString("[").append(QTime::currentTime().toString("hh:mm:ss")).append(i18n(("] Network Timeout.")));
	log->type=log_t::common;
	(static_cast<KuFtpThread *>(arg))->eventToCore(FTP_LOG,log);
	(static_cast<KuFtpThread *>(arg))->eventToCore(FTP_CONNECTIONLOST,NULL);*/
	return 1;
}
bool KuFtpThread::certCallback(void *,X509 *)
{
	return 1;
}

void KuFtpThread::logCallback(char *log,void *arg,int logType)
{
	QTextCodec *codec=QTextCodec::codecForName((static_cast<KuFtpThread *>(arg))->encoding);
	QString buf=codec->toUnicode(QCString(log));
	if(logType==0)
	{
		log_t *log=new log_t;
		QString s=QString("[").append(QTime::currentTime().toString("hh:mm:ss"));
		if(QString(buf).startsWith("PASS"))
		{
			log->string=QString(s).append("] PASS *");
		}
		else
		{
			log->string=QString(s).append("] %1").arg(QString(buf).section("\r\n",0,0));
		}
		log->type=log_t::in;
		(static_cast<KuFtpThread *>(arg))->eventToCore(FTP_LOG,log);

		if(QString(buf).startsWith("LIST")||QString(buf).startsWith("RETR")||QString(buf).startsWith("STOR"))
		{
			if((static_cast<KuFtpThread *>(arg))->useProxy)
			{
				log_t *log=new log_t;
				log->string=QString(s).append(i18n("] Transfer Data Through Proxy"));
				log->type=log_t::common;
				(static_cast<KuFtpThread *>(arg))->eventToCore(FTP_LOG,log);
			}
			if((static_cast<KuFtpThread *>(arg))->encryptMethod>1)
			{
				log_t *log=new log_t;
				log->string=QString(s).append(i18n("] Data channel secured with SSL"));
				log->type=log_t::common;
				(static_cast<KuFtpThread *>(arg))->eventToCore(FTP_LOG,log);
			}
		}
	}
	else if(logType==1)
	{
		int logNum=buf.contains("\r\n");
		int i=0;
		while(logNum--)
		{
			QString s(buf.section("\r\n",i,i));
			log_t *log=new log_t;
			log->string=QString("[").append(QTime::currentTime().toString("hh:mm:ss")).append("] %1").arg(s);
			log->type=log_t::out;
			(static_cast<KuFtpThread *>(arg))->eventToCore(FTP_LOG,log);
			i++;
		}
	}
	else
	{
		QString s(buf.section("\r\n",0,0));
		log_t *log=new log_t;
		log->string=QString("[").append(QTime::currentTime().toString("hh:mm:ss")).append("] %1").arg(s);
		if(logType==2)
		{
			log->type=log_t::common;
		}
		else
		{
			log->type=log_t::emer;
		}
		(static_cast<KuFtpThread *>(arg))->eventToCore(FTP_LOG,log);
	}
}


