/***************************************************************************
 *   This file is part of the KuFTP project                                *
 *   Copyright (C) 2008 by Jiang JingJing                                  *
 *   jiangjingjing214@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef KUFTPTHREAD_H
#define KUFTPTHREAD_H

#include <qthread.h>
#include <qobject.h>

#include "kuftplib.h"
#define BYTES_CALLBACK 1024
#define BUF_MAX 8192

#include <qstringlist.h>

#include "kuftpcacheinfo.h"
#include "kuftpsettings.h"

class QString;
class QTimer;
template<class type>
class QPtrList;
template<class type>
class QValueList;

class KuFtpQueue;
enum type{FTP_CONNECT=10000,FTP_CONNECT_USE_PROXY,FTP_AUTH,FTP_LOGIN,FTP_DIR,FTP_MIRROR,FTP_SYSTYPE,FTP_FEAT,
	FTP_GET,FTP_PUT,FTP_CHDIR,FTP_CDUP,FTP_PWD,FTP_MKDIR,FTP_RMDIR,FTP_SIZE,FTP_RENAME,FTP_DEL,FTP_QUIT,FTP_NOOP,
	FTP_RAW,FTP_SITE,FTP_CONNECTIONLOST,FTP_TIMEOUT,FTP_LOG,FTP_XFERED};

enum transfermode { ascii = 'A', image = 'I' };
enum connmode { pasv = 1, port };

struct log_t
{
	QString string;
	enum {in=0,out,common,emer} type; // 0:in,1:out,2:common,3:emer
};

struct dirCache_t
{
        QString remotePath;
        QPtrList<KuFtpCacheInfo> pl_cacheInfo;
        int fileNums;
        int dirNums;
        off64_t totalSize;
};

struct task_t
{
	QString host;
	unsigned short port;
	QString user;
	QString pass;
	QString cmd;
	QString localpath;
	QString remotepath;
	QStringList mirrorDirs;
	proxy_t *proxy;
	transfermode tmode;
	connmode cmode;
	off64_t offset;
 	type tasktype;
};
struct retData_t
{
	int success;
	QString string;
};
class KuFtpThread:public QObject,public QThread
{
	Q_OBJECT
	public:
		retData_t ret;
		QTimer *timer;
		QValueList<task_t> vl_task;

		KuFtpThread();
		~KuFtpThread();
		void setEventReceiver(QObject *);
		void startCheckTimeout();
		void stopCheckTimeout();
		void setEncryptMethod(int i){encryptMethod=i;}
		void setFtpEncoding(QString encoding);
		void setFtpCurrentQueue(KuFtpQueue *q);
		void setFtpCurrentLocalPath(QString c);
		void setFtpCurrentRemotePath(QString c);
		void setFtpDirCacheValueList(QValueList<dirCache_t> *l);

		void ConnMode(connmode cmode);
		void CorrectPasv();
		void SetMaxDownloadSpeed(int speed);
		void SetMaxUploadSpeed(int speed);
		void SetNetworkTimeout(int seconds);
		void StopTransfer();
		void SetFtpUseProxy(bool);
	private:
		void run();
		void Connect(const char *host,unsigned short port);
		void Connect(const char *host,unsigned short port,const char *proxyHost,unsigned short proxyPort,const char *proxyUser,const char *proxyPass,int type);
		void Auth();
		void Encrypt();
		void Login(const char *user,const char *pass);
		void Systype();
		void Feat();
		void Dir();
		int  DirAndCache();
		void Mirror(QStringList l);
		void Mirror1(QString l);
		void Get(const char *localpath,const char *remotepath,transfermode tmode,off64_t offset=0);
		void Put(const char *localpath,const char *remotepath,transfermode tmode,off64_t offset=0);
		void Del(const char *remotepath);
		void Chdir(const char *remotepath);
		void Cdup();
		void Pwd();
		void Raw(const char *cmd);
		void Site(const char *cmd);
		void Noop();
		void Mkdir(const char *remotepath);
		void Rmdir(const char *remotepath);
		void Size(const char *remotepath,transfermode tmode);
		void Rename(const char *remotesrc,const char *remotedst);
		void Quit();

		static int xferCallback(off64_t,void *);
		static void logCallback(char *log,void *arg,int out);
		static int idleCallback(void *);
		static bool certCallback(void *arg, X509 *cert);
		void eventToCore(type t,void *);

		KuFtpLib *thread;
		off64_t xfered;
		int encryptMethod;
		bool useProxy;
		QString encoding;
		task_t currentTask;
		QObject *eventreceiver;
		KuFtpQueue *currentQueue;
		QString currentLocalPath;
		QString currentRemotePath;
		QString fullRemoteMirrorDir;
		QValueList<dirCache_t> *p_vl_dirCache;
	public slots:
		void SLOT_checkTimeout();
};


#endif

